#<profile4xi profile = "12" />
# =================================================================================================
#<program4xi program = "apu2program4xi.py">
#<abstract4xi abstract = "APU program management"/>
#<refer4xi reference = "program4xi.py"/>
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
#<verify4xi verification = "10" target = "apu2program4xi5test.py">
# =================================================================================================
# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'APU2Program4Xi',                      # ACK 5.6     p132  

            'add2programwithbinary4apu',           # ACK 5.6.1   p132            
            'add2programwithsource4apu',           # ACK 5.6.1   p133  
                                
            'build2program4apu',                   # ACK 5.6.2   p137      

            'compile2program4apu',                 # ACK 5.6.3   p139            
            'create2programwithsource4apu',        # ACK 5.6.1   p132 
            'create2programwithbinary4apu',        # ACK 5.6.1   p133 
            'create2programwithbuiltinkernels4apu',# NACK 5.6.1  p135   
                       
            'delete2programwithbinary4apu',        # ACK  
            'delete2programwithsource4apu',        # ACK               
            
            'fetch2programbinary4apu',             # ACK
            'fetch2programsource4apu',             # ACK
            
            'get2programinfo4apu',                 # ACK 5.6.7   p150 
            'get2programbuildinfo4apu',            # NACK 5.6.7  p153
            
            'link2program4apu',                    # ACK 5.6.3   p142                          
            'release2program4apu',                 # ACK 5.6.1   p136                                         
            'retain2program4apu',                  # ACK 5.6.1   p136
 
            'set2programwithsource4apu',           # ACK
            'set2programwithbinary4apu',           # ACK            
            'set2programcallback4apu',             # ACK 
                              
            'trace2program4apu',                   # ACK
            'unload2compiler4apu'                  # NACK 5.6.6  p149 
          ] 
#"
#/>


# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
import os
from platform import processor
import sys
import threading                
import _thread   
from decimal import *
#"
#/>


# -------------------------------------------------------------------------------------------------
# Scope4Xi imports                               
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
import version4xi
from version4xi import VersionKB4Xi

# -------------------------------------------------------------------------------------------------
# Buffer                           
# -------------------------------------------------------------------------------------------------
from apu2buffer4xi import get2bufferinfo4apu

# -------------------------------------------------------------------------------------------------
# Command queue module                           
# -------------------------------------------------------------------------------------------------
from apu2commandqueue4xi import APU2CommandQueue4Xi
from apu2commandqueue4xi import create2commandqueue4apu 
from apu2commandqueue4xi import flush2commandqueue4apu
from apu2commandqueue4xi import finish2commandqueue4apu
from apu2commandqueue4xi import get2commandqueueinfo4apu
from apu2commandqueue4xi import release2commandqueue4apu
from apu2commandqueue4xi import retain2commandqueue4apu
from apu2commandqueue4xi import set2commandqueue4apu
from apu2commandqueue4xi import trace2commandqueue4apu

# -------------------------------------------------------------------------------------------------
# Context module                           
# -------------------------------------------------------------------------------------------------
from apu2context4xi import APU2Context4Xi 
from apu2context4xi import create2context4apu
from apu2context4xi import get2context4apu
from apu2context4xi import release2context4apu
from apu2context4xi import retain2context4apu
from apu2context4xi import set2context4apu


# -------------------------------------------------------------------------------------------------
# Device module                        
# -------------------------------------------------------------------------------------------------
from apu2device4xi import APU4Xi
from apu2device4xi import init2apu4xi 
from apu2device4xi import set2apu4xi
from apu2device4xi import get2apu4xi

# -------------------------------------------------------------------------------------------------
# Event module                           
# -------------------------------------------------------------------------------------------------
from apu2event4xi import APU2Event4Xi
from apu2event4xi import create2event4apu
from apu2event4xi import get2eventinfo4apu
from apu2event4xi import release2event4apu
from apu2event4xi import retain2event4apu
from apu2event4xi import set2eventcallback4apu 
from apu2event4xi import set2event4apu
from apu2event4xi import trace2event4apu
from apu2event4xi import wait2event4apu


# -------------------------------------------------------------------------------------------------
# Platform module                           
# -------------------------------------------------------------------------------------------------
from platformids4xi  import get2platformids4xi
from platforminfo4xi import PlatformInfo4Xi                                                                        
from platforminfo4xi import init2platforminfo4xi
from platforminfo4xi import set2platforminfo4xi
from platforminfo4xi import get2platforminfo4xi


#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
import _multiprocessing
#"
#/>


# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from arena4xi import Arena4Xi
from cpu_corecount4xi import cpu_corecount4xi
from dummyprocess4xi import DummyProcess4Xi
from print4xi import print4xi
from pipe4xi import Pipe4Xi
from process4xi import Process4Xi
from queue4xi import Queue4Xi  
from time4xi import time4xi
from reason_object4xi import reason_object4xi
from sense_object4xi import sense_object4xi


#"
#/>

# -------------------------------------------------------------------------------------------------
# Global constants                         
# ------------------------------------------------------------------------------------------------
#<globalize4xi globalization = "
global apu2program4xi                                                   # Program of APU
global apu2build4xi                                                     # Build of APU
global apu2compile4xi                                                   # Compile of APU
global apu2trace4xi                                                     # TRACE

global dir2binary4apu                                                   # Directory for binary files
global dir2source4apu                                                   # Directory for source files

global fetch2code4source                                                # Fetched source code
global fetch2code4binary                                                # Fetched binary code

global import2use4apu                                                   # Imports of target function for Process4Xi

global kb2fetch4apu                                                     # Knowledgebase for fetching

global folder2lib4apu                                                   # Folder of APU library

global profile2fetch4source                                             # Extension for source files
global keys2profile4source                                                
global profile2fetch4binary                                             # Extension for binary files
global keys2profile4binary                                               

global target2function4apu                                              # Target function for Process4Xi
#"
#/>

# -------------------------------------------------------------------------------------------------
# Settings
# -------------------------------------------------------------------------------------------------
#<set4xi setting = "
temp2apu = 'apu'
key2dir = temp2apu[0]                                                                                               # MORE COMPLIATED WAY DUE TO AUTOMATIC SOFTWARE GENERATION
folder2lib4apu = VersionKB4Xi[key2dir] 
dir2binary4apu = folder2lib4apu + '__pycache__//'                                                                   # Default directory is Python cache for binary files - C++ exe files are in //bin
dir2source4apu = folder2lib4apu                                                                                     # Directory for source files

profile2fetch4source = {}
profile2fetch4source ['.py']  = 'ACK'                                                                               # Example extensions: '.txt'= text '.py'=Python, '.cpp'=C++, '.hpp'=headers, '.c'=C
profile2fetch4source ['.cpp'] = 'ACK'
profile2fetch4source ['.c']   = 'ACK'
profile2fetch4source ['.cl']  = 'ACK'
keys2profile4source = profile2fetch4source.keys()

profile2fetch4binary = {}
profile2fetch4binary ['.pyc'] = 'ACK'                                                                              
profile2fetch4binary ['.exe'] = 'ACK' 
keys2profile4binary = profile2fetch4binary.keys()

kb2fetch4apu = {}                                                                                                   # Program to be fetched - always define both source and binary pairs                               
kb2fetch4apu[1] = folder2lib4apu + 'apu2sampler4xi5test.py'                                                         # This will be fetched if no parameters are given to create function
kb2fetch4apu[2] = folder2lib4apu + '__pycache__//apu2image4xi.cpython-33.pyc'
kb2fetch4apu[3] = folder2lib4apu + 'apu2dummy4xi5test.cpp'                                                          # C++ 
kb2fetch4apu[4] = folder2lib4apu + 'bin//apu2dummy4xi5test.exe'



# kb2fetch4apu[1]  = folder2lib4apu + 'apu2sampler4xi5test.py'                                                      
# kb2fetch4apu[2]  = folder2lib4apu + '__pycache__//apu2sampler4xi.cpython-33.pyc'                                  

# kb2fetch4apu[3]  = folder2lib4apu + 'apu2image4xi5test.py'                                                        # Note for an unresolved reason Python compiler does NOT always generate cache file!
# kb2fetch4apu[4]  = folder2lib4apu + '__pycache__//apu2image4xi.cpython-33.pyc'
# kb2fetch4apu[5]  = folder2lib4apu + 'apu3image4xi5test.py'
# kb2fetch4apu[6]  = folder2lib4apu + '__pycache__//apu3image4xi.cpython-33.pyc'
# kb2fetch4apu[7]  = folder2lib4apu + 'apu2commandqueue4xi5test.py'
# kb2fetch4apu[8]  = folder2lib4apu + '__pycache__//apu2commandqueue4xi.cpython-33.pyc'
# kb2fetch4apu[9]  = folder2lib4apu + 'apu2event4xi5test.py'
# kb2fetch4apu[10] = folder2lib4apu + '__pycache__//apu2event4xi.cpython-33.pyc'
# kb2fetch4apu[11]  = folder2lib4apu + 'apu2kernel4xi5test.py'
# kb2fetch4apu[12] = folder2lib4apu + '__pycache__//apu2kernel4xi.cpython-33.pyc'
# kb2fetch4apu[13]  = folder2lib4apu + 'apu2dummy4xi5test.cpp'                                                      # C++ 
# kb2fetch4apu[14] = folder2lib4apu + 'bin//apu2dummy4xi5test.exe'

fetch2code4source = {} 
fetch2code4binary = {} 

#"/>

# -------------------------------------------------------------------------------------------------
# Command queue module of APU imports for target function                           
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importcreate2commandqueue4apu" usage = " importing target function for Process4Xi">
def importcreate2commandqueue4apu():
   global target2function4apu
   from apu2commandqueue4xi import create2commandqueue4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importflush2commandqueue4apu" usage = " importing target function for Process4Xi">
def importflush2commandqueue4apu():
   global target2function4apu
   from apu2commandqueue4xi import flush2commandqueue4apu as target2function4apu
#</function4xi> 

#<function4xi function = "importfinish2commandqueue4apu" usage = " importing target function for Process4Xi">
def importfinish2commandqueue4apu():
   global target2function4apu
   from apu2commandqueue4xi import finish2commandqueue4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importget2commandqueueinfo4apu" usage = " importing target function for Process4Xi">
def importget2commandqueueinfo4apu():
   global target2function4apu
   from apu2commandqueue4xi import get2commandqueueinfo4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importrelease2commandqueue4apu" usage = " importing target function for Process4Xi">
def importrelease2commandqueue4apu():
   global target2function4apu
   from apu2commandqueue4xi import release2commandqueue4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importretain2commandqueue4apu" usage = " importing target function for Process4Xi">
def importretain2commandqueue4apu():
   global target2function4apu
   from apu2commandqueue4xi import retain2commandqueue4apu as target2function4apu
#</function4xi> 

#<function4xi function = "importset2commandqueue4apu" usage = " importing target function for Process4Xi">
def importset2commandqueue4apu():
   global target2function4apu
   from apu2commandqueue4xi import set2commandqueue4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importtrace2commandqueue4apu" usage = " importing target function for Process4Xi">
def importtrace2commandqueue4apu():
   global target2function4apu
   from apu2commandqueue4xi import trace2commandqueue4apu as target2function4apu
#</function4xi> 

# -------------------------------------------------------------------------------------------------
# Context module of APU imports for target function                              
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importcreate2context4apu" usage = " importing target function for Process4Xi">
def importcreate2context4apu():
   global target2function4apu
   from apu2context4xi import create2context4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importget2context4apu" usage = " importing target function for Process4Xi">
def importget2context4apu():
   global target2function4apu
   from apu2context4xi import get2context4apu as target2function4apu
#</function4xi> 

#<function4xi function = "importrelease2context4apu" usage = " importing target function for Process4Xi">
def importrelease2context4apu():
   global target2function4apu
   from apu2context4xi import release2context4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importretain2context4apu" usage = " importing target function for Process4Xi">
def importretain2context4apu():
   global target2function4apu
   from apu2context4xi import retain2context4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importset2context4apu" usage = " importing target function for Process4Xi">
def importset2context4apu():
   global target2function4apu
   from apu2context4xi import set2context4apu as target2function4apu
#</function4xi> 

# -------------------------------------------------------------------------------------------------
# Device module of APU imports for target function                           
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importinit2apu4xi" usage = " importing target function for Process4Xi">
def importinit2apu4xi():
   global target2function4apu
   from apu2device4xi import init2apu4xi as target2function4apu 
#</function4xi> 

#<function4xi function = "importset2apu4xi" usage = " importing target function for Process4Xi">
def importset2apu4xi():
   global target2function4apu
   from apu2device4xi import set2apu4xi   as target2function4apu
#</function4xi>
 
#<function4xi function = "importget2apu4xi" usage = " importing target function for Process4Xi">
def importget2apu4xi():
   global target2function4apu
   from apu2device4xi import get2apu4xi   as target2function4apu
#</function4xi> 

# -------------------------------------------------------------------------------------------------
# Event module of APU imports for target function                              
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importcreate2event4apu" usage = " importing target function for Process4Xi">
def importcreate2event4apu():
   global target2function4apu
   from apu2event4xi import create2event4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importget2eventinfo4apu" usage = " importing target function for Process4Xi">
def importget2eventinfo4apu():
   global target2function4apu
   from apu2event4xi import get2eventinfo4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importrelease2event4apu" usage = " importing target function for Process4Xi">
def importrelease2event4apu():
   global target2function4apu
   from apu2event4xi import release2event4apu as target2function4apu
#</function4xi> 

#<function4xi function = "importretain2event4apu" usage = " importing target function for Process4Xi">
def importretain2event4apu():
   global target2function4apu
   from apu2event4xi import retain2event4apu as target2function4apu
#</function4xi> 

#<function4xi function = "importset2eventcallback4apu" usage = " importing target function for Process4Xi">
def importset2eventcallback4apu():
   global target2function4apu
   from apu2event4xi import set2eventcallback4apu as target2function4apu 
#</function4xi> 

#<function4xi function = "importset2event4apu" usage = " importing target function for Process4Xi">
def importset2event4apu():
   global target2function4apu
   from apu2event4xi import set2event4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importtrace2event4apu" usage = " importing target function for Process4Xi">
def importtrace2event4apu():
   global target2function4apu
   from apu2event4xi import trace2event4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importwait2event4apu" usage = " importing target function for Process4Xi">
def importwait2event4apu():
   global target2function4apu
   from apu2event4xi import wait2event4apu as target2function4apu
#</function4xi> 

# -------------------------------------------------------------------------------------------------
# 2D Image module of APU imports for target function                              
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importcreate2image4apu" usage = " importing target function for Process4Xi">
def importcreate2image4apu():
   global target2function4apu
   from apu2image4xi import create2image4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importget2imageinfo4apu" usage = " importing target function for Process4Xi">
def importget2imageinfo4apu():
   global target2function4apu
   from apu2image4xi import get2imageinfo4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importget2supportedimageformats4apu" usage = " importing target function for Process4Xi">
def importget2supportedimageformats4apu():
   global target2function4apu
   from apu2image4xi import get2supportedimageformats4apu as target2function4apu
#</function4xi> 

#<function4xi function = "importmanoeuvre2copyimage4apu" usage = " importing target function for Process4Xi">
def importmanoeuvre2copyimage4apu():
   global target2function4apu
   from apu2image4xi import manoeuvre2copyimage4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importmanoeuvre2copyimagetobuffer4apu" usage = " importing target function for Process4Xi">
def importmanoeuvre2copyimagetobuffer4apu():
   global target2function4apu
   from apu2image4xi import manoeuvre2copyimagetobuffer4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importmanoeuvre2copybuffertoimage4apu" usage = " importing target function for Process4Xi">
def importmanoeuvre2copybuffertoimage4apu():
   global target2function4apu
   from apu2image4xi import manoeuvre2copybuffertoimage4apu as target2function4apu
#</function4xi> 

#<function4xi function = "importmanoeuvre2readimage4apu" usage = " importing target function for Process4Xi">
def importmanoeuvre2readimage4apu():
   global target2function4apu
   from apu2image4xi import manoeuvre2readimage4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importmanoeuvre2writeimage4apu" usage = " importing target function for Process4Xi">
def importmanoeuvre2writeimage4apu():
   global target2function4apu
   from apu2image4xi import manoeuvre2writeimage4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importmanoeuvre2mapimage4apu" usage = " importing target function for Process4Xi">
def importmanoeuvre2mapimage4apu():
   global target2function4apu
   from apu2image4xi import manoeuvre2mapimage4apu as target2function4apu
#</function4xi> 

#<function4xi function = "importcopy2image4apu" usage = " importing target function for Process4Xi">
def importcopy2image4apu():
   global target2function4apu
   from apu2image4xi import copy2image4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importcopy2imagetobuffer4apu" usage = " importing target function for Process4Xi">
def importcopy2imagetobuffer4apu():
   global target2function4apu
   from apu2image4xi import copy2imagetobuffer4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importcopy2buffertoimage4apu" usage = " importing target function for Process4Xi">
def importcopy2buffertoimage4apu():
   global target2function4apu
   from apu2image4xi import copy2buffertoimage4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importread2image4apu" usage = " importing target function for Process4Xi">
def importread2image4apu():
   global target2function4apu
   from apu2image4xi import read2image4apu as target2function4apu
#</function4xi> 

#<function4xi function = "importmap2image4apu" usage = " importing target function for Process4Xi">
def importmap2image4apu():
   global target2function4apu
   from apu2image4xi import map2image4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importrelease2image4apu" usage = " importing target function for Process4Xi">
def importrelease2image4apu():
   global target2function4apu
   from apu2image4xi import release2image4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importretain2image4apu" usage = " importing target function for Process4Xi">
def importretain2image4apu():
   global target2function4apu
   from apu2image4xi import retain2image4apu as target2function4apu
#</function4xi> 

#<function4xi function = "importset2imagecallback4apu" usage = " importing target function for Process4Xi">
def importset2imagecallback4apu():
   global target2function4apu
   from apu2image4xi import set2imagecallback4apu as target2function4apu 
#</function4xi>
 
#<function4xi function = "importset2image4apu" usage = " importing target function for Process4Xi">
def importset2image4apu():
   global target2function4apu
   from apu2image4xi import set2image4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importtrace2image4apu" usage = " importing target function for Process4Xi">
def importtrace2image4apu():
   global target2function4apu
   from apu2image4xi import trace2image4apu as target2function4apu
#</function4xi> 

#<function4xi function = "importwait2image4apu" usage = " importing target function for Process4Xi">
def importwait2image4apu():
   global target2function4apu
   from apu2image4xi import wait2image4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importwrite2image4apu" usage = " importing target function for Process4Xi">
def importwrite2image4apu():
   global target2function4apu
   from apu2image4xi import write2image4apu as target2function4apu
#</function4xi> 

# -------------------------------------------------------------------------------------------------
# 3D Image module of APU imports for target function                              
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importcreate3image4apu" usage = " importing target function for Process4Xi">
def importcreate3image4apu():
   global target2function4apu
   from apu3image4xi import create3image4apu as target2function4apu
#</function4xi> 

#<function4xi function = "importget3imageinfo4apu" usage = " importing target function for Process4Xi">
def importget3imageinfo4apu():
   global target2function4apu
   from apu3image4xi import get3imageinfo4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importget3supportedimageformats4apu" usage = " importing target function for Process4Xi">
def importget3supportedimageformats4apu():
   global target2function4apu
   from apu3image4xi import get3supportedimageformats4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importmanoeuvre3copyimage4apu" usage = " importing target function for Process4Xi">
def importmanoeuvre3copyimage4apu():
   global target2function4apu
   from apu3image4xi import manoeuvre3copyimage4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importmanoeuvre3copyimagetobuffer4apu" usage = " importing target function for Process4Xi">
def importmanoeuvre3copyimagetobuffer4apu():
   global target2function4apu
   from apu3image4xi import manoeuvre3copyimagetobuffer4apu as target2function4apu
#</function4xi> 

#<function4xi function = "importmanoeuvre3copybuffertoimage4apu" usage = " importing target function for Process4Xi">
def importmanoeuvre3copybuffertoimage4apu():
   global target2function4apu
   from apu3image4xi import manoeuvre3copybuffertoimage4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importmanoeuvre3readimage4apu" usage = " importing target function for Process4Xi">
def importmanoeuvre3readimage4apu():
   global target2function4apu
   from apu3image4xi import manoeuvre3readimage4apu as target2function4apu
#</function4xi> 

#<function4xi function = "importmanoeuvre3writeimage4apu" usage = " importing target function for Process4Xi">
def importmanoeuvre3writeimage4apu():
   global target2function4apu
   from apu3image4xi import manoeuvre3writeimage4apu as target2function4apu
#</function4xi> 

#<function4xi function = "importmanoeuvre3mapimage4apu" usage = " importing target function for Process4Xi">
def importmanoeuvre3mapimage4apu():
   global target2function4apu
   from apu3image4xi import manoeuvre3mapimage4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importcopy3image4apu" usage = " importing target function for Process4Xi">
def importcopy3image4apu():
   global target2function4apu
   from apu3image4xi import copy3image4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importcopy3imagetobuffer4apu" usage = " importing target function for Process4Xi">
def importcopy3imagetobuffer4apu():
   global target2function4apu
   from apu3image4xi import copy3imagetobuffer4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importcopy3buffertoimage4apu" usage = " importing target function for Process4Xi">
def importcopy3buffertoimage4apu():
   global target2function4apu
   from apu3image4xi import copy3buffertoimage4apu as target2function4apu
#</function4xi>

#<function4xi function = "importread3image4apu" usage = " importing target function for Process4Xi">
def importread3image4apu():
   global target2function4apu
   from apu3image4xi import read3image4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importmap3image4apu" usage = " importing target function for Process4Xi">
def importmap3image4apu():
   global target2function4apu
   from apu3image4xi import map3image4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importrelease3image4apu" usage = " importing target function for Process4Xi">
def importrelease3image4apu():
   global target2function4apu
   from apu3image4xi import release3image4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importretain3image4apu" usage = " importing target function for Process4Xi">
def importretain3image4apu():
   global target2function4apu
   from apu3image4xi import retain3image4apu as target2function4apu
#</function4xi> 

#<function4xi function = "importset3imagecallback4apu" usage = " importing target function for Process4Xi">
def importset3imagecallback4apu():
   global target2function4apu
   from apu3image4xi import set3imagecallback4apu as target2function4apu 
#</function4xi> 

#<function4xi function = "importset3image4apu" usage = " importing target function for Process4Xi">
def importset3image4apu():
   global target2function4apu
   from apu3image4xi import set3image4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importtrace3image4apu" usage = " importing target function for Process4Xi">
def importtrace3image4apu():
   global target2function4apu
   from apu3image4xi import trace3image4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importwait3image4apu" usage = " importing target function for Process4Xi">
def importwait3image4apu():
   global target2function4apu
   from apu3image4xi import wait3image4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importwrite3image4apu" usage = " importing target function for Process4Xi">
def importwrite3image4apu():
   global target2function4apu
   from apu3image4xi import write3image4apu as target2function4apu
#</function4xi> 

# -------------------------------------------------------------------------------------------------
# Sampler module of APU imports for target function                              
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importcreate2sampler4apu" usage = " importing target function for Process4Xi">
def importcreate2sampler4apu():
   global target2function4apu
   from apu2sampler4xi import create2sampler4apu as target2function4apu
#</function4xi> 

#<function4xi function = "importget2samplerinfo4apu" usage = " importing target function for Process4Xi">
def importget2samplerinfo4apu():
   global target2function4apu
   from apu2sampler4xi import get2samplerinfo4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importrelease2sampler4apu" usage = " importing target function for Process4Xi">
def importrelease2sampler4apu():
   global target2function4apu
   from apu2sampler4xi import release2sampler4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importretain2sampler4apu" usage = " importing target function for Process4Xi">
def importretain2sampler4apu():
   global target2function4apu
   from apu2sampler4xi import retain2sampler4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importset2samplercallback4apu" usage = " importing target function for Process4Xi">
def importset2samplercallback4apu():
   global target2function4apu
   from apu2sampler4xi import set2samplercallback4apu as target2function4apu 
#</function4xi>
 
#<function4xi function = "importset2sampler4apu" usage = " importing target function for Process4Xi">
def importset2sampler4apu():
   global target2function4apu
   from apu2sampler4xi import set2sampler4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importtrace2sampler4apu" usage = " importing target function for Process4Xi">
def importtrace2sampler4apu():
   global target2function4apu
   from apu2sampler4xi import trace2sampler4apu as target2function4apu
#</function4xi> 

# -------------------------------------------------------------------------------------------------
# Kernel module of APU imports for target function                              
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importadd2kernel4apu2kernel4apu" usage = " importing target function for Process4Xi">
def importadd2kernel4apu2kernel4apu():
   global target2function4apu
   from apu2kernel4xi import add2kernel4apu2kernel4apu as target2function4apu
#</function4xi>

#<function4xi function = "importexecute2kernel4apu2kernel4apu" usage = " importing target function for Process4Xi">
def importexecute2kernel4apu2kernel4apu():
   global target2function4apu
   from apu2kernel4xi import execute2kernel4apu2kernel4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importexecute2task4apu2kernel4apu" usage = " importing target function for Process4Xi">
def importexecute2task4apu2kernel4apu():
   global target2function4apu
   from apu2kernel4xi import execute2task4apu2kernel4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importexecute2nativekernel4apu2kernel4apu" usage = " importing target function for Process4Xi">
def importexecute2nativekernel4apu2kernel4apu():
   global target2function4apu
   from apu2kernel4xi import execute2nativekernel4apu2kernel4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importcreate2kernel4apu2kernel4apu" usage = " importing target function for Process4Xi">
def importcreate2kernel4apu2kernel4apu():
   global target2function4apu
   from apu2kernel4xi import create2kernel4apu2kernel4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importcreate2kernelsinprogram4apu2kernel4apu" usage = " importing target function for Process4Xi">
def importcreate2kernelsinprogram4apu2kernel4apu():
   global target2function4apu
   from apu2kernel4xi import create2kernelsinprogram4apu2kernel4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importdelete2kernel4apu2kernel4apu" usage = " importing target function for Process4Xi">
def importdelete2kernel4apu2kernel4apu():
   global target2function4apu
   from apu2kernel4xi import delete2kernel4apu2kernel4apu as target2function4apu
#</function4xi> 

#<function4xi function = "importfetch2kernel4apu2kernel4apu" usage = " importing target function for Process4Xi">
def importfetch2kernel4apu2kernel4apu():
   global target2function4apu
   from apu2kernel4xi import fetch2kernel4apu2kernel4apu as target2function4apu
#</function4xi> 

#<function4xi function = "importget2kernelinfo4apu2kernel4apu" usage = " importing target function for Process4Xi">
def importget2kernelinfo4apu2kernel4apu():
   global target2function4apu
   from apu2kernel4xi import get2kernelinfo4apu2kernel4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importget2kernelworkgroupinfo4apu2kernel4apu" usage = " importing target function for Process4Xi">
def importget2kernelworkgroupinfo4apu2kernel4apu():
   global target2function4apu
   from apu2kernel4xi import get2kernelworkgroupinfo4apu2kernel4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importget2kernelarginfo4apu2kernel4apu" usage = " importing target function for Process4Xi">
def importget2kernelarginfo4apu2kernel4apu():
   global target2function4apu
   from apu2kernel4xi import get2kernelarginfo4apu2kernel4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importrelease2kernel4apu2kernel4apu" usage = " importing target function for Process4Xi">
def importrelease2kernel4apu2kernel4apu():
   global target2function4apu
   from apu2kernel4xi import release2kernel4apu2kernel4apu as target2function4apu
#</function4xi> 

#<function4xi function = "importretain2kernel4apu2kernel4apu" usage = " importing target function for Process4Xi">
def importretain2kernel4apu2kernel4apu():
   global target2function4apu
   from apu2kernel4xi import retain2kernel4apu2kernel4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importset2kernel4apu2kernel4apu" usage = " importing target function for Process4Xi">
def importset2kernel4apu2kernel4apu():
   global target2function4apu
   from apu2kernel4xi import set2kernel4apu2kernel4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importset2kernelcallback4apu2kernel4apu" usage = " importing target function for Process4Xi">
def importset2kernelcallback4apu2kernel4apu():
   global target2function4apu
   from apu2kernel4xi import set2kernelcallback4apu2kernel4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importset2kernelarg4apu2kernel4apu" usage = " importing target function for Process4Xi">
def importset2kernelarg4apu2kernel4apu():
   global target2function4apu
   from apu2kernel4xi import set2kernelarg4apu2kernel4apu as target2function4apu
#</function4xi>
 
#<function4xi function = "importtrace2kernel4apu2kernel4apu" usage = " importing target function for Process4Xi">
def importtrace2kernel4apu2kernel4apu():
   global target2function4apu
   from apu2kernel4xi import trace2kernel4apu2kernel4apu as target2function4apu
#</function4xi> 
 
# -------------------------------------------------------------------------------------------------
# Test modules of APU imports for target function                             
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importapu2commandqueue4xi5test" usage = " importing target function for Process4Xi">
def importapu2commandqueue4xi5test():
   global target2function4apu
   from apu2commandqueue4xi5test import apu2commandqueue4xi5test as target2function4apu
#</function4xi>
 
#<function4xi function = "importapu2event4xi5test" usage = " importing target function for Process4Xi">
def importapu2event4xi5test():
   global target2function4apu
   from apu2event4xi5test import apu2event4xi5test as target2function4apu
#</function4xi>
 
#<function4xi function = "importapu2image4xi5test" usage = " importing target function for Process4Xi">
def importapu2image4xi5test():
   global target2function4apu
   from apu2image4xi5test import apu2image4xi5test as target2function4apu
#</function4xi> 

#<function4xi function = "importapu3image4xi5test" usage = " importing target function for Process4Xi">
def importapu3image4xi5test():
   global target2function4apu
   from apu3image4xi5test import apu3image4xi5test as target2function4apu
#</function4xi>
 
#<function4xi function = "importapu2sampler4xi5test" usage = " importing target function for Process4Xi">
def importapu2sampler4xi5test():
   global target2function4apu
   from apu2sampler4xi5test import apu2sampler4xi5test as target2function4apu
#</function4xi> 

#<function4xi function = "importapu2kernel4xi5test" usage = " importing target function for Process4Xi">
def importapu2kernel4xi5test():
   global target2function4apu
   from apu2kernel4xi5test import apu2kernel4xi5test as target2function4apu
#</function4xi> 

# -------------------------------------------------------------------------------------------------
# Error handler of APU imports for target function                             
# -------------------------------------------------------------------------------------------------
#<function4xi function = "handle2error4apu" usage = " Error handling for importing target function for Process4Xi">
def handle2error4apu():
   global import2use4apu
   print ("HANDLE2ERROR4APU OF APU2PROGRAM4XI.PY: *** FAILED TO IMPORT TARGET FUNCTION ***")
   keys2import4apu = import2use4apu.keys()
   for key2import4apu in keys2import4apu:
       print(key2import4apu,import2use4apu[key2import4apu]) 
#</function4xi> 


import2use4apu = {}
# -------------------------------------------------------------------------------------------------
# Command queue module of APU imports for target function                           
# -------------------------------------------------------------------------------------------------
import2use4apu["create2commandqueue4apu"] = importcreate2commandqueue4apu        
import2use4apu["flush2commandqueue4apu"] = importflush2commandqueue4apu 
import2use4apu["finish2commandqueue4apu"] = importfinish2commandqueue4apu 
import2use4apu["get2commandqueueinfo4apu"] = importget2commandqueueinfo4apu 
import2use4apu["release2commandqueue4apu"] = importrelease2commandqueue4apu 
import2use4apu["retain2commandqueue4apu"] = importretain2commandqueue4apu 
import2use4apu["set2commandqueue4apu"] = importset2commandqueue4apu 
import2use4apu["trace2commandqueue4apu"] = importtrace2commandqueue4apu 

# -------------------------------------------------------------------------------------------------
# Context module of APU imports for target function                              
# -------------------------------------------------------------------------------------------------
import2use4apu["create2context4apu"] = importcreate2context4apu 
import2use4apu["get2context4apu"] = importget2context4apu 
import2use4apu["release2context4apu"] = importrelease2context4apu 
import2use4apu["retain2context4apu"] = importretain2context4apu 
import2use4apu["set2context4apu"] = importset2context4apu 

# -------------------------------------------------------------------------------------------------
# Device module of APU imports for target function                           
# -------------------------------------------------------------------------------------------------
import2use4apu["init2apu4xi"] = importinit2apu4xi 
import2use4apu["set2apu4xi"] = importset2apu4xi 
import2use4apu["get2apu4xi"] = importget2apu4xi 

# -------------------------------------------------------------------------------------------------
# Event module of APU imports for target function                              
# -------------------------------------------------------------------------------------------------
import2use4apu["create2event4apu"] = importcreate2event4apu 
import2use4apu["get2eventinfo4apu"] = importget2eventinfo4apu 
import2use4apu["release2event4apu"] = importrelease2event4apu 
import2use4apu["retain2event4apu"] = importretain2event4apu 
import2use4apu["set2eventcallback4apu"] = importset2eventcallback4apu 
import2use4apu["set2event4apu"] = importset2event4apu 
import2use4apu["trace2event4apu"] = importtrace2event4apu 
import2use4apu["wait2event4apu"] = importwait2event4apu 

# -------------------------------------------------------------------------------------------------
# 2D Image module of APU imports for target function                              
# -------------------------------------------------------------------------------------------------
import2use4apu["create2image4apu"] = importcreate2image4apu 
import2use4apu["get2imageinfo4apu"] = importget2imageinfo4apu 
import2use4apu["get2supportedimageformats4apu"] = importget2supportedimageformats4apu 
import2use4apu["manoeuvre2copyimage4apu"] = importmanoeuvre2copyimage4apu 
import2use4apu["manoeuvre2copyimagetobuffer4apu"] = importmanoeuvre2copyimagetobuffer4apu 
import2use4apu["manoeuvre2copybuffertoimage4apu"] = importmanoeuvre2copybuffertoimage4apu 
import2use4apu["manoeuvre2readimage4apu"] = importmanoeuvre2readimage4apu 
import2use4apu["manoeuvre2writeimage4apu"] = importmanoeuvre2writeimage4apu 
import2use4apu["manoeuvre2mapimage4apu"] = importmanoeuvre2mapimage4apu 
import2use4apu["copy2image4apu"] = importcopy2image4apu 
import2use4apu["copy2imagetobuffer4apu"] = importcopy2imagetobuffer4apu 
import2use4apu["copy2buffertoimage4apu"] = importcopy2buffertoimage4apu 
import2use4apu["read2image4apu"] = importread2image4apu 
import2use4apu["map2image4apu"] = importmap2image4apu 
import2use4apu["release2image4apu"] = importrelease2image4apu 
import2use4apu["retain2image4apu"] = importretain2image4apu 
import2use4apu["set2imagecallback4apu"] = importset2imagecallback4apu 
import2use4apu["set2image4apu"] = importset2image4apu 
import2use4apu["trace2image4apu"] = importtrace2image4apu 
import2use4apu["wait2image4apu"] = importwait2image4apu 
import2use4apu["write2image4apu"] = importwrite2image4apu 

# -------------------------------------------------------------------------------------------------
# 3D Image module of APU imports for target function                              
# -------------------------------------------------------------------------------------------------
import2use4apu["create3image4apu"] = importcreate3image4apu 
import2use4apu["get3imageinfo4apu"] = importget3imageinfo4apu 
import2use4apu["get3supportedimageformats4apu"] = importget3supportedimageformats4apu 
import2use4apu["manoeuvre3copyimage4apu"] = importmanoeuvre3copyimage4apu 
import2use4apu["manoeuvre3copyimagetobuffer4apu"] = importmanoeuvre3copyimagetobuffer4apu 
import2use4apu["manoeuvre3copybuffertoimage4apu"] = importmanoeuvre3copybuffertoimage4apu 
import2use4apu["manoeuvre3readimage4apu"] = importmanoeuvre3readimage4apu 
import2use4apu["manoeuvre3writeimage4apu"] = importmanoeuvre3writeimage4apu 
import2use4apu["manoeuvre3mapimage4apu"] = importmanoeuvre3mapimage4apu 
import2use4apu["copy3image4apu"] = importcopy3image4apu 
import2use4apu["copy3imagetobuffer4apu"] = importcopy3imagetobuffer4apu 
import2use4apu["copy3buffertoimage4apu"] = importcopy3buffertoimage4apu 
import2use4apu["read3image4apu"] = importread3image4apu 
import2use4apu["map3image4apu"] = importmap3image4apu 
import2use4apu["release3image4apu"] = importrelease3image4apu 
import2use4apu["retain3image4apu"] = importretain3image4apu 
import2use4apu["set3imagecallback4apu"] = importset3imagecallback4apu 
import2use4apu["set3image4apu"] = importset3image4apu 
import2use4apu["trace3image4apu"] = importtrace3image4apu 
import2use4apu["wait3image4apu"] = importwait3image4apu 
import2use4apu["write3image4apu"] = importwrite3image4apu 

# -------------------------------------------------------------------------------------------------
# Sampler module of APU imports for target function                              
# -------------------------------------------------------------------------------------------------
import2use4apu["create2sampler4apu"] = importcreate2sampler4apu 
import2use4apu["get2samplerinfo4apu"] = importget2samplerinfo4apu 
import2use4apu["release2sampler4apu"] = importrelease2sampler4apu 
import2use4apu["retain2sampler4apu"] = importretain2sampler4apu 
import2use4apu["set2samplercallback4apu"] = importset2samplercallback4apu 
import2use4apu["set2sampler4apu"] = importset2sampler4apu 
import2use4apu["trace2sampler4apu"] = importtrace2sampler4apu 


# -------------------------------------------------------------------------------------------------
# Test modules of APU imports for target function                             
# -------------------------------------------------------------------------------------------------
import2use4apu["apu2commandqueue4xi5test"] = importapu2commandqueue4xi5test 
import2use4apu["apu2event4xi5test"] = importapu2event4xi5test 
import2use4apu["apu2image4xi5test"] = importapu2image4xi5test 
import2use4apu["apu3image4xi5test"] = importapu3image4xi5test 
import2use4apu["apu2kernel4xi5test"] = importapu2kernel4xi5test
import2use4apu["apu2sampler4xi5test"] = importapu2sampler4xi5test 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "trace2program4apu" usage = "
#
#   print('=' *33)          
#   print('(Step 15A/25) OF APU2PROGRAM4XI5TEST - TRYING TO TRACE APU: ', time4xi())      
#   print('=' *33)      
#   trace2apu = None
#   try:
#       trace2apu = trace2program4apu() 
#       print('(Step 15B/25) OF APU2PROGRAM4XI5TEST - TRACE2APU: ', trace2apu)  
#       # {'MESSAGE': '{(Step 2Z0/33) failed; trace2program4apu-failed {}; time: 04:32:25 11/07/11 Pacific Standard Time}', 'CLASS': 'trace2program4apu', 'METHOD': 'trace2program4apu'} 
#   except:
#       print('(Step 15Z1/25) OF APU2PROGRAM4XI5TEST *** FAILED TO TRACE APU: ', trace2apu) 
#      
#">
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------   
def trace2program4apu(task4trace2apu=None):  

    '''
    creates callback to apu2sourceprogram4xi
    '''
    
    global apu2trace4xi


    # -----------------------------------------------------------------------------------------
    # (Step 1/33)         
    # -----------------------------------------------------------------------------------------          
    class2print = "trace2program4apu" 
    method2print = "trace2program4apu"

    message2print = "{(Step 1A/33) started" + "; trace2program4apu-started: " + str(trace2program4apu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                               
    # <function trace2program4apu at 0x0000000003A202C8> 
  
             
    message2print = "{(Step 1B/33) passed" + "; task4trace2apu-to-trace2program4apu: " + str(task4trace2apu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              # 
    # 

    code2ack  = ' '
    code2nack = ' '    
    
    apu2trace4xi = {}                                                                                               # INITIALIZES TO EMPTY DICTIONARY
      
    if task4trace2apu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY DRAGONIZED CALLBACK !
        # -----------------------------------------------------------------------------------------    
        task4trace2apu = {}                                                                                            
            
        task4trace2apu['CLASS']   = "trace2program4apu"                                                               # DRAGONIZE ZPU TRACE!
        task4trace2apu['METHOD']  = "trace2program4apu" 
        task4trace2apu['MESSAGE'] = "{(Step 2Z0/33) failed" + "; trace2program4apu-failed " + str(apu2trace4xi)  + "; time: " + time4xi() + "}"               
  
        
        message2print = "{(Step 2A/33) set" + "; task4trace2apu-set-by-trace2program4apu to: " + str(task4trace2apu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
    
        code2ack = code2ack + '2A-'            


    # -----------------------------------------------------------------------------------------
    # PARAMETER GIVEN BY DRAGONIZED CALLBACK !
    # -----------------------------------------------------------------------------------------          
    message2print = "{(Step 3A/33) try" + "; task4trace2apu-try-by-trace2program4apu: " + str(task4trace2apu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)    
    # 

    try:
        apu2trace4xi ['CLASS']  = task4trace2apu['CLASS']                                                      # DRAGONIZE APU TRACE!
        apu2trace4xi ['METHOD']  = task4trace2apu['METHOD']
        apu2trace4xi ['MESSAGE'] = task4trace2apu['MESSAGE']
        
        message2print = "{(Step 3B/33) set" + "; apu2trace4xi-set-by-trace2program4apu to: " + str(apu2trace4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 

        code2ack = code2ack + '3B-'         
    except:
        message2print = "{(Step 3Z1/33) failed" + "; apu-trace-failed-by-trace2program4apu: " + str(task4trace2apu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z1-' 
        
    try:
        # =================================
        # SIMULATE APU TRACE
        # =================================
        class2print   = task4trace2apu['CLASS']                                                                # DRAGONIZE APU TRACE!
        method2print  = task4trace2apu['METHOD']
        
        message2print = task4trace2apu['MESSAGE']
        print4xi(class2print, method2print, message2print)                                                     # DRAGONIZED TRACE FOR OPENCL CALLBACK!
        # 
        
        
        code2ack = code2ack + '3C-'         
    except:
        class2print = "trace2program4apu"                                                                        # SET TRACE BACK TO THIS FUNCTION!
        method2print = "trace2program4apu"   
        message2print = "{(Step 3Z2/33) failed" + "; simulation-of-apu-trace-failed-by-trace2program4apu: " + str(task4trace2apu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z2-' 
        
    try:
        # =================================
        # NORMAL TRACE
        # =================================                     
        class2print = "trace2program4apu"                                                                        # SET TRACE BACK TO THIS FUNCTION!
        method2print = "trace2program4apu"                
        message2print = "{(Step 3D/33) performed" + "; apu-trace-performed-by-trace2program4apu: " + str(task4trace2apu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        code2ack = code2ack + '3D-' 
    except:
        message2print = "{(Step 3Z3/33) failed" + "; apu-trace-failed-by-trace2program4apu: " + str(task4trace2apu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z3-' 

    # -----------------------------------------------------------------------------------------
    # (Step 33/3)           
    # -----------------------------------------------------------------------------------------
    message2print = "{(Step 33/33) returning" + "; apu2trace4xi-returning-by-trace2program4apu: " + str(apu2trace4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # {'MESSAGE': '{(Step 2Z0/33) failed; trace2program4apu-failed {}; time: 04:40:23 11/16/11 Pacific Standard Time}', 
    #  'CLASS': 'trace2program4apu', 
    #  'METHOD': 'trace2program4apu'}; time: 04:40:23 11/16/11 Pacific Standard Time}

    
    return apu2trace4xi
            
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "unload2compiler4apu" usage = "
#
#   response4unload2compiler = unload2compiler4apu()
#      
#">
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------   
def unload2compiler4apu(task4unload2apu=None):  

    '''
    unloads compiler
    '''


            
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)         
    # -----------------------------------------------------------------------------------------          
    class2print = "unload2compiler4apu" 
    method2print = "unload2compiler4apu"

    message2print = "{(Step 1A/33) started" + "; unload2compiler4apu-started: " + str(unload2compiler4apu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                              
    # 
  
          
    message2print = "{(Step 1B/33) passed" + "; task4unload2apu-to-unload2compiler4apu: " + str(task4unload2apu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                            # 
    # 
    status4unload2apu = 'CL_SUCCESS'
    
    code2ack  = ' '
    code2nack = ' '    
    
                                                                                                                  #       
    if task4unload2apu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN - BUILD THEM
        # -----------------------------------------------------------------------------------------    
        task4unload2apu = {}                                                                                            

        message2print = "{(Step 2A/33) set" + "; task4unload2apu-set-by-unload2compiler4apu to: " + str(task4unload2apu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
    
        code2ack = code2ack + '2A-'            


    # -----------------------------------------------------------------------------------------
    # 
    # -----------------------------------------------------------------------------------------          
    status4unload2apu = 'CL_SUCCESS'
    message2print = "{(Step 3A/33) set" + "; status4unload2apu-set-by-unload2compiler4apu to: " + str(status4unload2apu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 
    

    # -----------------------------------------------------------------------------------------
    # (Step 33/3)           
    # -----------------------------------------------------------------------------------------
    message2print = "{(Step 33/33) returning" + "; status4unload2apu-returning-by-unload2compiler4apu: " + str(status4unload2apu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    
    return status4unload2apu
            
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "set2programcallback4apu" usage = "
#
#   print('(Step 14A/700) OF APU2PROGRAM4XI5TEST - TRYING TO SET PROGRAM CALLBACK OF APU ', time4xi())  
#   try:                               
#       response4setcallback2program = set2programcallback4apu()                                                                                                 
#       print('(Step 14B/700) OF APU2PROGRAM4XI5TEST - SET PROGRAM CALLBACK OF APU TO: ', response4setcallback2program)   
#       # 
#   except:
#       print('(Step 14Z1/700) OF APU2PROGRAM4XI5TEST *** FAILED TO SET PROGRAM CALLBACK OF APU ', time4xi())  
# 
#">
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
# ------------------------------------------------------------------------------------------------- 
 
def set2programcallback4apu(task4set2programwithsourcecallback=None):

    '''
    registers callback function to apu2program4xi
    '''
    global apu2program4xi

            
    class2print = "set2programcallback4apu" 
    method2print = "set2programcallback4apu"
    
    message2print = "{(Step 1A/70) started" + "; set2programcallback4apu-started: " + str(set2programcallback4apu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # set2programcallback4apu-started
    
    try:
    
        apu2program4xi ['CALLBACK'] = trace2program4apu
        
        
        message2print = "{(Step 1C/70) returning" + "; callback-of-apu2program4xi-returning-by-set2programcallback4apu: " + str(apu2program4xi ['CALLBACK']) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print) 
        # <function trace2program4apu at 0x0000000003A985C8>
        
        return apu2program4xi ['CALLBACK']      
        
    except:
        message2print = "{(Step 1Z1/70) failed" + "; callback2program-failed-by-set2programcallback4apu: " + str(callback2program) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         

        message2print = "{(Step 1Z2/70) returning" + "; trace2program4apu-returning-by-set2programcallback4apu: " + str(trace2program4apu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)      
            
        return trace2program4apu
            
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "build2program4apu" usage = "
#
#   task4build2program  = {}
#   task4build2program  ['PROGRAM'] = folder2lib4apu + 'apu2sampler4xi.py'
#   task4build2program  ['OPTIONS'] = {}
#   task4build2program  ['OPTIONS'] ['MATH']          = 'SINGLE-PRECISION-CONSTANT'
#   task4build2program  ['OPTIONS'] ['OPTIMIZATION']  = 'DISABLE'
#   task4build2program  ['OPTIONS'] ['VERSION']       = 'CL1.1'
#   task4build2program  ['OPTIONS'] ['WARNINGS']      = 'INHIBIT'
#   print('(Step 99A/700) OF APU2PROGRAM4XI5TEST - TRYING TO BUILD PROGRAM OF APU DEVICE ', time4xi())    
#   # 
#   try:                               
#       response4build2program = build2program4apu(task4build2program)
#       print('(Step 99B/700) OF APU2PROGRAM4XI5TEST - BUILT PROGRAM OF APU DEVICE ', time4xi())
#   except:
#       print('(Step 99Z1/700) OF APU2PROGRAM4XI5TEST *** FAILED TO BUILD PROGRAM OF APU DEVICE ', time4xi())  
#   try:                                                                                                                 
#       keys2response = response4build2program.keys()
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4build2program[key2response])
#           # ACK2RETURN  :   11A1-12A1-
#           # NACK2RETURN  :   13Z1-14Z1-
#           # OPTIONS  :  {'VERSION': 'CL1.1', 'OPTIMIZATION': 'DISABLE', 'MATH': 'SINGLE-PRECISION-CONSTANT', 'WARNINGS': 'INHIBIT'}
#           # PROGRAM  :  folder2lib4apu + 'apu2sampler4xi.py'
#       
#   except:
#       print('(Step 99Z2/700) OF APU2PROGRAM4XI5TEST *** FAILED TO PRINT RESPONSE4BUILD2PROGRAM ', response4build2program) 
#                 
#">
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------   
def build2program4apu(task4build2apu=None):

    '''
    building program to APU 
    '''
    global apu2build4xi

    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "build2program4apu"
    method2print = "build2program4apu" 

    message2print = "{(Step 1A/33) started" + "; build2program4apu-started: " + str(build2program4apu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # build2program4apu-started
    
    
    message2print = "{(Step 1B/33) passed" + "; task4build2apu-to-build2program4apu: " + str(task4build2apu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'PROGRAM': '...//apu2sampler4xi.py', 
    #  'OPTIONS': {'VERSION': 'CL1.1', 
    #  'OPTIMIZATION': 'DISABLE', 
    #  'MATH': 'SINGLE-PRECISION-CONSTANT', 
    #  'WARNINGS': 'INHIBIT'}}
    # ; time: 07:57:00 11/18/11 Pacific Standard Time}

             
    code2ack  = ' '
    code2nack = ' '

    message2print = "{(Step 1A/70) try" + "; build-try-by-build2program4apu: None"  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    
    try:             
        if task4build2apu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR CREATION - BUILD TASK TO APU! 
            # -----------------------------------------------------------------------------------------    
            task4build2apu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1C/70) initialized" + "; task4build2apu-created-by-build2program4apu: " + str(task4build2apu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1) SOURCE PROGRAM - REQUIRED                         
            # ---------------------------------------------------------------------------------------------    
            program2apu = {}
            message2print = "{(Step 1A/70) trying" + "; program2apu-trying-by-create2programwithsource4apu: " + str(program2apu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                program2apu = create2programwithsource4apu()                                                                     
                
                message2print = "{(Step 1B/70) got" + "; program2apu-got-by-create2programwithsource4apu: " + str(program2apu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '1B-'         
            except:
                message2print = "{(Step 1Z/70) failed" + "; program2apu-failed-by-create2programwithsource4apu: " + str(program2apu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '1Z-'
                
            task4build2apu ['PROGRAM'] = program2apu
            
            # ---------------------------------------------------------------------------------------------
            # (2) OPTIONS - REQUIRED                         
            # --------------------------------------------------------------------------------------------- 
            task4build2program  ['OPTIONS'] = {}
            task4build2program  ['OPTIONS'] ['MATH']          = 'SINGLE-PRECISION-CONSTANT'
            task4build2program  ['OPTIONS'] ['OPTIMIZATION']  = 'DISABLE'
            task4build2program  ['OPTIONS'] ['VERSION']       = 'CL1.1'
            task4build2program  ['OPTIONS'] ['WARNINGS']      = 'INHIBIT'            
            
            # ---------------------------------------------------------------------------------------------
            # (3) CALLBACK - REQUIRED                         
            # --------------------------------------------------------------------------------------------- 
            function2callback4apu = trace2program4apu                                                               # CALLBACK FUNCTION FOR APU
            
            message2print = "{(Step 3A/70) initialized" + "; function2callback4apu-created-by-build2program4apu: " + str(function2callback4apu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '3A-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "build2program4apu"                                                            # MODULE OF DRAGONIZED OPENCL MODULE
            args2callback['METHOD']  = "build2program4apu"                                                            # FUNCTION OF DRAGONIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 3Z1/33) failed" + "; build2program4apu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR DRAGONIZED OPENCL TO RETURN    

            message2print = "{(Step 3B/70) initialized" + "; args2callback-created-by-build2program4apu: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code2ack  = code2ack + '3B-'
            
           
            task4build2apu['DATA2CALLBACK'] = args2callback                                                       # USER DATA PASSED TO DRAGONIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4build2apu['FUNCTION2CALLBACK'] = function2callback4apu                                           # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            code2ack  = code2ack + '3C-'

            
            # ---------------------------------------------------------------------------------------------
            # (4) USERDATA - REQUIRED                         
            # ---------------------------------------------------------------------------------------------
            task4build2apu['USERDATA'] = None 
            
            message2print = "{(Step 4A/70) initialized" + "; task4build2apu-created-by-build2program4apu: " + str(task4build2apu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
        else:
            pass                                                                  
    except:
        message2print = "{(Step 1Z1/70) yielded" + "; task4build2apu-yielded-by-build2program4apu: " + str(task4build2apu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '1Z1-'  
                                                                                                 
    # ---------------------------------------------------------------------------------------------
    # TASK EITHER GENERATED BY THIS PROGRAM (NONE GIVEN) OR GIVEN BY USER    
    # ---------------------------------------------------------------------------------------------
    keys2task = task4build2apu.keys()        
    message2print = "{(Step 9/70) using" + "; keys2task-using-by-build2program4apu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    # ['OPTIONS', 'PROGRAM']
  
    # ---------------------------------------------------------------------------------------------
    # (Step 10A/70) - CREATE BUILD FOR APU ! 
    # ---------------------------------------------------------------------------------------------    
    apu2build4xi = {}                                                                                                                                                                               
    message2print = "{(Step 10A/70) initialized" + "; apu2build4xi-initialized-by-build2program4apu to: " + str(apu2build4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # {}
    
    message2print = "{(Step 11A0/70) try" + "; program-try-by-build2program4apu: {}" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 11A/70) - PROGRAM - REQUIRED 
        # -----------------------------------------------------------------------------------------               
        apu2build4xi['PROGRAM'] = task4build2apu['PROGRAM']
        
        message2print = "{(Step 11A1/70) initialized" + "; program-created-by-build2program4apu: " + str(apu2build4xi['PROGRAM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # ...//apu2sampler4xi.py
        
        code2ack  = code2ack + '11A1-'
    except: 
        message2print = "{(Step 11Z1/70) yielded" + "; program-apu2build4xi-yielded-by-build2program4apu: " + str(apu2build4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '11Z1-' 
           
    message2print = "{(Step 12A0/70) try" + "; options-try-by-build2program4apu: {}" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 12A/70) - OPTIONS - REQUIRED 
        # -----------------------------------------------------------------------------------------               
        apu2build4xi['OPTIONS'] = task4build2apu['OPTIONS']
        
        message2print = "{(Step 12A1/70) initialized" + "; options-created-by-build2program4apu: " + str(apu2build4xi['OPTIONS'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'VERSION': 'CL1.1', 'OPTIMIZATION': 'DISABLE', 'MATH': 'SINGLE-PRECISION-CONSTANT', 'WARNINGS': 'INHIBIT'}
        
        code2ack  = code2ack + '12A1-'
    except: 
        message2print = "{(Step 12Z1/70) yielded" + "; options-apu2build4xi-yielded-by-build2program4apu: " + str(apu2build4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '12Z1-' 
        
    message2print = "{(Step 13A0/70) try" + "; callback-try-by-build2program4apu: {}" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 13A/70) - CALLBACK - REQUIRED 
        # -----------------------------------------------------------------------------------------               
        apu2build4xi['CALLBACK'] = task4build2apu['CALLBACK']
        
        message2print = "{(Step 13A1/70) initialized" + "; callback-created-by-build2program4apu: " + str(apu2build4xi['CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        code2ack  = code2ack + '13A1-'
    except: 
        message2print = "{(Step 13Z1/70) yielded" + "; callback-apu2build4xi-yielded-by-build2program4apu: " + str(apu2build4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '13Z1-' 
        
    message2print = "{(Step 14A0/70) try" + "; userdata-try-by-build2program4apu: {}" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 14A/70) - USERDATA - REQUIRED 
        # -----------------------------------------------------------------------------------------               
        apu2build4xi['USERDATA'] = task4build2apu['USERDATA']
        
        message2print = "{(Step 14A1/70) initialized" + "; userdata-created-by-build2program4apu: " + str(apu2build4xi['USERDATA'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        code2ack  = code2ack + '14A1-'
    except: 
        message2print = "{(Step 14Z1/70) yielded" + "; userdata-apu2build4xi-yielded-by-build2program4apu: " + str(apu2build4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '14Z1-'
                               
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    apu2build4xi ['ACK2RETURN']  = code2ack
    apu2build4xi ['NACK2RETURN'] = code2nack  
    
    message2print = "{(Step 70/70) returning" + "; apu2build4xi-returning-by-build2program4apu: " + str(apu2build4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'ACK2RETURN': ' 11A1-12A1-', 
    #  'PROGRAM': '...//apu2sampler4xi.py', 
    #  'NACK2RETURN': ' 13Z1-14Z1-', 
    #  'OPTIONS': {'VERSION': 'CL1.1', 
    #  'OPTIMIZATION': 'DISABLE', 
    #  'MATH': 'SINGLE-PRECISION-CONSTANT', 
    #  'WARNINGS': 'INHIBIT'}
    # }; time: 07:57:00 11/18/11 Pacific Standard Time}

    
    return apu2build4xi

#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "create2programwithsource4apu" usage = "
#     
#   print('(Step 2A1/700) OF APU2PROGRAM4XI5TEST - TRYING TO CREATE PROGRAM SOURCE FOR APU ', time4xi())       
#   try:
#       response4create2programwithsource = create2programwithsource4apu()
#   except:
#       print('(Step 2Z1/700) OF APU2PROGRAM4XI5TEST *** FAILED TO CREATE PROGRAM SOURCE FOR APU *** ', time4xi())
#   try:                                                                                      
#       print('COUNT4SOURCE: ', response4create2programwithsource['COUNT4SOURCE'])
#       print('DIRECTORY4SOURCE: ', response4create2programwithsource['DIRECTORY4SOURCE'])
#       print('LANGUAGE4SOURCE: ', response4create2programwithsource['LANGUAGE4SOURCE'])
#       print('SIZE4SOURCE: ', response4create2programwithsource['SIZE4SOURCE'])
#       keys2source = response4create2programwithsource['CODE4SOURCE'].keys()
#       for key2source in keys2source:
#           print('SOURCE: ', key2source)
#           print(response4create2programwithsource ['CODE4SOURCE'] [key2source] [1])
#           print(response4create2programwithsource ['CODE4SOURCE'] [key2source] [3])
#           print(response4create2programwithsource ['CODE4SOURCE'] [key2source] [4])
#           print(response4create2programwithsource ['CODE4SOURCE'] [key2source] [5])
#           
#       # COUNT4SOURCE:  1
#       # DIRECTORY4SOURCE:  folder2lib4apu
#       # LANGUAGE4SOURCE:  Python
#       # SIZE4SOURCE:  {folder2lib4apu + 'apu2sampler4xi5test.py': 1337}
#       # SOURCE:  folder2lib4apu + 'apu2sampler4xi5test.py'
#       # ... profile = "11" />
#       # ... program = "apu2sampler4xi5test.py">
#       # ... abstract = "tests apu2sampler4xi.py"/>
#       # ... verification = "11" target = "apu2sampler4xi.py"/>                
#   except:
#       print('(Step 2Z2/700) OF APU2PROGRAM4XI5TEST *** FAILED TO PRINT response4create2programwithsource *** ', response4create2programwithsource)
#          
#">
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------   
def create2programwithsource4apu(task4program2apu=None):

    '''
    creates program for APU
    '''
    global apu2program4xi                                                                                           # Program of APU    

    global dir2source4apu                                                                                           # Directory for source files      


    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "create2programwithsource4apu" 
    method2print = "create2programwithsource4apu"

    message2print = "{(Step 1A/70) started" + "; create2programwithsource4apu-started: " + str(create2programwithsource4apu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function create2programwithsource4apu at 0x00000000040D77C8> 
  
             
    message2print = "{(Step 1B/70) passed" + "; task4program2apu-to-create2programwithsource4apu: " + str(task4program2apu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None

    code2ack  = ' '
    code2nack = ' '    
    
    apu2program4xi = {}                                                                                              # INITIALIZES TO EMPTY DICTIONARY
               
    try:             
        if task4program2apu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR CREATION - BUILD TASK TO APU! 
            # -----------------------------------------------------------------------------------------    
            
            task4program2apu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1C/70) initialized" + "; task4program2apu-created-by-create2programwithsource4apu: " + str(task4program2apu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {}
            
            function2callback4apu = trace2program4apu                                                               # CALLBACK FUNCTION FOR APU
            
            message2print = "{(Step 1D/70) initialized" + "; function2callback4apu-created-by-create2programwithsource4apu: " + str(function2callback4apu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2program4apu at 0x0000000003AB12C8>
            
            code2ack  = code2ack + '1D-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "create2programwithsource4apu"                                               # MODULE OF DRAGONIZED OPENCL MODULE
            args2callback['METHOD']  = "create2programwithsource4apu"                                               # FUNCTION OF DRAGONIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; create2programwithsource4apu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR DRAGONIZED OPENCL TO RETURN    

            message2print = "{(Step 1E/70) initialized" + "; args2callback-created-by-create2programwithsource4apu: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; create2programwithsource4apu-failed ; time: 06:57:23 11/15/11 Pacific Standard Time}', 
            #  'CLASS': 'create2programwithsource4apu', 
            #  'METHOD': 'create2programwithsource4apu'}

            code2ack  = code2ack + '1E-'
            
            # ---------------------------------------------------------------------------------------------
            # (1) CALLBACKS - EXTRA                      
            # ---------------------------------------------------------------------------------------------              
            task4program2apu['DATA2CALLBACK'] = args2callback                                                       # USER DATA PASSED TO DRAGONIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4program2apu['FUNCTION2CALLBACK'] = function2callback4apu                                           # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            code2ack  = code2ack + '2C-'
            message2print = "{(Step 1F/70) initialized" + "; task4program2apu-created-by-create2programwithsource4apu: " + str(task4program2apu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2program4apu at 0x0000000003AB12C8>, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2programwithsource4apu-failed ; time: 06:57:23 11/15/11 Pacific Standard Time}', 
            #  'CLASS': 'create2programwithsource4apu', 
            #  'METHOD': 'create2programwithsource4apu'}}; time: 06:57:23 11/15/11 Pacific Standard Time}

            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - EXTRA                      
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-create2programwithsource4apu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-create2programwithsource4apu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 06:57:23 11/15/11 Pacific Standard Time}
                 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-create2programwithsource4apu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4program2apu['PLATFORM'] = platform2xi                                                              # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------    
            context2apu = {}
            message2print = "{(Step 3A/70) trying" + "; context2apu-trying-by-create2programwithsource4apu: " + str(context2apu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2apu = get2context4apu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2apu-got-by-create2programwithsource4apu: " + str(context2apu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': ... 
                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2apu-failed-by-create2programwithsource4apu: " + str(context2apu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4program2apu ['CONTEXT'] = context2apu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - EXTRA                            
            # ---------------------------------------------------------------------------------------------    
            device2apu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2apu-trying-by-create2programwithsource4apu: " + str(device2apu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2apu = get2apu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2apu-got-by-create2programwithsource4apu: " + str(device2apu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568' ...
                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2apu-failed-by-create2programwithsource4apu: " + str(device2apu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
                
            task4program2apu ['DEVICE'] = device2apu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) ERROR CODE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            task4program2apu ['CODE2ERROR'] = 0             
            code2ack  = code2ack + '5B-'              

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - EXTRA                          
            # ---------------------------------------------------------------------------------------------         
            queue2apu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2apu-trying-by-create2programwithsource4apu: " + str(queue2apu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2apu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2apu-got-by-create2programwithsource4apu: " + str(queue2apu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000003ACCF98>
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2apu-failed-by-create2programwithsource4apu: " + str(queue2apu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4program2apu ['QUEUE'] = queue2apu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - EXTRA                              
            # ---------------------------------------------------------------------------------------------         
            pipe2apu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2apu-trying-by-create2programwithsource4apu: " + str(pipe2apu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2apu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2apu-got-by-create2programwithsource4apu: " + str(pipe2apu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 388>, <read-write PipeConnection, handle 392>)
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2apu-failed-by-create2programwithsource4apu: " + str(pipe2apu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4program2apu ['PIPE'] = pipe2apu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2apu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2apu-trying-by-create2programwithsource4apu: " + str(commandqueueinfo2apu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2apu = get2commandqueueinfo4apu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2apu-got-by-create2programwithsource4apu: " + str(commandqueueinfo2apu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0 ... 
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2apu-failed-by-create2programwithsource4apu: " + str(commandqueueinfo2apu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4program2apu ['COMMANDQUEUE'] = commandqueueinfo2apu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2apu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2apu-trying-by-create2programwithsource4apu: " + str(bufferinfo2apu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2apu = get2bufferinfo4apu()                                                               # SYSTEM CREATES BUFFER FOR APU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2apu-got-by-create2programwithsource4apu: " + str(bufferinfo2apu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0 ...
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2apu-failed-by-create2programwithsource4apu: " + str(bufferinfo2apu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4program2apu ['BUFFER'] = bufferinfo2apu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            eventinfo2apu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2apu-trying-by-create2programwithsource4apu: " + str(eventinfo2apu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2apu = get2eventinfo4apu()                                                                 # SYSTEM CREATES EVENT IF NECESSAY   
                message2print = "{(Step 10B/70) got" + "; eventinfo2apu-got-by-create2programwithsource4apu: " + str(eventinfo2apu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'DATA2CALLBACK': ...
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2apu-failed-by-create2programwithsource4apu: " + str(eventinfo2apu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4program2apu ['EVENT'] = eventinfo2apu             
            
            # ---------------------------------------------------------------------------------------------
            # (11) COUNT - REQUIRED                     
            # ---------------------------------------------------------------------------------------------             
            task4program2apu ['COUNT'] = 0
            message2print = "{(Step 11A/70) trying" + "; count2apu-trying-by-create2programwithsource4apu: " + str(task4program2apu['COUNT'])  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:            
                task4program2apu ['COUNT'] = int(len(kb2fetch4apu)/2)                                               # NUMBER OF SOURCE FILES TO BE READ AND EMBEDDED
                message2print = "{(Step 11B/70) set" + "; count2apu-set-by-create2programwithsource4apu to: " + str(task4program2apu['COUNT'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 1                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; count2apu-failed-by-create2programwithsource4apu: " + str(task4program2apu['COUNT']) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
                                               
            # ---------------------------------------------------------------------------------------------
            # (12) DIRECTORY - REQUIRED - COMMON TO ALL KIND OF SOURCE FILES (Python, C++, C ...)                 
            # ---------------------------------------------------------------------------------------------             
            dir2apu = dir2source4apu                                                                                                                                                          
            message2print = "{(Step 12B/70) set" + "; dir2apu-set-by-create2programwithsource4apu to: " + str(dir2apu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #  
            code2ack  = code2ack + '12B-' 
        
            task4program2apu ['DIRECTORY'] = dir2apu
            
                        
            # ---------------------------------------------------------------------------------------------
            # (14) CODE AND SIZE - REQUIRED                 
            # ---------------------------------------------------------------------------------------------             
            message2print = "{(Step 14A/70) using" + "; DirectoryWalker-using-by-create2programwithsource4apu: " + str(DirectoryWalker)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # <class 'apu2program4xi.DirectoryWalker'>
            
            for file2walk in DirectoryWalker(dir2apu):
                if os.path.isdir(file2walk):
                    message2print = "{(Step 14B/70) examining" + "; directory-examining-by-create2programwithsource4apu: " + str(file2walk)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                if os.path.isfile(file2walk):
                    message2print = "{(Step 14C/70) examining" + "; file-examining-by-create2programwithsource4apu: " + str(file2walk)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    #folder2lib4apu +  apu2buffer4xi.py
                    
                    message2print = "{(Step 14D/70) calling" + "; fetch2programsource4apu-calling-by-create2programwithsource4apu: " + str(fetch2programsource4apu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    # <function fetch2programsource4apu at 0x0000000003FE18C8>
                    
                    message2print = "{(Step 14E/70) passing" + "; file2walk-passing-to-fetch2programsource4apu-by-create2programwithsource4apu: " + str(file2walk) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                    #folder2lib4apu +  apu2buffer4xi.py
                    
                    fetch2programsource4apu(file2walk)                                                              # fetches program files into fetch2code4source dictionary 
                    
                    message2print = "{(Step 14E/70) performed" + "; fetch2programsource4apu-performed-by-create2programwithsource4apu: " + str(fetch2programsource4apu) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                                      
                    # 
                    
                    message2print = "{(Step 14F/70) got" + "; fetch2code4source-got-by-create2programwithsource4apu: " + str(fetch2code4source) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                                      
                    # 
                                        
            task4program2apu ['CODE'] = fetch2code4source                                                           # dictionary of source code(s)      
                    
            size2source4apu  = {}
            keys2source4apu = fetch2code4source.keys()                                                              # apu2sampler4xi5test.py
            for key2source4apu in keys2source4apu:
                size2source4apu[key2source4apu] = len(fetch2code4source[key2source4apu])                            # length of saved source   
                       
            message2print = "{(Step 14B/70) set" + "; size2source4apu-set-by-create2programwithsource4apu to: " + str(size2source4apu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
            # {folder2lib4apu + 'apu2sampler4xi5test.py': 1337}
            code2ack  = code2ack + '14B-' 
        
            task4program2apu ['SIZE'] = size2source4apu
            
            # ---------------------------------------------------------------------------------------------
            # (15) LANGUAGE - EXTRA                             
            # ---------------------------------------------------------------------------------------------             
            language2apu  = "Python"  # C++; C
           
            message2print = "{(Step 15B/70) set" + "; language2apu-set-by-create2programwithsource4apu to: " + str(language2apu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # Python
            code2ack  = code2ack + '15B-' 
        
            task4program2apu ['LANGUAGE'] = language2apu     

                                    
            # ---------------------------------------------------------------------------------------------
            # (16) COUNT2REFERENCE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            count2reference2apu  = 1
           
            message2print = "{(Step 17B/70) set" + "; count2reference2apu-set-by-create2programwithbinary4apu to: " + str(count2reference2apu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1 
            code2ack  = code2ack + '17B-' 
        
            task4program2apu ['COUNT2REFERENCE'] = count2reference2apu                                 
                                                                                                                      
        else:
            pass                                                                                                    # PARAMETER DATA RECEIVED                       
    except: 
        message2print = "{(Step 1Z1/70) yielded" + "; task4program2apu-yielded-by-create2programwithsource4apu: " + str(task4program2apu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '1Z1-' 

    # ---------------------------------------------------------------------------------------------
    # TASK EITHER GENERATED BY THIS PROGRAM (NONE GIVEN) OR GIVEN BY USER    
    # ---------------------------------------------------------------------------------------------
    keys2task = task4program2apu.keys()         
    message2print = "{(Step 19A/70) using" + "; keys2task-using-by-create2programwithsource4apu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    # ['BUFFER',
    #  'CODE',
    #  'CODE2ERROR',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'COUNT',
    #  'COUNT2REFERENCE',
    #  'DATA2CALLBACK',
    #  'DEVICE',
    #  'DIRECTORY',
    #  'EVENT',
    #  'FUNCTION2CALLBACK',
    #  'LANGUAGE',
    #  'PIPE',
    #  'PLATFORM',
    #  'QUEUE',
    #  'SIZE']
    
    # ---------------------------------------------------------------------------------------------
    # (Step 20A/70) - CREATE PROGRAM OBJECT FOR APU ! 
    # ---------------------------------------------------------------------------------------------    
    apu2program4xi = {}                                                                                             # CREATE INITIALIZES DICTIONARY!                                                                                                                                                                               
    message2print = "{(Step 20A/70) initialized" + "; apu2program4xi-initialized-by-create2programwithsource4apu to: " + str(apu2program4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 

    if 'DATA2CALLBACK' in keys2task:    
        message2print = "{(Step 21A0/70) try" + "; data2callback-try-by-create2programwithsource4apu: {}" + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)      
        # 
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 21A/70) - DATA2CALLBACK - EXTRA 
            # -----------------------------------------------------------------------------------------               
            apu2program4xi['DATA2CALLBACK'] = task4program2apu['DATA2CALLBACK']
            
            message2print = "{(Step 21A1/70) initialized" + "; data2callback-created-by-create2programwithsource4apu: " + str(apu2program4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'MESSAGE': '{(Step 1Z1/33) failed; create2programwithsource4apu-failed ; time: 01:21:23 11/16/11 Pacific Standard Time}', 
            #  'CLASS': 'create2programwithsource4apu', 
            #  'METHOD': 'create2programwithsource4apu'}; time: 01:21:23 11/16/11 Pacific Standard Time}
     
            
            code2ack  = code2ack + '21A1-'
        except: 
            message2print = "{(Step 21Z1/70) yielded" + "; data2callback-apu2program4xi-yielded-by-create2programwithsource4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '21Z1-'

    if 'FUNCTION2CALLBACK' in keys2task:     
        message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-create2programwithsource4apu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        # 
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 22B/70) - FUNCTION2CALLBACK - EXTRA
            # -----------------------------------------------------------------------------------------               
            apu2program4xi['FUNCTION2CALLBACK'] = task4program2apu['FUNCTION2CALLBACK']
            
            message2print = "{(Step 22B1/70) initialized" + "; function2callback-created-by-create2programwithsource4apu: " + str(apu2program4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <function trace2program4apu at 0x000000000392E2C8>
            
            code2ack  = code2ack + '22B1-'
        except: 
            message2print = "{(Step 22Z2/70) yielded" + "; function2callback-apu2program4xi-yielded-by-create2programwithsource4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '22Z2-' 

    if 'PLATFORM' in keys2task:     
        message2print = "{(Step 22C0/70) try" + "; platform-try-by-create2programwithsource4apu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        # 
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 22C/70) - PLATFORM - EXTRA
            # -----------------------------------------------------------------------------------------               
            apu2program4xi['PLATFORM'] = task4program2apu['PLATFORM']
            
            message2print = "{(Step 22C1/70) initialized" + "; platform-created-by-create2programwithsource4apu: " + str(apu2program4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 01:21:23 11/16/11 Pacific Standard Time}
    
            code2ack  = code2ack + '22C1-'
        except: 
            message2print = "{(Step 22Z3/70) yielded" + "; platform-apu2program4xi-yielded-by-create2programwithsource4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '22Z3-' 
        

    if 'CONTEXT' in keys2task:         
          
        message2print = "{(Step 23A/70) try" + "; context2apu-try-by-create2programwithsource4apu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 23/70) - CONTEXT - REQUIRED
            # -----------------------------------------------------------------------------------------               
            apu2program4xi['CONTEXT'] = task4program2apu['CONTEXT']
            
            message2print = "{(Step 23B/70) initialized" + "; context-created-by-create2programwithsource4apu: " + str(apu2program4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': ...        
            
            code2ack  = code2ack + '23B-'
        except: 
            message2print = "{(Step 23Z1/70) yielded" + "; context-apu2program4xi-yielded-by-create2programwithsource4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
        
            code2nack  = code2nack + '23Z1-' 

    if 'DEVICE' in keys2task:         
        message2print = "{(Step 24A/70) try" + "; device-try-by-create2programwithsource4apu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 24/70) - DEVICE - EXTRA
            # -----------------------------------------------------------------------------------------               
            apu2program4xi['DEVICE'] = task4program2apu['DEVICE']
            
            message2print = "{(Step 24B/70) initialized" + "; device-created-by-create2programwithsource4apu: " + str(apu2program4xi['DEVICE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568' ... 
            
            code2ack  = code2ack + '24B-'
        except: 
            message2print = "{(Step 24Z1/70) yielded" + "; device-apu2program4xi-yielded-by-create2programwithsource4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '24Z1-'

    if 'CODE2ERROR' in keys2task:                  
        message2print = "{(Step 25A/70) try" + "; code2error-try-by-create2programwithsource4apu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 25/70) - ERROR CODE - REQUIRED
            # -----------------------------------------------------------------------------------------               
            apu2program4xi['CODE2ERROR'] = task4program2apu['CODE2ERROR']
            
            message2print = "{(Step 25B/70) initialized" + "; code2error-created-by-create2programwithsource4apu: " + str(apu2program4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 0 
            
            code2ack  = code2ack + '25B-'
        except: 
            message2print = "{(Step 25Z1/70) yielded" + "; code2error-apu2program4xi-yielded-by-create2programwithsource4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '25Z1-'                

    if 'QUEUE' in keys2task:                 
              
        message2print = "{(Step 26A/70) try" + "; queue-try-by-create2programwithsource4apu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 26/70) - QUEUE - EXTRA
            # -----------------------------------------------------------------------------------------               
            apu2program4xi['QUEUE'] = task4program2apu['QUEUE']
            
            message2print = "{(Step 26B/70) initialized" + "; queue-created-by-create2programwithsource4apu: " + str(apu2program4xi['QUEUE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <queue4xi.Queue4Xi object at 0x0000000003EEC978> 
            
            code2ack  = code2ack + '26B-'
        except: 
            message2print = "{(Step 26Z1/70) yielded" + "; queue-apu2program4xi-yielded-by-create2programwithsource4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '26Z1-' 

    if 'PIPE' in keys2task:                                 
        message2print = "{(Step 27A/70) try" + "; pipe-try-by-create2programwithsource4apu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                     
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 27/70) - PIPE - EXTRA
            # -----------------------------------------------------------------------------------------               
            apu2program4xi['PIPE'] = task4program2apu['PIPE']
            
            message2print = "{(Step 27B/70) initialized" + "; pipe-created-by-create2programwithsource4apu: " + str(apu2program4xi['PIPE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # (<read-write PipeConnection, handle 384>, <read-write PipeConnection, handle 388>) 
            
            code2ack  = code2ack + '27B-'
        except: 
            message2print = "{(Step 27Z1/70) yielded" + "; pipe-apu2program4xi-yielded-by-create2programwithsource4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '27Z1-'   

    if 'COMMANDQUEUE' in keys2task: 
        message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-create2programwithsource4apu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
                            
            apu2program4xi['COMMANDQUEUE']  = task4program2apu['COMMANDQUEUE']  
     
            message2print = "{(Step 28B/70) initialized" + "; commandqueue-created-by-create2programwithsource4apu: " + str(apu2program4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, ... 
                    
            code2ack  = code2ack + '28B-'
        except: 
            message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-create2programwithsource4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                  
            code2nack  = code2nack + '28Z1-'    

    if 'BUFFER' in keys2task: 
        message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-create2programwithsource4apu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
                            
            apu2program4xi['BUFFER']  = task4program2apu['BUFFER']  
     
            message2print = "{(Step 29B/70) initialized" + "; bufferinfo-created-by-create2programwithsource4apu: " + str(apu2program4xi['BUFFER'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0 ...
            
            
            code2ack  = code2ack + '29B-'
        except: 
            message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-create2programwithsource4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '29Z1-'            

    if 'EVENT' in keys2task: 
        message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-create2programwithsource4apu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
                            
            apu2program4xi['EVENT']  = task4program2apu['EVENT']  
     
            message2print = "{(Step 30B/70) initialized" + "; eventinfo-created-by-create2programwithsource4apu: " + str(apu2program4xi['EVENT'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'DATA2CALLBACK':  ... 
            
            code2ack  = code2ack + '30B-'
        except: 
            message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-create2programwithsource4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '30Z1-'            

    if 'COUNT' in keys2task:                 
        message2print = "{(Step 31A1/70) try" + "; task4program2apu-try-by-create2programwithsource4apu: " + str(task4program2apu)  + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # {'COUNT': 1, 'CODE2ERROR': 0 ... 
        message2print = "{(Step 31A2/70) try" + "; count-try-by-create2programwithsource4apu: " + str(task4program2apu['COUNT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 1 
        try:            
            # ---------------------------------------------------------------------------------------------
            # (31) COUNT - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            apu2program4xi['COUNT4SOURCE']  = task4program2apu['COUNT']  
    
            message2print = "{(Step 31B/70) initialized" + "; count-created-by-create2programwithsource4apu: " + str(apu2program4xi['COUNT4SOURCE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1
            code2ack  = code2ack + '31B-'
        except: 
            message2print = "{(Step 31Z1/70) yielded" + "; count-yielded-by-create2programwithsource4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '31Z1-'     

    if 'DIRECTORY' in keys2task: 
        message2print = "{(Step 32A/70) try" + "; directory-try-by-create2programwithsource4apu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (32) DIRECTORY - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            apu2program4xi['DIRECTORY4SOURCE']  = task4program2apu['DIRECTORY']  
     
            message2print = "{(Step 32B/70) initialized" + "; directory-created-by-create2programwithsource4apu: " + str(apu2program4xi['DIRECTORY4SOURCE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #
            code2ack  = code2ack + '32B-'
        except: 
            message2print = "{(Step 32Z1/70) yielded" + "; DIRECTORY-yielded-by-create2programwithsource4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '32Z1-'  

    if 'CODE' in keys2task:         
        message2print = "{(Step 33A/70) try" + "; codetask-try-by-create2programwithsource4apu: " + str(task4program2apu ['CODE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        # {folder2lib4apu + 'apu2sampler4xi5test.py': {1: '... profile = "11" />\n', 2: ...
        
        apu2program4xi['CODE4SOURCE'] = {}
        try:            
            # ---------------------------------------------------------------------------------------------
            # (33) CODE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            apu2program4xi['CODE4SOURCE'] = task4program2apu ['CODE']
    
     
            message2print = "{(Step 33B/70) created" + "; code-created-by-create2programwithsource4apu: " + str(apu2program4xi['CODE4SOURCE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {folder2lib4apu + 'apu2sampler4xi5test.py': {1: '... profile = "11" />\n', 2:
             
            code2ack  = code2ack + '33B-'
        except: 
            message2print = "{(Step 33Z1/70) yielded" + "; source-yielded-by-create2programwithsource4apu: " + str(apu2program4xi['CODE4SOURCE']) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '33Z1-'     

    if 'SIZE' in keys2task:         
        message2print = "{(Step 34A/70) try" + "; size-try-by-create2programwithsource4apu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (34) SIZE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            apu2program4xi['SIZE4SOURCE']  = task4program2apu['SIZE']  
     
            message2print = "{(Step 34B/70) initialized" + "; size-initialized-by-create2programwithsource4apu to: " + str(apu2program4xi['SIZE4SOURCE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {folder2lib4apu + 'apu2sampler4xi5test.py': 1337}
            code2ack  = code2ack + '34B-'
        except: 
            message2print = "{(Step 34Z1/70) yielded" + "; size-yielded-by-create2programwithsource4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '34Z1-'     

    if 'LANGUAGE' in keys2task:         
        message2print = "{(Step 35A/70) try" + "; language-try-by-create2programwithsource4apu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (35) LANGUAGE - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
                            
            apu2program4xi['LANGUAGE4SOURCE']  = task4program2apu['LANGUAGE']  
     
            message2print = "{(Step 35B/70) initialized" + "; language-created-by-create2programwithsource4apu: " + str(apu2program4xi['LANGUAGE4SOURCE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # Python
            code2ack  = code2ack + '35B-'
        except: 
            message2print = "{(Step 35Z1/70) yielded" + "; language-yielded-by-create2programwithsource4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '35Z1-'     

    if 'COUNT2REFERENCE' in keys2task:                 
        message2print = "{(Step 36A/70) try" + "; count2reference-try-by-create2programwithsource4apu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (36) COUNT2REFERENCE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
                            
            apu2program4xi['COUNT2REFERENCE']  = task4program2apu['COUNT2REFERENCE']  
     
            message2print = "{(Step 36B/70) initialized" + "; count2reference-created-by-create2programwithsource4apu: " + str(apu2program4xi['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1
            code2ack  = code2ack + '36B-'
        except: 
            message2print = "{(Step 36Z1/70) yielded" + "; count2reference-yielded-by-create2programwithsource4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '36Z1-'     
        
                                                                                                               
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)     
    # -----------------------------------------------------------------------------------------
    apu2program4xi ['ACK2RETURN']  = code2ack
    apu2program4xi ['NACK2RETURN'] = code2nack  
    keys2program4apu = apu2program4xi.keys()
    for key2program4apu in keys2program4apu:
        message2print = "{(Step 70/70) returning" + "; key2program4apu-returning-by-create2programwithsource4apu: " + str(key2program4apu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # COUNT4SOURCE
        # SIZE4SOURCE
        # DATA2CALLBACK
        # BUFFER
        # FUNCTION2CALLBACK
        # ACK2RETURN
        # DIRECTORY4SOURCE
        # QUEUE
        # PLATFORM
        # COUNT2REFERENCE
        # NACK2RETURN
        # CONTEXT
        # COMMANDQUEUE
        # DEVICE
        # CODE4SOURCE
        # EVENT
        # PIPE

    return apu2program4xi
            
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "add2programwithbinary4apu" usage = "
#     
#   print('(Step 3A2/700) OF APU2PROGRAM4XI5TEST - TRYING TO ADD BINARY INTO PROGRAM OBJECT WITH FUNCTION ', time4xi())        
#   try:
#       task4add2program = {}
#       task4add2program ['CODE4BINARY'] = {}
#       # NOT OK - EXISTS
#       task4add2program ['CODE4BINARY'] [folder2lib4apu + '__pycache__//apu2image4xi.cpython-33.pyc'] = {}
#       task4add2program ['CODE4BINARY'] [folder2lib4apu + '__pycache__//apu2image4xi.cpython-33.pyc'] [1] = b'l\x0c\r\n'
#       task4add2program ['CODE4BINARY'] [folder2lib4apu + '__pycache__//apu2image4xi.cpython-33.pyc'] [2] = b'l\x0c\z\n' 
#       task4add2program ['SIZE4BINARY'] = {}
#       task4add2program ['SIZE4BINARY'] [folder2lib4apu + '__pycache__//apu2image4xi.cpython-33.pyc'] = len(task4add2program ['CODE4BINARY'] [folder2lib4apu + '__pycache__//apu2image4xi.cpython-33.pyc'])
#                
#       # OK
#       task4add2program ['CODE4BINARY'] [folder2lib4apu + '__pycache__//apu3image4xi.cpython-33.pyc'] = {}
#       task4add2program ['CODE4BINARY'] [folder2lib4apu + '__pycache__//apu3image4xi.cpython-33.pyc'] [1] = b'l\x0c\r\n'
#       task4add2program ['CODE4BINARY'] [folder2lib4apu + '__pycache__//apu3image4xi.cpython-33.pyc'] [2] = b'l\x0c\z\n' 
#       task4add2program ['SIZE4BINARY'] = {}
#       task4add2program ['SIZE4BINARY'] [folder2lib4apu + '__pycache__//apu3image4xi.cpython-33.pyc'] = len(task4add2program ['CODE4BINARY'] [folder2lib4apu + '__pycache__//apu3image4xi.cpython-33.pyc'])
#       
#       response4add2programwithbinary = add2programwithbinary4apu(task4add2program)
#       print('(Step 3A3/700) OF APU2PROGRAM4XI5TEST - ADDED BINARY INTO PROGRAM OBJECT WITH FUNCTION ', time4xi())
#   except:
#       print('(Step 3Z2/700) OF APU2PROGRAM4XI5TEST *** FAILED TO PRINT response4add2programwithbinary: ', response4add2programwithbinary)
#   try:                                                                                          
#       keys2response = response4add2programwithbinary.keys()
#       print('COUNT2REFERENCE:', response4add2programwithbinary ['COUNT2REFERENCE'])   
#       print('COUNT4BINARY:', response4add2programwithbinary ['COUNT4BINARY'])
#       print('DIRECTORY4BINARY:', response4add2programwithbinary ['DIRECTORY4BINARY']) 
#       print('FUNCTION2CALLBACK:', response4add2programwithbinary ['FUNCTION2CALLBACK'])   
#       print('LANGUAGE4BINARY:', response4add2programwithbinary ['LANGUAGE4BINARY'])
#       print('SIZE4BINARY:', response4add2programwithbinary ['SIZE4BINARY'])                
#       # 
#       keys2source = response4add2programwithbinary['CODE4BINARY'].keys()
#       for key2source in keys2source:
#           print('BINARY: ', key2source)
#           print(response4add2programwithbinary ['CODE4BINARY'] [key2source] [1])
#           # 
#           # COUNT2REFERENCE: 1
#           # COUNT4BINARY: 2
#           # DIRECTORY4BINARY: ...//__pycache__//
#           # FUNCTION2CALLBACK: <function trace2program4apu at 0x0000000003F96AC8>
#           # LANGUAGE4BINARY: Python
#           # SIZE4BINARY: {'...//__pycache__//apu2image4xi.cpython-33.pyc': 1006, '...//__pycache__//apu3image4xi.cpython-33.pyc': 2}
#           # BINARY:  ...//__pycache__//apu2image4xi.cpython-33.pyc
#           # b'l\x0c\r\n'
#           # BINARY:  ...//__pycache__//apu3image4xi.cpython-33.pyc
#           # b'l\x0c\r\n'                       
#   except:
#       print('(Step 3Z3/700) OF APU2PROGRAM4XI5TEST *** FAILED TO PRINT response4add2programwithbinary : ', response4add2programwithbinary)
#          
#">
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------   
def add2programwithbinary4apu(task4program2apu):

    '''
    adds program for APU
    '''
    global apu2program4xi                                                                                           # Program of APU    
    
  
    # -----------------------------------------------------------------------------------------
    # (Step 10/70)      
    # -----------------------------------------------------------------------------------------          
    class2print = "add2programwithbinary4apu" 
    method2print = "add2programwithbinary4apu"

    message2print = "{(Step 10A/70) started" + "; add2programwithbinary4apu-started: " + str(add2programwithbinary4apu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function add2programwithbinary4apu at 0x000000000418DC48>
  
    message2print = "{(Step 10B/70) passed" + "; task4program2apu-to-add2programwithbinary4apu: " + str(task4program2apu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                              
    # {'CODE4BINARY': {folder2lib4apu + '__pycache__//apu2image4xi.cpython-33.pyc': {1: b'l\x0c\r\n', 2: b'l\x0c\\z\n'}, 
    #                  folder2lib4apu + '__pycache__//apu3image4xi.cpython-33.pyc': {1: b'l\x0c\r\n', 2: b'l\x0c\\z\n'}}, 
    #  'SIZE4BINARY': {folder2lib4apu + '__pycache__//apu3image4xi.cpython-33.pyc': 2}
    # }
    
    code2ack  = ' '
    code2nack = ' ' 
    try:
        keys2binary4task = task4program2apu ['CODE4BINARY'].keys()
        message2print = "{(Step 10C/70) using" + "; keys2binary4task-using-by-add2programwithbinary4apu: " + str(keys2binary4task)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # //__pycache__//apu2image4xi.cpython-33.pyc
        # //__pycache__//apu3image4xi.cpython-33.pyc          
    except: 
        message2print = "{(Step 10Z1/70) failed" + "; keys2binary4task-failed-by-add2programwithbinary4apu: " + str(task4program2apu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
    try:
        keys2exist4task =  apu2program4xi ['CODE4BINARY'].keys() 
        message2print = "{(Step 10D/70) using" + "; keys2exist4task-using-by-add2programwithbinary4apu: " + str(keys2exist4task)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
    except:
        try:
            response2createwb4apu = create2programwithbinary4apu()
            keys2createwb4apu = response2createwb4apu.keys()
            for key2createwb4apu in keys2createwb4apu:
                if key2createwb4apu == 'CODE4BINARY':
                    message2print = "{(Step 10E/70) created" + "; key2createwb4apu-created-by-add2programwithbinary4apu: " + str(key2createwb4apu) + "; time: " + time4xi() + "}" 
                    print4xi(class2print, method2print, message2print)
                    # {'... //__pycache__//apu2image4xi.cpython-33.pyc'
            
            keys2exist4task =  apu2program4xi ['CODE4BINARY'].keys() 
            message2print = "{(Step 10F/70) using" + "; keys2exist4task-using-by-add2programwithbinary4apu: " + str(keys2exist4task)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # //__pycache__//apu2image4xi.cpython-33.pyc'
        
        except: 
            message2print = "{(Step 10Z2/70) failed" + "; apu2program4xi-failed-by-add2programwithbinary4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
    try:            
        # ---------------------------------------------------------------------------------------------
        # (33) CODE - EXPECTED - SEVERAL BINARY CODE CAN BE ADDED INTO ONE PROGRAM OBJECT                              
        # ---------------------------------------------------------------------------------------------             
        for key2binary4task in keys2binary4task:
            try:
                message2print = "{(Step 33A/70) using" + "; key2binary4task-using-by-add2programwithbinary4apu: " + str(key2binary4task) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # //__pycache__//apu2image4xi.cpython-33.pyc
                            
                if key2binary4task in keys2exist4task:
                    message2print = "{(Step 33Z0/70) yielded" + "; key2binary4task-yielded-by-add2programwithbinary4apu to: " + str(key2binary4task)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    #folder2lib4apu +  __pycache__//apu2image4xi.cpython-33.pyc
                     
                    code2nack  = code2nack + '33Z0-'
                else: 
                    try:
                        apu2program4xi ['CODE4BINARY'] [key2binary4task] = task4program2apu ['CODE4BINARY'] [key2binary4task]
                    except:
                        message2print = "{(Step 33Z1/70) failed" + "; task4program2apu-failed-by-add2programwithbinary4apu: " + str(task4program2apu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        code2nack  = code2nack + '33Z1-' 
                    try:             
                        message2print = "{(Step 33B/70) added" + "; key2binary4task-added-by-add2programwithbinary4apu to: " + str(key2binary4task)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        #folder2lib4apu +  __pycache__//apu3image4xi.cpython-33.pyc
                    except:
                        message2print = "{(Step 33Z2/70) failed" + "; task4program2apu-failed-by-add2programwithbinary4apu: " + str(task4program2apu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        code2nack  = code2nack + '33Z2-' 
                    try:             
                        message2print = "{(Step 33C/70) added" + "; code-added-by-add2programwithbinary4apu to: " + str(apu2program4xi ['CODE4BINARY'] [key2binary4task])  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        # {1: b'l\x0c\r\n', 2: b'l\x0c\\z\n'}
                    except:
                        message2print = "{(Step 33Z3/70) failed" + "; task4program2apu-failed-by-add2programwithbinary4apu: " + str(task4program2apu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        code2nack  = code2nack + '33Z3-'                 
                    try:
                        apu2program4xi ['COUNT4BINARY'] = apu2program4xi ['COUNT4BINARY'] + 1
                        message2print = "{(Step 33D/70) added" + "; count4binary-added-by-add2programwithbinary4apu to: " + str(apu2program4xi ['COUNT4BINARY'])  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        # 2 
                    except:
                        message2print = "{(Step 33Z4/70) failed" + "; task4program2apu-failed-by-add2programwithbinary4apu: " + str(task4program2apu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        code2nack  = code2nack + '33Z4-' 
                    try:                            
                        apu2program4xi ['SIZE4BINARY'] [key2binary4task]  = task4program2apu['SIZE4BINARY'] [key2binary4task]  
                        message2print = "{(Step 33E/70) set" + "; size4binary-set-by-add2programwithbinary4apu to: " + str(apu2program4xi ['SIZE4BINARY'] [key2binary4task])  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        # 2
                        code2ack  = code2ack + '33D-'
                    except:
                        message2print = "{(Step 33Z5/70) failed" + "; task4program2apu-failed-by-add2programwithbinary4apu: " + str(task4program2apu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        code2nack  = code2nack + '33Z5-'    
            except:
                message2print = "{(Step 33Z6/70) failed" + "; task4program2apu-failed-by-add2programwithbinary4apu: " + str(task4program2apu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                code2nack  = code2nack + '33Z6-'                     
    except: 
        message2print = "{(Step 33Z7/70) yielded" + "; binary-yielded-by-add2programwithbinary4apu: " + str(apu2program4xi ['CODE4BINARY']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '33Z7-'       
          
                                                                                                                   
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    apu2program4xi ['ACK2RETURN']  = code2ack
    apu2program4xi ['NACK2RETURN'] = code2nack  
    
    keys2binaryprograms = apu2program4xi ['CODE4BINARY'].keys()
    for key2binaryprogram in sorted(keys2binaryprograms):
        message2print = "{(Step 70A/70) got" + "; key2binaryprogram-got-by-add2programwithbinary4apu: " + str(key2binaryprogram) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)                                                              
        #folder2lib4apu +  __pycache__//apu2image4xi.cpython-33.pyc
        #folder2lib4apu +  __pycache__//apu3image4xi.cpython-33.pyc

    keys2program4apu = apu2program4xi.keys()
    for key2program4apu in keys2program4apu:
        message2print = "{(Step 70B/70) returning" + "; key2program4apu-returning-by-add2programwithbinary4apu: " + str(key2program4apu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # 
        
    return apu2program4xi
            
#</function4xi>



# -------------------------------------------------------------------------------------------------
#<function4xi function = "add2programwithsource4apu" usage = "
#     
#   print('(Step 2A3/700) OF APU2PROGRAM4XI5TEST - TRYING TO ADD SOURCE INTO PROGRAM OBJECT WITH FUNCTION ', time4xi())          
#   try:
#       task4add2program = {}
#       # OK
#       task4add2program ['CODE4SOURCE'] = {}
#       task4add2program ['CODE4SOURCE'] [folder2lib4apu + 'apu2buffer4xi.py'] = {}
#       task4add2program ['CODE4SOURCE'] [folder2lib4apu + 'apu2buffer4xi.py'] [1] = '... profile = "33" />\n'
#       task4add2program ['CODE4SOURCE'] [folder2lib4apu + 'apu2buffer4xi.py'] [2] = '... program = "apu2buffer4xi.py" />\n'
#       task4add2program ['SIZE4SOURCE'] = {}
#       task4add2program ['SIZE4SOURCE'] [folder2lib4apu + 'apu2buffer4xi.py'] = len(task4add2program ['CODE4SOURCE'] [folder2lib4apu + 'apu2buffer4xi.py'])
#       # NOT OK - EXISTS
#       task4add2program ['CODE4SOURCE'] [folder2lib4apu + 'apu2sampler4xi5test.py'] = {}
#       task4add2program ['CODE4SOURCE'] [folder2lib4apu + 'apu2sampler4xi5test.py'] [1] = '... profile = "33" />\n'
#       task4add2program ['CODE4SOURCE'] [folder2lib4apu + 'apu2sampler4xi5test.py'] [2] = '... program = "apu2sampler4xi5test.py" />\n'
#       
#       task4add2program ['SIZE4SOURCE'] = {}
#       task4add2program ['SIZE4SOURCE'] [folder2lib4apu + 'apu2sampler4xi5test.py'] = len(task4add2program ['CODE4SOURCE'] [folder2lib4apu + 'apu2sampler4xi5test.py'])            
#       task4add2program ['SIZE4SOURCE'] [folder2lib4apu + 'apu2buffer4xi.py'] = len(task4add2program ['CODE4SOURCE'] [folder2lib4apu + 'apu2buffer4xi.py'])            
#       
#       response4add2programwithsource = add2programwithsource4apu(task4add2program)
#       print('(Step 2A4/700) OF APU2PROGRAM4XI5TEST - ADDED SOURCE INTO PROGRAM OBJECT WITH FUNCTION ', time4xi())
#       #                                                                                   
#   except:
#       print('(Step 2Z3/700) OF APU2PROGRAM4XI5TEST *** FAILED TO ADD SOURCE INTO PROGRAM OBJECT WITH FUNCTION *** ', time4xi())
#   try:                                                                                      
#       print('COUNT4SOURCE: ', response4add2programwithsource['COUNT4SOURCE'])
#       print('DIRECTORY4SOURCE: ', response4add2programwithsource['DIRECTORY4SOURCE'])
#       print('LANGUAGE4SOURCE: ', response4add2programwithsource['LANGUAGE4SOURCE'])
#       print('SIZE4SOURCE: ', response4add2programwithsource['SIZE4SOURCE'])
#       keys2source = response4add2programwithsource['CODE4SOURCE'].keys()
#       for key2source in keys2source:
#           print('SOURCE: ', key2source)
#           print(response4add2programwithsource ['CODE4SOURCE'] [key2source] [1])
#           # COUNT4SOURCE:  2
#           # DIRECTORY4SOURCE:  folder2lib4apu
#           # LANGUAGE4SOURCE:  Python
#           # SIZE4SOURCE:  {'folder2lib4apu + apu2buffer4xi.py': 2, 'folder2lib4apu + apu2sampler4xi5test.py': 1337}
#           # SOURCE:  folder2lib4apu + apu2buffer4xi.py
#           # ... profile = "33" />
#           # 
#           # SOURCE:  folder2lib4apu + apu2sampler4xi5test.py
#           # ... profile = "11" />           
#   except:
#       print('(Step 2Z4/700) OF APU2PROGRAM4XI5TEST *** FAILED TO PRINT response4add2programwithsource *** ', response4add2programwithsource)     
#          
#">
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------   
def add2programwithsource4apu(task4program2apu): 

    '''
    adds program for APU
    '''
    global apu2program4xi                                                                                           # Program of APU    

    global dir2source4apu                                                                                           # Directory for source files      

    
        
    # -----------------------------------------------------------------------------------------
    # (Step 10/70)      
    # -----------------------------------------------------------------------------------------          
    class2print = "add2programwithsource4apu" 
    method2print = "add2programwithsource4apu"

    message2print = "{(Step 10A/70) started" + "; add2programwithsource4apu-started: " + str(add2programwithsource4apu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function add2programwithsource4apu at 0x000000000418DC48>
  
    message2print = "{(Step 10B/70) passed" + "; task4program2apu-to-add2programwithsource4apu: " + str(task4program2apu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                              
    # {'CODE4SOURCE': {folder2lib4apu + 'apu2buffer4xi.py': {1: '... profile = "33" />\n', 2: '... program = "apu2buffer4xi.py" />\n'}, 
    #                  folder2lib4apu + 'apu2sampler4xi5test.py': {1: '... profile = "33" />\n', 2: '... program = "apu2sampler4xi5test.py" />\n'}}, 
    #  'SIZE4SOURCE': {folder2lib4apu + 'apu2buffer4xi.py': 2, 
    #                  folder2lib4apu + 'apu2sampler4xi5test.py': 2}}
    code2ack  = ' '
    code2nack = ' ' 
    try:
        keys2source4task = task4program2apu ['CODE4SOURCE'].keys()
        message2print = "{(Step 10C/70) using" + "; keys2source4task-using-by-add2programwithsource4apu: " + str(keys2source4task)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
              
    except: 
        message2print = "{(Step 10Z1/70) failed" + "; keys2source4task-failed-by-add2programwithsource4apu: " + str(task4program2apu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
    try:
        keys2exist4task =  apu2program4xi ['CODE4SOURCE'].keys() 
        message2print = "{(Step 10D/70) using" + "; keys2exist4task-using-by-add2programwithsource4apu: " + str(keys2exist4task)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
    except:
        try:
            response2createwb4apu = create2programwithsource4apu()
            keys2createwb4apu = response2createwb4apu.keys()
            for key2createwb4apu in keys2createwb4apu:
                if key2createwb4apu == 'CODE4SOURCE':
                    message2print = "{(Step 10E/70) created" + "; key2createwb4apu-created-by-add2programwithsource4apu: " + str(key2createwb4apu) + "; time: " + time4xi() + "}" 
                    print4xi(class2print, method2print, message2print)
                    # CODE4SOURCE 
            
            keys2exist4task =  apu2program4xi ['CODE4SOURCE'].keys() 
            message2print = "{(Step 10F/70) using" + "; keys2exist4task-using-by-add2programwithsource4apu: " + str(keys2exist4task)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # dict_keys(['...//apu2sampler4xi5test.py'])
        
        except: 
            message2print = "{(Step 10Z2/70) failed" + "; apu2program4xi-failed-by-add2programwithsource4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
    try:            
        # ---------------------------------------------------------------------------------------------
        # (33) CODE - EXPECTED - SEVERAL SOURCE CODE CAN BE ADDED INTO ONE PROGRAM OBJECT                              
        # ---------------------------------------------------------------------------------------------             
        for key2source4task in keys2source4task:
            if key2source4task in keys2exist4task:
                message2print = "{(Step 33Z0/70) yielded" + "; key2source4task-yielded-by-add2programwithsource4apu to: " + str(key2source4task)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #folder2lib4apu +  apu2sampler4xi5test.py
                 
                code2nack  = code2nack + '33Z0-'
            else: 
                apu2program4xi ['CODE4SOURCE'] [key2source4task] = task4program2apu ['CODE4SOURCE'] [key2source4task]
 
                message2print = "{(Step 33B/70) added" + "; key2source4task-added-by-add2programwithsource4apu to: " + str(key2source4task)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #folder2lib4apu +  apu2buffer4xi.py
 
                message2print = "{(Step 33C/70) added" + "; code-added-by-add2programwithsource4apu to: " + str(apu2program4xi ['CODE4SOURCE'] [key2source4task])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {1: '... profile = "33" />\n', 2: '... program = "apu2buffer4xi.py" />\n'}
         
            
                apu2program4xi ['COUNT4SOURCE'] = apu2program4xi ['COUNT4SOURCE'] + 1
                message2print = "{(Step 33D/70) added" + "; count4source-added-by-add2programwithsource4apu to: " + str(apu2program4xi ['COUNT4SOURCE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 2
                
                apu2program4xi ['SIZE4SOURCE'] [key2source4task]  = task4program2apu['SIZE4SOURCE'] [key2source4task]  
                message2print = "{(Step 33E/70) set" + "; size4source-set-by-add2programwithsource4apu to: " + str(apu2program4xi ['SIZE4SOURCE'] [key2source4task])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 2
                
                         
                code2ack  = code2ack + '33D-'
    except: 
        message2print = "{(Step 33Z1/70) yielded" + "; source-yielded-by-add2programwithsource4apu: " + str(apu2program4xi ['CODE4SOURCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # apu2sampler4xi5test.py
        
        code2nack  = code2nack + '33Z1-'     
        

                                                                                                                   
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    apu2program4xi ['ACK2RETURN']  = code2ack
    apu2program4xi ['NACK2RETURN'] = code2nack  
    
    keys2sourceprograms = apu2program4xi ['CODE4SOURCE'].keys()
    for key2sourceprograms in sorted(keys2sourceprograms):
        message2print = "{(Step 70A/70) got" + "; key2sourceprograms-got-by-add2programwithsource4apu: " + str(key2sourceprograms) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)                                                              
        #folder2lib4apu +  apu2buffer4xi.py
        #folder2lib4apu +  apu2sampler4xi5test.py
          
     
    keys2program4apu = apu2program4xi.keys()
    for key2program4apu in keys2program4apu:
        message2print = "{(Step 70B/70) returning" + "; key2program4apu-returning-by-add2programwithsource4apu: " + str(key2program4apu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # COUNT4SOURCE
        # SIZE4SOURCE
        # DATA2CALLBACK
        # BUFFER
        # FUNCTION2CALLBACK
        # ACK2RETURN
        # DIRECTORY4SOURCE
        # QUEUE
        # PLATFORM
        # COUNT2REFERENCE
        # NACK2RETURN
        # CONTEXT
        # COMMANDQUEUE
        # DEVICE
        # CODE4SOURCE
        # EVENT
        # PIPE 

    return apu2program4xi
            
#</function4xi>


# -------------------------------------------------------------------------------------------------
#<function4xi function = "compile2program4apu" usage = "
#     
#   print('(Step 2A2/700) OF APU2PROGRAM4XI5TEST - TRYING TO COMPILE SOURCE INTO PROGRAM OBJECT WITH FUNCTION ', time4xi())           
#   try:
#       response4compile = compile2program4apu()
#   except:
#       print('(Step 2Z1/700) OF APU2PROGRAM4XI5TEST *** FAILED TO COMPILE SOURCE INTO PROGRAM OBJECT WITH FUNCTION *** ', time4xi())                                                                      
#   try:
#       print('(Step 2B2/700) OF APU2PROGRAM4XI5TEST - RESPONSE4COMPILE: ', time4xi())                                                                                      
#       keys2response4compile = response4compile.keys()
#       for key2response4compile in keys2response4compile:
#           print('KEY2RESPONSE4COMPILE: ', key2response4compile)
#           # apu2sampler4xi5test.py            
#   except:
#       print('(Step 2Z2/700) OF APU2PROGRAM4XI5TEST *** FAILED TO PRINT response4compile *** ', response4compile)
#   tailor2trace4program(t2xi)                                                                              # TAILOR TRACE AGAIN - TARGET FUNCTIONS OF ACTIVATED PROCESSES MIGHT HAVE RESET TRACE ! 
#          
#">
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------   
def compile2program4apu(task4program2apu=None):

    '''
    compiles program for APU
    '''
    global apu2program4xi                                                                                           # Program of APU    
    global import2use4apu                                                                                           # Imports of APU
    global target2function4apu                                                                                      # Target function of Process4Xi
    

    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "compile2program4apu" 
    method2print = "compile2program4apu"

    message2print = "{(Step 1A/70) started" + "; compile2program4apu-started: " + str(compile2program4apu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function compile2program4apu at 0x00000000039D5E48>
  
             
    message2print = "{(Step 1B/70) passed" + "; task4program2apu-to-compile2program4apu: " + str(task4program2apu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None

    code2ack  = ' '
    code2nack = ' '    
    
    source2program4apu = {}                                                                                       # INITIALIZES TO EMPTY DICTIONARY
    message2print = "{(Step 1C/70) initialized" + "; source2program4apu-initialized-by-compile2program4apu to: " + str(source2program4apu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    #                
    try:             
        if task4program2apu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER
            # -----------------------------------------------------------------------------------------    
            pass                                                                       
        else:
            # -----------------------------------------------------------------------------------------
            # PARAMETERS GIVEN ARE OMITTED 
            # -----------------------------------------------------------------------------------------         
            pass                                                                                                    # PARAMETER DATA RECEIVED                       
    except: 
        message2print = "{(Step 1Z1/70) yielded" + "; task4program2apu-yielded-by-compile2program4apu: " + str(task4program2apu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
       
        code2nack  = code2nack + '1Z1-' 

    
    # ---------------------------------------------------------------------------------------------
    # (Step 20A/70) - COMPILE SOURCES INSIDE PROGRAM OBJECT FOR APU ! 
    # ---------------------------------------------------------------------------------------------    
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) - SOURCE CODE WITHOUT EOL CHARACTERS                            
        # ---------------------------------------------------------------------------------------------             
        keys2source4apu = apu2program4xi['CODE4SOURCE'].keys()
        message2print = "{(Step 30A/70) using" + "; keys2source4apu-using-by-compile2program4apu: " + str(keys2source4apu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # dict_keys([folder2lib4apu + 'apu2sampler4xi5test.py'])
        
        unit4processing = "apu"
        
        for key2source4apu in keys2source4apu:
            index2search=str.find(key2source4apu,unit4processing) 
            if index2search == -1:
                pass
            else:
                target2compile = key2source4apu[index2search:]                                                          # apu2sampler4xi5test.py
                index2search=str.find(target2compile,".py")
                if index2search == -1:
                    pass
                else:
                    target2compile4apu = target2compile[:index2search]                                                  
                    message2print = "{(Step 32A/70) set" + "; target2compile4apu-set-by-compile2program4apu to: " + str(target2compile4apu)  + "; time: " + time4xi() + "}"  
                    print4xi(class2print, method2print, message2print)
                    # apu2sampler4xi5test

                    p2source4apu = None    
                    try:
                        # ---------------------------------------------------------------------------------------------    
                        # IMPORT TARGET FUNCTION FROM ITS MODULE AS TARGET2FUNCTION4APU 
                        # ---------------------------------------------------------------------------------------------    
             
                        try:
                            import2use4apu.get(target2compile4apu, handle2error4apu) ()
                            
                            message2print = "{(Step 32B/70) imported" + "; target2compile4apu-imported-by-compile2program4apu: " + str(target2compile4apu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)
                            # apu2sampler4xi5test
                            code2ack  = code2ack + '32B-'
                        except:
                            message2print = "{(Step 32Z1/70) failed" + "; import2use4apu-failed-by-compile2program4apu: " + str(import2use4apu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)                            
                            # 
                            message2print = "{(Step 32Z2/70) used" + "; target2compile4apu-used-by-compile2program4apu: " + str(target2compile4apu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)
                            # 
                            code2nack  = code2nack + '32Z2-'
                                    
                        # ---------------------------------------------------------------------------------------------    
                        # CREATE PROCESS WITH TARGET2FUNCTION4APU
                        # ---------------------------------------------------------------------------------------------                          
                        p2apu = Process4Xi(target=target2function4apu, args=())                                
                        message2print = "{(Step 33A/70) created" + "; p2apu-created-by-compile2program4apu: " + str(p2apu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        # <Process4Xi(Process4Xi-1, initial)>
                        
                        code2ack  = code2ack + '33A-'
                        try:
                            # ---------------------------------------------------------------------------------------------    
                            # START PROCESS
                            # --------------------------------------------------------------------------------------------- 
                            p2apu.start()
                            message2print = "{(Step 34A/70) started" + "; p2apu-started-by-compile2program4apu: " + str(p2apu.name) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)
                            # Process4Xi-1
                            
                            try:
                                # ---------------------------------------------------------------------------------------------    
                                # STATUS OF PROCESS
                                # ---------------------------------------------------------------------------------------------                             
                                status2process4apu = p2apu.is_alive()
                                message2print = "{(Step 35A/70) got" + "; status-got-by-compile2program4apu: " + str(status2process4apu) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)
                                # True
                                
                                try:
                                    # ---------------------------------------------------------------------------------------------    
                                    # RUN PROCESS
                                    # ---------------------------------------------------------------------------------------------                                  
                                    p2apu.run()
                                    message2print = "{(Step 36A/70) run" + "; p2apu-run-by-compile2program4apu: " + str(p2apu) + "; time: " + time4xi() + "}"
                                    print4xi(class2print, method2print, message2print)
                                    # <Process4Xi(Process4Xi-1, stopped[1])>
                                except:
                                    message2print = "{(Step 36Z/70) failed" + "; run-failed-by-compile2program4apu: " + str(p2apu) + "; time: " + time4xi() + "}"
                                    print4xi(class2print, method2print, message2print)  
                                    code2nack  = code2nack + '36Z-'            
                                    # 
                            except:
                                message2print = "{(Step 35Z/70) failed" + "; status-failed-by-compile2program4apu: " + str(p2apu) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)  
                                # 
                                code2nack  = code2nack + '35Z-'                                   
                                
                        except:
                            message2print = "{(Step 33Z/70) failed" + "; start-failed-by-compile2program4apu: " + str(p2apu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)  
                            code2nack  = code2nack + '33Z-'     
                        
                    except:
                        message2print = "{(Step 32Z/70) failed" + "; p2source4apu-failed-by-compile2program4apu: " + str(p2source4apu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        code2nack  = code2nack + '32Z-' 
                                                                                    
            # ---------------------------------------------------------------------------------------------
            # SAVE SOURCE WITHOUT END OF LINES
            # ---------------------------------------------------------------------------------------------        
            keys2lines = apu2program4xi['CODE4SOURCE'][key2source4apu].keys()     
            source2program4apu[key2source4apu] = ""
            for key2line in keys2lines:
                code2add4apu = apu2program4xi ['CODE4SOURCE'] [key2source4apu] [key2line]
                source2program4apu[key2source4apu] = source2program4apu[key2source4apu] + code2add4apu.replace("\n","")
        
            message2print = "{(Step 34A/70) saved" + "; lenght-source4apu-saved-by-compile2program4apu to: " + str(len(source2program4apu[key2source4apu]))  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #     
                
 
        message2print = "{(Step 38A/70) set" + "; lenght-source2program4apu-set-by-compile2program4apu to: " + str(len(source2program4apu))  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
         
        code2ack  = code2ack + '38A-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; source-yielded-by-compile2program4apu: " + str(apu2program4xi['CODE4SOURCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '30Z1-'     

                                                                                                               
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    apu2program4xi ['ACK2RETURN']  = code2ack
    apu2program4xi ['NACK2RETURN'] = code2nack  
    
    keys2source4apu = source2program4apu.keys()
    for key2source4apu in keys2source4apu:
        message2print = "{(Step 70/70) returning" + "; key2source4apu-returning-by-compile2program4apu: " + str(key2source4apu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)                                                              
        # 
    
    return source2program4apu
            
#</function4xi> 
 
# -------------------------------------------------------------------------------------------------
#<function4xi function = "delete2programwithbinary4apu" usage = "
#     
#   print('(Step 3A3/700) OF APU2PROGRAM4XI5TEST - TRYING TO DELETE BINARY INSIDE PROGRAM OBJECT WITH FUNCTION ', time4xi())        
#   try:
#       task4delete2program = {}
#       task4delete2program ['CODE4BINARY'] = {}
#       task4delete2program ['CODE4BINARY'] [folder2lib4apu + '__pycache__//apu3image4xi.cpython-33.pyc'] = 'Yes'
#       task4delete2program ['CODE4BINARY'] [folder2lib4apu + '__pycache__//dummy3image4xi.cpython-33.pyc'] = 'Yes'
#            task4delete2program ['SIZE4BINARY'] = {}
#       task4delete2program ['SIZE4BINARY'] [folder2lib4apu + '__pycache__//apu3image4xi.cpython-33.pyc'] = 'Yes'
#       task4delete2program ['SIZE4BINARY'] [folder2lib4apu + '__pycache__//dummy3image4xi.cpython-33.pyc'] = 'Yes'
#       
#       response4delete2programwithbinary = delete2programwithbinary4apu(task4delete2program) 
#       print('(Step 3A3/700) OF APU2PROGRAM4XI5TEST - DELETED BINARY INSIDE PROGRAM OBJECT WITH FUNCTION ', time4xi())
#   except:
#       print('(Step 3Z3/700) OF APU2PROGRAM4XI5TEST *** FAILED TO DELETE BINARY INSIDE PROGRAM OBJECT WITH FUNCTION *** ', time4xi()) 
#   try:                                                                                          
#       keys2response = response4delete2programwithbinary.keys()
#       print('COUNT2REFERENCE:', response4delete2programwithbinary ['COUNT2REFERENCE'])   
#       print('COUNT4BINARY:', response4delete2programwithbinary ['COUNT4BINARY'])
#       print('DIRECTORY4BINARY:', response4delete2programwithbinary ['DIRECTORY4BINARY']) 
#       print('FUNCTION2CALLBACK:', response4delete2programwithbinary ['FUNCTION2CALLBACK'])   
#       print('LANGUAGE4BINARY:', response4delete2programwithbinary ['LANGUAGE4BINARY'])
#       print('SIZE4BINARY:', response4delete2programwithbinary ['SIZE4BINARY'])                
#       # 
#       keys2source = response4delete2programwithbinary['CODE4BINARY'].keys()
#       for key2source in keys2source:
#           print('BINARY: ', key2source)
#           print(response4delete2programwithbinary ['CODE4BINARY'] [key2source] [1])
#       # COUNT2REFERENCE: 1
#       # COUNT4BINARY: 1
#       # DIRECTORY4BINARY: ...//__pycache__//
#       # FUNCTION2CALLBACK: <function trace2program4apu at 0x0000000003B46AC8>
#       # LANGUAGE4BINARY: Python
#       # SIZE4BINARY: {'...//__pycache__//apu2image4xi.cpython-33.pyc': 1006}
#       # BINARY:  ... //__pycache__//apu2image4xi.cpython-33.pyc                       
#   except:
#       print('(Step 3Z2/700) OF APU2PROGRAM4XI5TEST *** FAILED TO PRINT response4delete2programwithbinary : ', response4delete2programwithbinary)   
#          
#">
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------   
def delete2programwithbinary4apu(task4program2apu):

    '''
    deletes program for APU
    '''
    global apu2program4xi                                                                                     


    # -----------------------------------------------------------------------------------------
    # (Step 10/70)      
    # -----------------------------------------------------------------------------------------          
    class2print = "delete2programwithbinary4apu" 
    method2print = "delete2programwithbinary4apu"

    message2print = "{(Step 10A/70) started" + "; delete2programwithbinary4apu-started: " + str(delete2programwithbinary4apu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function delete2programwithbinary4apu at 0x000000000405CEC8>
  
    message2print = "{(Step 10B/70) passed" + "; task4program2apu-to-delete2programwithbinary4apu: " + str(task4program2apu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 


    code2ack  = ' '
    code2nack = ' '    
    #
    ack4key2programbinary = False
    try:
        # ---------------------------------------------------------------------------------------------
        # TASK MUST HAVE BEEN GIVEN BY USER - CONTAINS BINARY KEYS    
        # ---------------------------------------------------------------------------------------------
        keys2delete4binary = task4program2apu ['CODE4BINARY'].keys()        
        message2print = "{(Step 19A/70) using" + "; keys2delete4binary-using-by-delete2programwithbinary4apu: " + str(sorted(keys2delete4binary))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        # 
        
        # ---------------------------------------------------------------------------------------------
        # (Step 20A/70) - DELETE BINARY INSIDE PROGRAM OBJECT FOR APU ! 
        # ---------------------------------------------------------------------------------------------    
        message2print = "{(Step 20A0/70) try" + "; key-try-by-delete2programwithbinary4apu: None" + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)      

        for key2delete4binary in keys2delete4binary:
            try:            
                del apu2program4xi ['CODE4BINARY'] [key2delete4binary]        
               
                message2print = "{(Step 20C/70) deleted" + "; code4binary-deleted-by-delete2programwithbinary4apu: " + str(key2delete4binary) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                    
                # 
                
                code2ack  = code2ack + '20C-' 
                
                del apu2program4xi ['SIZE4BINARY'] [key2delete4binary]        
               
                message2print = "{(Step 20D/70) deleted" + "; size4binary-deleted-by-delete2programwithbinary4apu: " + str(key2delete4binary) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                    
                # 
                
                code2ack  = code2ack + '20D-' 
                
                apu2program4xi ['COUNT4BINARY'] = apu2program4xi ['COUNT4BINARY'] - 1
                message2print = "{(Step 20E/70) upgraded" + "; count4binary-upgraded-by-delete2programwithbinary4apu: " + str(apu2program4xi ['COUNT4BINARY'] ) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                    
                #                                
                
            except:
                message2print = "{(Step 20Z1/70) yielded" + "; key2delete4binary-yielded-by-delete2programwithbinary4apu: " + str(key2delete4binary) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #                
                code2nack  = code2nack + '20Z1-' 
    except: 
        message2print = "{(Step 20Z0/70) failed" + "; task4program2apu-apu2program4xi-failed-by-delete2programwithbinary4apu: " + str(task4program2apu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '20Z0-'    
                                                                                            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    apu2program4xi ['ACK2RETURN']  = code2ack
    apu2program4xi ['NACK2RETURN'] = code2nack  
    try:
        keys2binaryprograms = apu2program4xi ['CODE4BINARY'] .keys()
        for key2binaryprograms in keys2binaryprograms:
            message2print = "{(Step 70A/70) got" + "; key2binaryprograms-got-by-delete2programwithbinary4apu: " + str(key2binaryprograms) + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print)                                                              
            #  
    except:
        pass 
               
     
    keys2program4apu = apu2program4xi.keys()
    for key2program4apu in keys2program4apu:
        message2print = "{(Step 70B/70) returning" + "; key2program4apu-returning-by-delete2programwithbinary4apu: " + str(key2program4apu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # 


    return apu2program4xi
            
#</function4xi> 
 
# -------------------------------------------------------------------------------------------------
#<function4xi function = "delete2programwithsource4apu" usage = "
#     
#   print('(Step 2A4/700) OF APU2PROGRAM4XI5TEST - TRYING TO DELETE SOURCE INTO PROGRAM OBJECT WITH FUNCTION ', time4xi())        
#   try:
#       task4delete2program = {}
#       task4delete2program ['CODE4SOURCE'] = {}
#       task4delete2program ['CODE4SOURCE'] [folder2lib4apu + 'apu2buffer4xi.py'] = 'Yes'
#       task4delete2program ['CODE4SOURCE'] [folder2lib4apu + 'apu2dummy4xi.py'] = 'Yes'
#            task4delete2program ['SIZE4SOURCE'] = {}
#       task4delete2program ['SIZE4SOURCE'] [folder2lib4apu + 'apu2buffer4xi.py'] = 'Yes'
#       task4delete2program ['SIZE4SOURCE'] [folder2lib4apu + 'apu2dummy4xi.py'] = 'Yes'
#       
#       response4delete2programwithsource = delete2programwithsource4apu(task4delete2program)
#       print('(Step 2A5/700) OF APU2PROGRAM4XI5TEST - DELETED SOURCE INTO PROGRAM OBJECT WITH FUNCTION ', time4xi())
#   except:
#       print('(Step 2Z4/700) OF APU2PROGRAM4XI5TEST *** FAILED TO DELETE SOURCE INTO PROGRAM OBJECT WITH FUNCTION *** ', time4xi()) 
#   try:                                                                                      
#       print('COUNT4SOURCE: ', response4delete2programwithsource['COUNT4SOURCE'])
#       print('DIRECTORY4SOURCE: ', response4delete2programwithsource['DIRECTORY4SOURCE'])
#       print('LANGUAGE4SOURCE: ', response4delete2programwithsource['LANGUAGE4SOURCE'])
#       print('SIZE4SOURCE: ', response4delete2programwithsource['SIZE4SOURCE'])
#       keys2source = response4delete2programwithsource['CODE4SOURCE'].keys()
#       for key2source in keys2source:
#           print('SOURCE: ', key2source)
#           print(response4delete2programwithsource ['CODE4SOURCE'] [key2source] [1])
#           # COUNT4SOURCE:  1
#           # DIRECTORY4SOURCE:  OK
#           # LANGUAGE4SOURCE:  Python
#           # SIZE4SOURCE:  {'... //apu2sampler4xi5test.py': 1337}
#           # SOURCE:  ... //apu2sampler4xi5test.py
#           # ... profile = "11" />       
#   except:
#       print('(Step 2Z5/700) OF APU2PROGRAM4XI5TEST *** FAILED TO PRINT response4delete2programwithsource *** ', response4delete2programwithsource) 
#          
#">
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------   
def delete2programwithsource4apu(task4program2apu):

    '''
    deletes program for APU
    '''
    global apu2program4xi                                                                                           # Program of APU    

    # -----------------------------------------------------------------------------------------
    # (Step 10/70)      
    # -----------------------------------------------------------------------------------------          
    class2print = "delete2programwithsource4apu" 
    method2print = "delete2programwithsource4apu"

    message2print = "{(Step 10A/70) started" + "; delete2programwithsource4apu-started: " + str(delete2programwithsource4apu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function delete2programwithsource4apu at 0x000000000405CEC8>
  
    message2print = "{(Step 10B/70) passed" + "; task4program2apu-to-delete2programwithsource4apu: " + str(task4program2apu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'CODE4SOURCE': {folder2lib4apu + 'apu2buffer4xi.py': 'Yes'}, 
    #  'SIZE4SOURCE': {folder2lib4apu + 'apu2buffer4xi.py': 'Yes'}
    # }; 


    code2ack  = ' '
    code2nack = ' '    
    #
    ack4key2programsource = False
    try:
        # ---------------------------------------------------------------------------------------------
        # TASK MUST HAVE BEEN GIVEN BY USER - CONTAINS SOURCE KEYS    
        # ---------------------------------------------------------------------------------------------
        keys2delete4source = task4program2apu ['CODE4SOURCE'].keys()        
        message2print = "{(Step 19A/70) using" + "; keys2delete4source-using-by-delete2programwithsource4apu: " + str(sorted(keys2delete4source))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        # [folder2lib4apu + 'apu2buffer4xi.py']
        
        # ---------------------------------------------------------------------------------------------
        # (Step 20A/70) - DELETE SOURCE INSIDE PROGRAM OBJECT FOR APU ! 
        # ---------------------------------------------------------------------------------------------    
        message2print = "{(Step 20A0/70) try" + "; key-try-by-delete2programwithsource4apu: None" + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)      

        for key2delete4source in keys2delete4source:
            try:            
                del apu2program4xi ['CODE4SOURCE'] [key2delete4source]        
               
                message2print = "{(Step 20C/70) deleted" + "; code4source-deleted-by-delete2programwithsource4apu: " + str(key2delete4source) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                    
                #folder2lib4apu +  apu2buffer4xi.py; apu2dummy4xi5test.py
                
                code2ack  = code2ack + '20C-' 
                
                del apu2program4xi ['SIZE4SOURCE'] [key2delete4source]        
               
                message2print = "{(Step 20D/70) deleted" + "; size4source-deleted-by-delete2programwithsource4apu: " + str(key2delete4source) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                    
                #folder2lib4apu +  apu2buffer4xi.py
                
                code2ack  = code2ack + '20D-' 
                
                apu2program4xi ['COUNT4SOURCE'] = apu2program4xi ['COUNT4SOURCE'] - 1
                message2print = "{(Step 20E/70) upgraded" + "; count4source-upgraded-by-delete2programwithsource4apu: " + str(apu2program4xi ['COUNT4SOURCE'] ) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                    
                # 1                                  
                
            except:
                message2print = "{(Step 20Z1/70) yielded" + "; key2delete4source-yielded-by-delete2programwithsource4apu: " + str(key2delete4source) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #folder2lib4apu +  apu2dummy4xi.py                    
                code2nack  = code2nack + '20Z1-' 
    except: 
        message2print = "{(Step 20Z0/70) failed" + "; task4program2apu-apu2program4xi-failed-by-delete2programwithsource4apu: " + str(task4program2apu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '20Z0-'    
                                                                                            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    apu2program4xi ['ACK2RETURN']  = code2ack
    apu2program4xi ['NACK2RETURN'] = code2nack  
    try:
        keys2sourceprograms = apu2program4xi ['CODE4SOURCE'] .keys()
        for key2sourceprograms in keys2sourceprograms:
            message2print = "{(Step 70A/70) got" + "; key2sourceprograms-got-by-delete2programwithsource4apu: " + str(key2sourceprograms) + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print)                                                              
            #folder2lib4apu +  apu2sampler4xi5test.py    
    except:
        pass 
               
     
    keys2program4apu = apu2program4xi.keys()
    for key2program4apu in keys2program4apu:
        message2print = "{(Step 70B/70) returning" + "; key2program4apu-returning-by-delete2programwithsource4apu: " + str(key2program4apu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # CODE2ERROR ...

    return apu2program4xi
            
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "create2programwithbinary4apu" usage = "
#     
#   print('(Step 3A1/700) OF APU2PROGRAM4XI5TEST - TRYING TO CREATE PROGRAM BINARY FOR APU ', time4xi())          
#   try:
#       response4create2programwithbinary = create2programwithbinary4apu()
#       print('(Step 3A2/700) OF APU2PROGRAM4XI5TEST - CREATED PROGRAM BINARY FOR APU ', time4xi())
#   except:
#       print('(Step 3Z1/700) OF APU2PROGRAM4XI5TEST *** FAILED TO CREATE PROGRAM BINARY FOR APU *** ', time4xi())
#   try:                                                                                          
#       keys2response = response4create2programwithbinary.keys()
#       print('COUNT2REFERENCE:', response4create2programwithbinary ['COUNT2REFERENCE'])   
#       print('COUNT4BINARY:', response4create2programwithbinary ['COUNT4BINARY'])
#       print('DIRECTORY4BINARY:', response4create2programwithbinary ['DIRECTORY4BINARY']) 
#       print('FUNCTION2CALLBACK:', response4create2programwithbinary ['FUNCTION2CALLBACK'])   
#       print('LANGUAGE4BINARY:', response4create2programwithbinary ['LANGUAGE4BINARY'])
#       print('SIZE4BINARY:', response4create2programwithbinary ['SIZE4BINARY'])                
#       # COUNT2REFERENCE: 1
#       # COUNT4BINARY: 1
#       # DIRECTORY4BINARY: ...__pycache__//
#       # FUNCTION2CALLBACK: <function trace2program4apu at 0x0000000003F53AC8>
#       # LANGUAGE4BINARY: Python
#       # SIZE4BINARY: {'...__pycache__//apu2image4xi.cpython-33.pyc': 1006}
#       keys2source = response4create2programwithbinary['CODE4BINARY'].keys()
#       for key2source in keys2source:
#           print('BINARY: ', key2source)
#           print(response4create2programwithbinary ['CODE4BINARY'] [key2source] [1])
#           # BINARY: ...//__pycache__//apu2image4xi.cpython-33.pyc
#           # b'l\x0c\r\n'    
#   except:
#       print('(Step 3Z2/700) OF APU2PROGRAM4XI5TEST *** FAILED TO PRINT response4create2programwithbinary : ', response4create2programwithbinary)
#
#">
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------   
def create2programwithbinary4apu(task4program2apu=None):

    '''
    creates program with binary for APU
    '''
    global apu2program4xi                                                                                           # Program of APU    

    global dir2binary4apu                                                                                           # Directory for binary files      

    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "create2programwithbinary4apu" 
    method2print = "create2programwithbinary4apu"

    message2print = "{(Step 1A/70) started" + "; create2programwithbinary4apu-started: " + str(create2programwithbinary4apu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function create2programwithbinary4apu at 0x0000000003B19048>
  
             
    message2print = "{(Step 1B/70) passed" + "; task4program2apu-to-create2programwithbinary4apu: " + str(task4program2apu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None

    code2ack  = ' '
    code2nack = ' '    
    
    apu2program4xi = {}                                                                                             # INITIALIZES TO EMPTY DICTIONARY
    message2print = "{(Step 1E/70) initialized" + "; apu2program4xi-initialized-by-create2programwithbinary4apu to: " + str(apu2program4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                 
    # {}
    try:              
        if task4program2apu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR CREATION - BUILD TASK TO APU! 
            # -----------------------------------------------------------------------------------------    
            
            task4program2apu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1C/70) initialized" + "; task4program2apu-created-by-create2programwithbinary4apu: " + str(task4program2apu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {}
            
            function2callback4apu = trace2program4apu                                                               # CALLBACK FUNCTION FOR APU
            
            message2print = "{(Step 1D/70) initialized" + "; function2callback4apu-created-by-create2programwithbinary4apu: " + str(function2callback4apu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2program4apu at 0x00000000039F12C8>
            
            code2ack  = code2ack + '1D-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "create2programwithbinary4apu"                                               # MODULE OF DRAGONIZED OPENCL MODULE
            args2callback['METHOD']  = "create2programwithbinary4apu"                                               # FUNCTION OF DRAGONIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; create2programwithbinary4apu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR DRAGONIZED OPENCL TO RETURN    

            message2print = "{(Step 1E/70) initialized" + "; args2callback-created-by-create2programwithbinary4apu: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; create2programwithbinary4apu-failed ; time: 22:50:41 11/25/11 Pacific Standard Time}', 
            #  'CLASS': 'create2programwithbinary4apu', 
            #  'METHOD': 'create2programwithbinary4apu'}; time: 22:50:41 11/25/11 Pacific Standard Time}


            code2ack  = code2ack + '1E-'
            
            # ---------------------------------------------------------------------------------------------
            # (1) CALLBACKS - EXTRA                      
            # ---------------------------------------------------------------------------------------------              
            task4program2apu['DATA2CALLBACK'] = args2callback                                                       # USER DATA PASSED TO DRAGONIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4program2apu['FUNCTION2CALLBACK'] = function2callback4apu                                           # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            code2ack  = code2ack + '2C-'
            message2print = "{(Step 1F/70) initialized" + "; task4program2apu-created-by-create2programwithbinary4apu: " + str(task4program2apu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2program4apu at 0x0000000003B18AC8>, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2programwithbinary4apu-failed ; time: 22:50:41 11/25/11 Pacific Standard Time}', 
            #  'CLASS': 'create2programwithbinary4apu', 'METHOD': 'create2programwithbinary4apu'}}; time: 22:50:41 11/25/11 Pacific Standard Time}

            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - EXTRA                      
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-create2programwithbinary4apu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-create2programwithbinary4apu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 22:50:41 11/25/11 Pacific Standard Time}

                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-create2programwithbinary4apu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4program2apu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------    
            context2apu = {}
            message2print = "{(Step 3A/70) trying" + "; context2apu-trying-by-create2programwithbinary4apu: " + str(context2apu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2apu = get2context4apu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2apu-got-by-create2programwithbinary4apu: " + str(context2apu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK':  ...
                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2apu-failed-by-create2programwithbinary4apu: " + str(context2apu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4program2apu ['CONTEXT'] = context2apu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - EXTRA                            
            # ---------------------------------------------------------------------------------------------    
            device2apu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2apu-trying-by-create2programwithbinary4apu: " + str(device2apu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2apu = get2apu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2apu-got-by-create2programwithbinary4apu: " + str(device2apu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568' ...
                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2apu-failed-by-create2programwithbinary4apu: " + str(device2apu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
                
            task4program2apu ['DEVICE'] = device2apu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) ERROR CODE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            task4program2apu ['CODE2ERROR'] = 0             
            code2ack  = code2ack + '5B-'              

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - EXTRA                          
            # ---------------------------------------------------------------------------------------------         
            queue2apu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2apu-trying-by-create2programwithbinary4apu: " + str(queue2apu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2apu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2apu-got-by-create2programwithbinary4apu: " + str(queue2apu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000003BD1E10>
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2apu-failed-by-create2programwithbinary4apu: " + str(queue2apu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4program2apu ['QUEUE'] = queue2apu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - EXTRA                              
            # ---------------------------------------------------------------------------------------------         
            pipe2apu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2apu-trying-by-create2programwithbinary4apu: " + str(pipe2apu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2apu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2apu-got-by-create2programwithbinary4apu: " + str(pipe2apu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 388>, <read-write PipeConnection, handle 392>)
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2apu-failed-by-create2programwithbinary4apu: " + str(pipe2apu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4program2apu ['PIPE'] = pipe2apu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2apu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2apu-trying-by-create2programwithbinary4apu: " + str(commandqueueinfo2apu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2apu = get2commandqueueinfo4apu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2apu-got-by-create2programwithbinary4apu: " + str(commandqueueinfo2apu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0 ...
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2apu-failed-by-create2programwithbinary4apu: " + str(commandqueueinfo2apu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4program2apu ['COMMANDQUEUE'] = commandqueueinfo2apu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2apu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2apu-trying-by-create2programwithbinary4apu: " + str(bufferinfo2apu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2apu = get2bufferinfo4apu()                                                                   # SYSTEM CREATES BUFFER FOR APU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2apu-got-by-create2programwithbinary4apu: " + str(bufferinfo2apu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0 ... 
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2apu-failed-by-create2programwithbinary4apu: " + str(bufferinfo2apu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4program2apu ['BUFFER'] = bufferinfo2apu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            eventinfo2apu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2apu-trying-by-create2programwithbinary4apu: " + str(eventinfo2apu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2apu = get2eventinfo4apu()                                                                     # SYSTEM CREATES EVENT IF NECESSAY   
                message2print = "{(Step 10B/70) got" + "; eventinfo2apu-got-by-create2programwithbinary4apu: " + str(eventinfo2apu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'DATA2CALLBACK': ... 
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2apu-failed-by-create2programwithbinary4apu: " + str(eventinfo2apu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4program2apu ['EVENT'] = eventinfo2apu             
            
            # ---------------------------------------------------------------------------------------------
            # (11) COUNT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            task4program2apu ['COUNT'] = 0
            message2print = "{(Step 11A/70) trying" + "; count2apu-trying-by-create2programwithbinary4apu: " + str(task4program2apu['COUNT'])  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
            # 0            
            try:            
                task4program2apu ['COUNT'] = int(len(kb2fetch4apu)/2)                                               # number of binary files to be read and embedded
                message2print = "{(Step 11B/70) set" + "; count2apu-set-by-create2programwithbinary4apu to: " + str(task4program2apu['COUNT'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 1                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; count2apu-failed-by-create2programwithbinary4apu: " + str(task4program2apu['COUNT']) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
                               
            # ---------------------------------------------------------------------------------------------
            # (12) DIRECTORY - EXTRA                       
            # ---------------------------------------------------------------------------------------------             
            dir2apu = dir2binary4apu                                                                                # default Python cache folder for binaries                                                                                                                                                                 
            
            message2print = "{(Step 12B/70) set" + "; dir2apu-set-by-create2programwithbinary4apu to: " + str(dir2apu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # folder2lib4apu +  __pycache__//
            
            code2ack  = code2ack + '12B-' 
            task4program2apu ['DIRECTORY'] = dir2apu
            
                        
                        
            # ---------------------------------------------------------------------------------------------
            # (14) CODE AND SIZE - REQUIRED                   
            # ---------------------------------------------------------------------------------------------             
            message2print = "{(Step 14A/70) using" + "; DirectoryWalker-using-by-create2programwithbinary4apu: " + str(DirectoryWalker)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # <class 'apu2program4xi.DirectoryWalker'> 
            
            for file2walk in DirectoryWalker(dir2apu):
                if os.path.isdir(file2walk):
                    message2print = "{(Step 14B/70) examining" + "; directory-examining-by-create2programwithbinary4apu: " + str(file2walk)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                      
                if os.path.isfile(file2walk):
                    message2print = "{(Step 14C/70) examining" + "; file-examining-by-create2programwithbinary4apu: " + str(file2walk)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    # folder2lib4apu +  __pycache__//apu2buffer4xi.cpython-33.pyc
                    
                    message2print = "{(Step 14D/70) calling" + "; fetch2programbinary4apu-calling-by-create2programwithbinary4apu: " + str(fetch2programbinary4apu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    # <function fetch2programbinary4apu at 0x00000000040079C8>
                    
                    message2print = "{(Step 14E/70) passing" + "; file2walk-passing-to-fetch2programbinary4apu-by-create2programwithbinary4apu: " + str(file2walk) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                    # folder2lib4apu +  __pycache__//apu2buffer4xi.cpython-33.pyc
                    
                    fetch2programbinary4apu(file2walk)                                                                 # fetches program files into fetch2code4binary dictionary 
                    
                    message2print = "{(Step 14F/70) performed" + "; fetch2programbinary4apu-performed-by-create2programwithbinary4apu: " + str(fetch2programbinary4apu) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                                      
                    # <function fetch2programbinary4apu at 0x00000000040079C8>
                    
            task4program2apu ['CODE'] = fetch2code4binary        
            size2binary4apu  = {}
            keys2binary4apu = fetch2code4binary.keys()                                                                 # apu2sampler4xi5test.py
            for key2binary4apu in keys2binary4apu:
                size2binary4apu[key2binary4apu] = len(fetch2code4binary[key2binary4apu])                               # length of saved binary   
                
                message2print = "{(Step 14H/70) using" + "; key2binary4apu-using-by-create2programwithbinary4apu: " + str(key2binary4apu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # folder2lib4apu +  __pycache__//apu2image4xi.cpython-33.pyc                       
                message2print = "{(Step 14I/70) set" + "; size2binary4apu-set-by-create2programwithbinary4apu to: " + str(len(fetch2code4binary[key2binary4apu]))  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 1006
            
            code2ack  = code2ack + '14B-' 
        
            task4program2apu ['SIZE'] = size2binary4apu
            message2print = "{(Step 14J/70) set" + "; size-of-task4program2apu-set-by-create2programwithbinary4apu to: " + str(task4program2apu ['SIZE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #  folder2lib4apu +  __pycache__//apu2image4xi.cpython-33.pyc': 1006}
                            
            # ---------------------------------------------------------------------------------------------
            # (15) LANGUAGE - EXTRA                             
            # ---------------------------------------------------------------------------------------------             
            language2apu  = "Python"  # C++; C
           
            message2print = "{(Step 15B/70) set" + "; language2apu-set-by-create2programwithbinary4apu to: " + str(language2apu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # Python
            code2ack  = code2ack + '15B-' 
        
            task4program2apu ['LANGUAGE'] = language2apu     

                                    
            # ---------------------------------------------------------------------------------------------
            # (16) COUNT2REFERENCE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            count2reference2apu  = 1
           
            message2print = "{(Step 17B/70) set" + "; count2reference2apu-set-by-create2programwithbinary4apu to: " + str(count2reference2apu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1 
            code2ack  = code2ack + '17B-' 
        
            task4program2apu ['COUNT2REFERENCE'] = count2reference2apu                                 
                                                                                                                      
        else:
            pass                                                                                                    # PARAMETER DATA RECEIVED                       
    except: 
        message2print = "{(Step 1Z1/70) yielded" + "; task4program2apu-yielded-by-create2programwithbinary4apu: " + str(task4program2apu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '1Z1-' 

    # ---------------------------------------------------------------------------------------------
    # TASK EITHER GENERATED BY THIS PROGRAM (NONE GIVEN) OR GIVEN BY USER    
    # ---------------------------------------------------------------------------------------------
    keys2task = task4program2apu.keys()        
    message2print = "{(Step 19A/70) using" + "; keys2task-using-by-create2programwithbinary4apu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    # ['BUFFER', 'CODE' ... 
  
    
    # ---------------------------------------------------------------------------------------------
    # (Step 20A/70) - CREATE PROGRAM OBJECT FOR APU ! 
    # ---------------------------------------------------------------------------------------------    
    apu2program4xi = {}                                                                                                                                                                               
    message2print = "{(Step 20A/70) initialized" + "; apu2program4xi-initialized-by-create2programwithbinary4apu to: " + str(apu2program4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # {}

    if 'DATA2CALLBACK' in keys2task:    
        message2print = "{(Step 21A0/70) try" + "; data2callback-try-by-create2programwithbinary4apu: {}" + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)      
        # 
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 21A/70) - DATA2CALLBACK - EXTRA 
            # -----------------------------------------------------------------------------------------               
            apu2program4xi['DATA2CALLBACK'] = task4program2apu['DATA2CALLBACK']
            
            message2print = "{(Step 21A1/70) initialized" + "; data2callback-created-by-create2programwithbinary4apu: " + str(apu2program4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'MESSAGE': '{(Step 1Z1/33) failed; create2programwithbinary4apu-failed ; time: 02:37:08 11/17/11 Pacific Standard Time}', 'CLASS': 'create2programwithbinary4apu', 'METHOD': 'create2programwithbinary4apu'}; time: 02:37:08 11/17/11 Pacific Standard Time}
            
            code2ack  = code2ack + '21A1-'
        except: 
            message2print = "{(Step 21Z1/70) yielded" + "; data2callback-apu2program4xi-yielded-by-create2programwithbinary4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '21Z1-'

    if 'FUNCTION2CALLBACK' in keys2task:       
        message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-create2programwithbinary4apu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        # 
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 22B/70) - FUNCTION2CALLBACK - EXTRA
            # -----------------------------------------------------------------------------------------               
            apu2program4xi['FUNCTION2CALLBACK'] = task4program2apu['FUNCTION2CALLBACK']
            
            message2print = "{(Step 22B1/70) initialized" + "; function2callback-created-by-create2programwithbinary4apu: " + str(apu2program4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <function trace2program4apu at 0x00000000039912C8>
            
            code2ack  = code2ack + '22B1-'
        except: 
            message2print = "{(Step 22Z2/70) yielded" + "; function2callback-apu2program4xi-yielded-by-create2programwithbinary4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '22Z2-' 

    if 'PLATFORM' in keys2task:           
        message2print = "{(Step 22C0/70) try" + "; platform-try-by-create2programwithbinary4apu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        # 
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 22C/70) - PLATFORM - EXTRA
            # -----------------------------------------------------------------------------------------               
            apu2program4xi['PLATFORM'] = task4program2apu['PLATFORM']
            
            message2print = "{(Step 22C1/70) initialized" + "; platform-created-by-create2programwithbinary4apu: " + str(apu2program4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 02:37:08 11/17/11 Pacific Standard Time}
            
            code2ack  = code2ack + '22C1-'
        except: 
            message2print = "{(Step 22Z3/70) yielded" + "; platform-apu2program4xi-yielded-by-create2programwithbinary4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '22Z3-' 
        
        

    if 'CONTEXT' in keys2task:         
        message2print = "{(Step 23A/70) try" + "; context2apu-try-by-create2programwithbinary4apu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 23/70) - CONTEXT - REQUIRED
            # -----------------------------------------------------------------------------------------               
            apu2program4xi['CONTEXT'] = task4program2apu['CONTEXT']
            
            message2print = "{(Step 23B/70) initialized" + "; context-created-by-create2programwithbinary4apu: " + str(apu2program4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK' ...       
            
            code2ack  = code2ack + '23B-'
        except: 
            message2print = "{(Step 23Z1/70) yielded" + "; context-apu2program4xi-yielded-by-create2programwithbinary4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
        
            code2nack  = code2nack + '23Z1-' 

    if 'DEVICE' in keys2task:           
        message2print = "{(Step 24A/70) try" + "; device-try-by-create2programwithbinary4apu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 24/70) - DEVICE - EXTRA
            # -----------------------------------------------------------------------------------------               
            apu2program4xi['DEVICE'] = task4program2apu['DEVICE']
            
            message2print = "{(Step 24B/70) initialized" + "; device-created-by-create2programwithbinary4apu: " + str(apu2program4xi['DEVICE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'RESOLUTION2TIMER4PROFILING': 318 ... 
            
            code2ack  = code2ack + '24B-'
        except: 
            message2print = "{(Step 24Z1/70) yielded" + "; device-apu2program4xi-yielded-by-create2programwithbinary4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '24Z1-'

    if 'CODE2ERROR' in keys2task:                    
        message2print = "{(Step 25A/70) try" + "; code2error-try-by-create2programwithbinary4apu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 25/70) - ERROR CODE - REQUIRED
            # -----------------------------------------------------------------------------------------               
            apu2program4xi['CODE2ERROR'] = task4program2apu['CODE2ERROR']
            
            message2print = "{(Step 25B/70) initialized" + "; code2error-created-by-create2programwithbinary4apu: " + str(apu2program4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 0
            
            code2ack  = code2ack + '25B-'
        except: 
            message2print = "{(Step 25Z1/70) yielded" + "; code2error-apu2program4xi-yielded-by-create2programwithbinary4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '25Z1-'                
                

    if 'QUEUE' in keys2task:             
        message2print = "{(Step 26A/70) try" + "; queue-try-by-create2programwithbinary4apu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 26/70) - QUEUE - EXTRA
            # -----------------------------------------------------------------------------------------               
            apu2program4xi['QUEUE'] = task4program2apu['QUEUE']
            
            message2print = "{(Step 26B/70) initialized" + "; queue-created-by-create2programwithbinary4apu: " + str(apu2program4xi['QUEUE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <queue4xi.Queue4Xi object at 0x0000000003BEE630>
            
            code2ack  = code2ack + '26B-'
        except: 
            message2print = "{(Step 26Z1/70) yielded" + "; queue-apu2program4xi-yielded-by-create2programwithbinary4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '26Z1-' 

    if 'PIPE' in keys2task:                                   
        message2print = "{(Step 27A/70) try" + "; pipe-try-by-create2programwithbinary4apu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                     
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 27/70) - PIPE - EXTRA
            # -----------------------------------------------------------------------------------------               
            apu2program4xi['PIPE'] = task4program2apu['PIPE']
            
            message2print = "{(Step 27B/70) initialized" + "; pipe-created-by-create2programwithbinary4apu: " + str(apu2program4xi['PIPE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>)
            
            code2ack  = code2ack + '27B-'
        except: 
            message2print = "{(Step 27Z1/70) yielded" + "; pipe-apu2program4xi-yielded-by-create2programwithbinary4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '27Z1-'   

    if 'COMMANDQUEUE' in keys2task:   
        message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-create2programwithbinary4apu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
                            
            apu2program4xi['COMMANDQUEUE']  = task4program2apu['COMMANDQUEUE']  
     
            message2print = "{(Step 28B/70) initialized" + "; commandqueue-created-by-create2programwithbinary4apu: " + str(apu2program4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0 ...
                    
            code2ack  = code2ack + '28B-'
        except: 
            message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-create2programwithbinary4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                  
            code2nack  = code2nack + '28Z1-'    

    if 'BUFFER' in keys2task:   
        message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-create2programwithbinary4apu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
                            
            apu2program4xi['BUFFER']  = task4program2apu['BUFFER']  
     
            message2print = "{(Step 29B/70) initialized" + "; bufferinfo-created-by-create2programwithbinary4apu: " + str(apu2program4xi['BUFFER'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0 ... 
            
            
            code2ack  = code2ack + '29B-'
        except: 
            message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-create2programwithbinary4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '29Z1-'            

    if 'EVENT' in keys2task:   
        message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-create2programwithbinary4apu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
                            
            apu2program4xi['EVENT']  = task4program2apu['EVENT']  
     
            message2print = "{(Step 30B/70) initialized" + "; eventinfo-created-by-create2programwithbinary4apu: " + str(apu2program4xi['EVENT'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'DATA2CALLBACK': ...
            
            code2ack  = code2ack + '30B-'
        except: 
            message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-create2programwithbinary4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '30Z1-'            
                

    if 'COUNT4BINARY' in keys2task:   
        message2print = "{(Step 31A/70) try" + "; count-try-by-create2programwithbinary4apu: " + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
        try:            
            # ---------------------------------------------------------------------------------------------
            # (31) COUNT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            apu2program4xi['COUNT4BINARY']  = task4program2apu['COUNT']  
     
            message2print = "{(Step 31B/70) initialized" + "; count-created-by-create2programwithbinary4apu: " + str(apu2program4xi['COUNT4BINARY'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1 
            code2ack  = code2ack + '31B-'
        except: 
            message2print = "{(Step 31Z1/70) yielded" + "; count-yielded-by-create2programwithbinary4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '31Z1-'     

    if 'DIRECTORY4BINARY' in keys2task:   
        message2print = "{(Step 32A/70) try" + "; DIRECTORY-try-by-create2programwithbinary4apu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (32) DIRECTORY - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            apu2program4xi['DIRECTORY4BINARY']  = task4program2apu['DIRECTORY']  
     
            message2print = "{(Step 32B/70) initialized" + "; directory-created-by-create2programwithbinary4apu: " + str(apu2program4xi['DIRECTORY4BINARY'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #folder2lib4apu +  __pycache__//
            code2ack  = code2ack + '32B-'
        except: 
            message2print = "{(Step 32Z1/70) yielded" + "; DIRECTORY-yielded-by-create2programwithbinary4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '32Z1-'  

    if 'CODE4BINARY' in keys2task:               
        message2print = "{(Step 33A/70) try" + "; binary-try-by-create2programwithbinary4apu: None" + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)        
        apu2program4xi['CODE4BINARY'] = {}
        try:            
            # ---------------------------------------------------------------------------------------------
            # (33) CODE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            apu2program4xi['CODE4BINARY'] = task4program2apu ['CODE']
    
     
            message2print = "{(Step 33B/70) created" + "; code-created-by-create2programwithsource4apu: " + str(len(apu2program4xi['CODE4BINARY']))  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1
             
            code2ack  = code2ack + '33B-'
        except: 
            message2print = "{(Step 33Z1/70) yielded" + "; binary-yielded-by-create2programwithbinary4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '33Z1-'     

    if 'SIZE4BINARY' in keys2task:           
        message2print = "{(Step 34A/70) try" + "; size-try-by-create2programwithbinary4apu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (34) SIZE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            apu2program4xi['SIZE4BINARY']  = task4program2apu['SIZE']  
     
            message2print = "{(Step 34B/70) initialized" + "; size-initialized-by-create2programwithbinary4apu to: " + str(apu2program4xi['SIZE4BINARY'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {folder2lib4apu + '__pycache__//apu2image4xi.cpython-33.pyc': 1006}
            code2ack  = code2ack + '34B-'
        except: 
            message2print = "{(Step 34Z1/70) yielded" + "; size-yielded-by-create2programwithbinary4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '34Z1-'     

    if 'LANGUAGE4BINARY' in keys2task:           
        message2print = "{(Step 35A/70) try" + "; language-try-by-create2programwithbinary4apu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (35) LANGUAGE - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
                            
            apu2program4xi['LANGUAGE4BINARY']  = task4program2apu['LANGUAGE']  
     
            message2print = "{(Step 35B/70) initialized" + "; language-created-by-create2programwithbinary4apu: " + str(apu2program4xi['LANGUAGE4BINARY'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # Python
            code2ack  = code2ack + '35B-'
        except: 
            message2print = "{(Step 35Z1/70) yielded" + "; language-yielded-by-create2programwithbinary4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '35Z1-'     

    if 'COUNT2REFERENCE' in keys2task:                   
        message2print = "{(Step 36A/70) try" + "; count2reference-try-by-create2programwithbinary4apu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (36) COUNT2REFERENCE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
                            
            apu2program4xi['COUNT2REFERENCE']  = task4program2apu['COUNT2REFERENCE']  
     
            message2print = "{(Step 36B/70) initialized" + "; count2reference-created-by-create2programwithbinary4apu: " + str(apu2program4xi['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1 
            code2ack  = code2ack + '36B-'
        except: 
            message2print = "{(Step 36Z1/70) yielded" + "; count2reference-yielded-by-create2programwithbinary4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '36Z1-'     
        
                                                                                                               
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    apu2program4xi ['ACK2RETURN']  = code2ack
    apu2program4xi ['NACK2RETURN'] = code2nack  
    
    keys2program4apu = apu2program4xi.keys()
    for key2program4apu in keys2program4apu:
        message2print = "{(Step 70/70) returning" + "; key2program4apu-returning-by-create2programwithbinary4apu: " + str(key2program4apu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # CODE2ERROR ... CODE4BINARY ...
    
    return apu2program4xi
            
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "fetch2programsource4apu" usage = "
#   fetch2programsource4apu(file2walk)
#"> 
# -------------------------------------------------------------------------------------------------
def fetch2programsource4apu(program2explore4apu):
    global fetch2code4source                                                                                        # Fetched code dictionary
    
    global kb2fetch4apu                                                                                             # Knowledgebase for fetching
    global profile2fetch4source                                                                                     # Extension for files
    global keys2profile4source                                                                                      
    

    class2print  = "fetch2programsource4apu"
    method2print = "fetch2programsource4apu" 
     
    message2print = "{(Step 0A/33) passed" + "; program2explore4apu-passed-to-fetch2programsource4apu: " + str(program2explore4apu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    #folder2lib4apu +  apu2buffer4xi.py; apu2dummy4xi5test.cpp
    
    message2print = "{(Step 0B/33) using" + "; keys2profile4source-using-by-fetch2programsource4apu: " + str(keys2profile4source)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # ['.cl', '.c', '.py', '.cpp']
                  
    # ---------------------------------------------------------------------------------------------
    # SPLIT PROGRAM NAME WITH PATH AND EXTENSION TO TWO PIECES
    # ---------------------------------------------------------------------------------------------    
    name2program4source=os.path.splitext(program2explore4apu)

        
    if name2program4source[1] in keys2profile4source:
        try:          
            # -----------------------------------------------------------------------------------------
            # (Step 1/33) OF FETCH2SCOPE4XI - SOURCE PROGRAM
            # -----------------------------------------------------------------------------------------
            message2print = "{(Step 1A/33) analyzing" + "; program2explore4apu-analyzing-by-fetch2programsource4apu: " + str(program2explore4apu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            #folder2lib4apu +  apu2buffer4xi.py
                 
            path2input4source = program2explore4apu
            message2print = "{(Step 1B/33) set" + "; path2input4source-set-by-fetch2programsource4apu to: " + str(path2input4source)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            #folder2lib4apu +  apu2buffer4xi.py 
    
            # -----------------------------------------------------------------------------------------
            # (Step 2/33) OF FETCH2SCOPE4XI - CHECKING WHETHER PROGRAM MUST BE FETCHED
            # -----------------------------------------------------------------------------------------
            message2print = "{(Step 2A/33) analyzing" + "; path2input4source-analyzing-by-fetch2programsource4apu: " + str(path2input4source) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #folder2lib4apu +  apu2buffer4xi.py; apu2dummy4xi5test.cpp
                
            keys2fetch4source = kb2fetch4apu.keys()
            ack2match4source = False
            for key2fecth4source in keys2fetch4source:                                                              # 1,2 ...
                # -----------------------------------------------------------------------------------------
                # 
                # -----------------------------------------------------------------------------------------
                index2search=str.find(path2input4source,kb2fetch4apu[key2fecth4source])                             # kb2fetch4apu[1] = folder2lib4apu + 'apu2sampler4xi5test.py'
                if index2search == -1:                                         
                    # -----------------------------------------------------------------------------------------
                    # VALID FILE TYPE BUT NOT SELECTED TO BE FETCHED - NOT BE SKIPPED MIGHT GO THROUGH HERE TOO
                    # -----------------------------------------------------------------------------------------               
                    message2print = "{(Step 1E/33) skipped" + "; path2input4source-skipped-by-fetch2programsource4apu: " + str(path2input4source)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                      
                    #folder2lib4apu +  apu2buffer4xi.py 
    
                else:
                    ack2match4source = True
                    
                    message2print = "{(Step 2B/33) decided" + "; program-to-read-decided-by-fetch2programsource4apu: " + str(path2input4source) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)        
                    #folder2lib4apu +  apu2sampler4xi5test.py; apu2dummy4xi5test.cpp
    
                    # -----------------------------------------------------------------------------------------
                    # (Step 3/33) OF FETCH2SCOPE4XI - OPEN INPUT
                    # -----------------------------------------------------------------------------------------
                    file2input4source = open(path2input4source,'r')
                    message2print = "{(Step 3A/33) opened" + "; file2input4source-opened-by-fetch2programsource4apu: " + str(file2input4source) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)     
                    # <_io.TextIOWrapper name=folder2lib4apu + 'apu2sampler4xi5test.py' mode='r' encoding='cp1252'> ; apu2dummy4xi5test.cpp 
                    
                    # -----------------------------------------------------------------------------------------
                    # (Step 4/33) OF FETCH2SCOPE4XI - OUTPUT 
                    # -----------------------------------------------------------------------------------------
                    key2save4source = path2input4source
                    fetch2code4source[key2save4source] = {}                                                                # fetch2code4source['apu2sampler4xi5test.py'] = {}
                    number2line4source = 0  
                    message2print = "{(Step 3B/33) saving" + "; key2fecth4source-saving-by-fetch2programsource4apu: " + str(key2fecth4source) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)              
                    # 
             
                    while 1:
                        # -------------------------------------------------------------------------------------
                        # (Step 5/33) OF FETCH2SCOPE4XI - PROCESS FILE
                        # -------------------------------------------------------------------------------------             
                        line2process4source = file2input4source.readline()  
                        if not line2process4source:
                            # ---------------------------------------------------------------------------------
                            # (Step 30/33) OF FETCH2SCOPE4XI - ALL LINES READ, FINISH
                            # ---------------------------------------------------------------------------------             
                            break                                               
                        else:
                            # ---------------------------------------------------------------------------------
                            # (Step 7/33) OF FETCH2SCOPE4XI - SAVE LINE
                            # ---------------------------------------------------------------------------------
                            message2print = "{(Step 7A/33) read" + "; line2process4source-read-by-fetch2programsource4apu: " + str(line2process4source) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)              
                            # ... profile = "11" />
                            # ...
                            # # end of apu2sampler4xi5test.py 
                            # #/> 
                            try:
                                number2line4source = number2line4source + 1
                                fetch2code4source[key2save4source][number2line4source] = line2process4source
                            except:
                                message2print = "{(Step 7Z1/33) failed" + "; key2fecth4source-failed-by-fetch2programsource4apu: " + str(key2fecth4source) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)
                                
                                message2print = "{(Step 7Z2/33) failed" + "; line2process4source-failed-by-fetch2programsource4apu: " + str(line2process4source) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)
                                
                                message2print = "{(Step 7Z3/33) failed" + "; number2line4source-failed-by-fetch2programsource4apu: " + str(number2line4source) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)                            
                                                                 
                    # -----------------------------------------------------------------------------------------
                    # (Step 31/33) OF FETCH2SCOPE4XI - CLOSE INPUT
                    # -----------------------------------------------------------------------------------------            
                    file2input4source.close()
                    message2print = "{(Step 31A/33) closed" + "; file2input4source-closed-by-fetch2programsource4apu: " + str(file2input4source) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)           
                    # <_io.TextIOWrapper name=folder2lib4apu + 'apu2sampler4xi5test.py' mode='r' encoding='cp1252'>; apu2dummy4xi5test.cpp
                        
            
                    message2print = "{(Step 31B/33) saved" + "; fetch2code4source-saved-by-fetch2programsource4apu: " + str(fetch2code4source) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # {folder2lib4apu + 'apu2sampler4xi5test.py': {1: '... profile = "11" />\n', 2: '# -- ....
                     
                    # ---------------------------------------------------------------------------------------------
                    #
                    # ---------------------------------------------------------------------------------------------                                          
        except:
            message2print = "{(Step 1Z1/33) failed" + "; name2program4source-failed-by-fetch2programsource4apu: " + str(name2program4source)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
    else:
        try:
            # -----------------------------------------------------------------------------------------
            # INVALID FILE TYPE - NOT IN PROFILE
            # -----------------------------------------------------------------------------------------       
            message2print = "{(Step 1F/33) skipped" + "; name2program4source-skipped-by-fetch2programsource4apu: " + str(name2program4source)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                      
            # folder2lib4apu + '__pycache__//apu2buffer4xi.cpython-32', '.pyc')
            
            extension2skip = name2program4source[1]
            message2print = "{(Step 1G/33) skipped" + "; extension-skipped-by-fetch2programsource4apu: " + str(extension2skip[1:])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            # pyc
    
            message2print = "{(Step 1H/33) returning" + "; fetch2programsource4apu-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)   
        except:
            message2print = "{(Step 1Z2/33) yielded" + "; name2program4source-yielded-by-fetch2programsource4apu: " + str(name2program4source)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                    
    message2print = "{(Step 1H/33) returning" + "; fetch2programsource4apu-returning: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)            
    #   
                                            
#</function4xi>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "set2programwithsource4apu" usage = "
#
#   ... prepare task4set2programws ...    
#
#   response4set2program = {} 
#   print('(Step 17F/700) OF APU2PROGRAM4XI5TEST - TRYING TO SET PROGRAM WITH SOURCE: ', set2programwithsource4apu)   
#   # <function set2programwithsource4apu at 0x0000000003837948>
#   try:                               
#       response4set2pws = set2programwithsource4apu(task4set2programws)
#       print('(Step 17G/700) OF APU2PROGRAM4XI5TEST - SET PROGRAM SOURCE WITH SOURCE: ', set2programwithsource4apu)                                                                                                           
#   except:
#       print('(Step 17Z/700) OF APU2PROGRAM4XI5TEST *** FAILED TO SET PROGRAM SOURCE WITH SOURCE: ', set2programwithsource4apu)  
#   try:                                                                                          
#       keys2response = response4set2pws.keys()
#       print('COUNT2REFERENCE:', response4set2pws ['COUNT2REFERENCE'])   
#       print('COUNT4SOURCE:', response4set2pws ['COUNT4SOURCE'])
#       print('DIRECTORY4SOURCE:', response4set2pws ['DIRECTORY4SOURCE']) 
#       print('FUNCTION2CALLBACK:', response4set2pws ['FUNCTION2CALLBACK'])   
#       print('LANGUAGE4SOURCE:', response4set2pws ['LANGUAGE4SOURCE'])
#       print('SIZE4SOURCE:', response4set2pws ['SIZE4SOURCE'])                
#        keys2source = response4set2pws['CODE4SOURCE'].keys()
#       for key2source in keys2source:
#           print('SOURCE: ', key2source)
#           print(response4set2pws ['CODE4SOURCE'] [key2source] [1])
#           #
#           # COUNT2REFERENCE: 1
#           # COUNT4SOURCE: 1
#           # DIRECTORY4SOURCE: OK
#           # FUNCTION2CALLBACK: <function trace2program4apu at 0x0000000003A28B48>
#           # LANGUAGE4SOURCE: Python
#           # SIZE4SOURCE: 1
#           # SOURCE:  ... apu2image4xi5test.py
#           # ... profile = "11" />                  
#   except:
#       print('(Step 3Z2/700) OF APU2PROGRAM4XI5TEST *** FAILED TO PRINT response4set2pws : ', response4set2pws)                                                   
#
#">
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------   
def set2programwithsource4apu(task4programset2apu=None):

    '''
    sets programs for APU
    '''
    global apu2program4xi  

    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "set2programwithsource4apu"
    method2print = "set2programwithsource4apu" 

    message2print = "{(Step 1A/33) started" + "; set2programwithsource4apu-started: " + str(set2programwithsource4apu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function set2programwithsource4apu at 0x0000000003837948> 
    
    keys2task = task4programset2apu.keys()
    for key2task in sorted(keys2task):
        message2print = "{(Step 1B/33) passed" + "; key2task-passed-to-set2programwithsource4apu: " + str(key2task) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
        # CODE2ERROR
        # CONTEXT 
        # COUNT4SOURCE
        # DEVICE
        # DIRECTORY4SOURCE
        # LANGUAGE4SOURCE
        # PROPERTIES
        # QUEUE
          
    code2ack  = ' '
    code2nack = ' '
    # -----------------------------------------------------------------------------------------
    # CREATE IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if apu2program4xi is None:
            apu2program4xi = create2programwithsource4apu()                                                                              # INITIALIZE PROGRAM FOR APU 
            message2print = "{(Step 1C/70) set" + "; apu2program4xi-set-by-set2programwithsource4apu to: " + str(apu2program4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2programwithsource4apu-skipped-by-set2programwithsource4apu: " + str(create2programwithsource4apu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                 
    except:
        try:
            apu2program4xi = create2programwithsource4apu()
            message2print = "{(Step 1E/70) set" + "; apu2program4xi-set-by-set2programwithsource4apu to: " + str(apu2program4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            apu2program4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; apu2program4xi-yielded-by-set2programwithsource4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
                
    # ---------------------------------------------------------------------------------------------
    # SET       
    # ---------------------------------------------------------------------------------------------     
    message2print = "{(Step 22A0/70) try" + "; data2callback-try-by-set2programwithsource4apu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    #     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 21A/70) - DATA2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        apu2program4xi['DATA2CALLBACK'] = task4programset2apu['DATA2CALLBACK']
                                        
        message2print = "{(Step 21A1/70) set" + "; data2callback-set-by-set2programwithsource4apu to: " + str(apu2program4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  
        
        code2ack  = code2ack + '21A1-'
    except: 
        message2print = "{(Step 21Z1/70) yielded" + "; data2callback-apu2program4xi-yielded-by-set2programwithsource4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '21Z1-'
    
    message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-set2programwithsource4apu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22B/70) - FUNCTION2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        apu2program4xi['FUNCTION2CALLBACK'] = task4programset2apu['FUNCTION2CALLBACK']
        
        message2print = "{(Step 22B1/70) set" + "; function2callback-set-by-set2programwithsource4apu to: " + str(apu2program4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  
        code2ack  = code2ack + '22B1-'
    except: 
        message2print = "{(Step 22Z2/70) yielded" + "; function2callback-apu2program4xi-yielded-by-set2programwithsource4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        code2nack  = code2nack + '22Z2-' 
    
    message2print = "{(Step 22C0/70) try" + "; platform2callback-try-by-set2programwithsource4apu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22C/70) - PLATFORM - EXTRA
        # -----------------------------------------------------------------------------------------               
        apu2program4xi['PLATFORM'] = task4programset2apu['PLATFORM']
        
        message2print = "{(Step 2C1/70) set" + "; platform-set-by-set2programwithsource4apu to: " + str(apu2program4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #   
        code2ack  = code2ack + '2C1-'
    except: 
        message2print = "{(Step 22Z3/70) yielded" + "; platform-apu2program4xi-yielded-by-set2programwithsource4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '22Z3-' 
        
        
      
    message2print = "{(Step 23A/70) try" + "; context2apu-try-by-set2programwithsource4apu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)       
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 23/70) - CONTEXT - REQUIRED
        # -----------------------------------------------------------------------------------------               
        apu2program4xi['CONTEXT'] = task4programset2apu['CONTEXT']
        
        message2print = "{(Step 23B/70) set" + "; context-set-by-set2programwithsource4apu to: " + str(apu2program4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        code2ack  = code2ack + '23B-'
    except: 
        message2print = "{(Step 23Z1/70) yielded" + "; context-apu2program4xi-yielded-by-set2programwithsource4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
    
        code2nack  = code2nack + '23Z1-' 
        
    message2print = "{(Step 24A/70) try" + "; device-try-by-set2programwithsource4apu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 24/70) - DEVICE - EXTRA
        # -----------------------------------------------------------------------------------------               
        apu2program4xi['DEVICE'] = task4programset2apu['DEVICE']
        
        message2print = "{(Step 24B/70) set" + "; device-set-by-set2programwithsource4apu to: " + str(apu2program4xi['DEVICE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        
        code2ack  = code2ack + '24B-'
    except: 
        message2print = "{(Step 24Z1/70) yielded" + "; device-apu2program4xi-yielded-by-set2programwithsource4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '24Z1-'
                 
    message2print = "{(Step 25A/70) try" + "; code2error-try-by-set2programwithsource4apu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 25/70) - ERROR CODE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        apu2program4xi['CODE2ERROR'] = task4programset2apu['CODE2ERROR']
        
        message2print = "{(Step 25B/70) set" + "; code2error-set-by-set2programwithsource4apu to: " + str(apu2program4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        code2ack  = code2ack + '25B-'
    except: 
        message2print = "{(Step 25Z1/70) yielded" + "; code2error-apu2program4xi-yielded-by-set2programwithsource4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '25Z1-'                
                
          
    message2print = "{(Step 26A/70) try" + "; queue-try-by-set2programwithsource4apu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 26/70) - QUEUE - EXTRA
        # -----------------------------------------------------------------------------------------               
        apu2program4xi['QUEUE'] = task4programset2apu['QUEUE']
        
        message2print = "{(Step 26B/70) set" + "; queue-set-by-set2programwithsource4apu to: " + str(apu2program4xi['QUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
         
        code2ack  = code2ack + '26B-'
    except: 
        message2print = "{(Step 26Z1/70) yielded" + "; queue-apu2program4xi-yielded-by-set2programwithsource4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '26Z1-' 
                                
    message2print = "{(Step 27A/70) try" + "; pipe-try-by-set2programwithsource4apu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 27/70) - PIPE - EXTRA
        # -----------------------------------------------------------------------------------------               
        apu2program4xi['PIPE'] = task4programset2apu['PIPE']
        
        message2print = "{(Step 27B/70) set" + "; pipe-set-by-set2programwithsource4apu to: " + str(apu2program4xi['PIPE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        
        
        code2ack  = code2ack + '27B-'
    except: 
        message2print = "{(Step 27Z1/70) yielded" + "; pipe-apu2program4xi-yielded-by-set2programwithsource4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '27Z1-'   

    message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-set2programwithsource4apu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        apu2program4xi['COMMANDQUEUE']  = task4programset2apu['COMMANDQUEUE']  
 
        message2print = "{(Step 28B/70) set" + "; commandqueue-set-by-set2programwithsource4apu to: " + str(apu2program4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        
        code2ack  = code2ack + '28B-'
    except: 
        message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-set2programwithsource4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '28Z1-'    

    message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-set2programwithsource4apu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (29) BUFFER INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
        apu2program4xi['BUFFER']  = task4programset2apu['BUFFER']  
 
        message2print = "{(Step 29B/70) set" + "; bufferinfo-set-by-set2programwithsource4apu to: " + str(apu2program4xi['BUFFER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '29B-'
    except: 
        message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-set2programwithsource4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        code2nack  = code2nack + '29Z1-'            

    message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-set2programwithsource4apu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) EVENT INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        apu2program4xi['EVENT']  = task4programset2apu['EVENT']  
 
        message2print = "{(Step 30B/70) set" + "; eventinfo-set-by-set2programwithsource4apu to: " + str(apu2program4xi['EVENT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '30B-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-set2programwithsource4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '30Z1-'            
                
    message2print = "{(Step 31A/70) try" + "; count-try-by-set2programwithsource4apu: None" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31)  - COUNT                           
        # ---------------------------------------------------------------------------------------------             
        apu2program4xi['COUNT4SOURCE']  = task4programset2apu['COUNT4SOURCE']  
 
        message2print = "{(Step 31B/70) set" + "; count-set-by-set2programwithsource4apu to: " + str(apu2program4xi['COUNT4SOURCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 1
        code2ack  = code2ack + '31B-'
    except: 
        message2print = "{(Step 31Z1/70) yielded" + "; count-yielded-by-set2programwithsource4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '31Z1-'  

    message2print = "{(Step 32A/70) try" + "; directory-try-by-set2programwithsource4apu: " + str(task4programset2apu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print) 
    # {'CODE2ERROR': 0 ... 'DIRECTORY4SOURCE': '...//a' ... 

    try:            
        # ---------------------------------------------------------------------------------------------
        # (32) DIRECTORY - REQUIRED                     
        # ---------------------------------------------------------------------------------------------                                    
        apu2program4xi['DIRECTORY4SOURCE']  = task4programset2apu['DIRECTORY4SOURCE'] 
        
        message2print = "{(Step 32B/70) set" + "; directory-set-by-set2programwithdirectory4apu to: " + str(apu2program4xi['DIRECTORY4SOURCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # OK
        code2ack  = code2ack + '32B-'
    except: 
        message2print = "{(Step 32Z1/70) yielded" + "; directory-yielded-by-set2programwithdirectory4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '32Z1-' 
        
    message2print = "{(Step 33A/70) try" + "; code-try-by-set2programwithsource4apu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (33) CODE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        apu2program4xi['CODE4SOURCE']  = task4programset2apu['CODE4SOURCE']  
 
        message2print = "{(Step 33B/70) set" + "; code-set-by-set2programwithsource4apu to: " + str(apu2program4xi['CODE4SOURCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {folder2lib4apu + 'apu2image4xi5test.py': 
        #  {1: '... profile = "11" />\n', 
        #   2: '... program = "apu2image4xi5test.py">\n'}
 
        code2ack  = code2ack + '33B-'
    except: 
        message2print = "{(Step 33Z1/70) yielded" + "; code-yielded-by-set2programwithsource4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '33Z1-'             
        
    message2print = "{(Step 34A/70) try" + "; size-try-by-set2programwithsource4apu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34) SIZE - REQUIRED                         
        # ---------------------------------------------------------------------------------------------             
                        
        apu2program4xi['SIZE4SOURCE']  = task4programset2apu['SIZE4SOURCE']  
 
        message2print = "{(Step 34B/70) set" + "; size-set-by-set2programwithsource4apu to: " + str(apu2program4xi['SIZE4SOURCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '34B-'
    except: 
        message2print = "{(Step 34Z1/70) yielded" + "; size-yielded-by-set2programwithsource4apu: " + str(task4programset2apu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '34Z1-'    
                
    message2print = "{(Step 35A/70) try" + "; language-try-by-set2programwithsource4apu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) LANGUAGE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        apu2program4xi['LANGUAGE4SOURCE']  = task4programset2apu['LANGUAGE4SOURCE']  
 
        message2print = "{(Step 35B/70) set" + "; language-set-by-set2programwithsource4apu to: " + str(apu2program4xi['LANGUAGE4SOURCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '35B-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; language-yielded-by-set2programwithsource4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '35Z1-'          
                        
    message2print = "{(Step 36A/70) try" + "; count2reference-try-by-set2programwithsource4apu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (36) COUNT2REFERENCE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        apu2program4xi['COUNT2REFERENCE']  = task4programset2apu['COUNT2REFERENCE']  
 
        message2print = "{(Step 36B/70) set" + "; count2reference-set-by-set2programwithsource4apu to: " + str(apu2program4xi['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '36B-'
    except: 
        message2print = "{(Step 36Z1/70) yielded" + "; count2reference-yielded-by-set2programwithsource4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '36Z1-'                           

    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    apu2program4xi ['ACK2RETURN']  = code2ack
    apu2program4xi ['NACK2RETURN'] = code2nack 
    
     
    keys2program4apu = apu2program4xi.keys()
    for key2program4apu in keys2program4apu:
        message2print = "{(Step 70/70) returning" + "; key2program4apu-returning-by-set2programwithsource4apu: " + str(key2program4apu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # COUNT4SOURCE ...

    return apu2program4xi

           
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "fetch2programbinary4apu" usage = "
#   fetch2programbinary4apu(file2walk)
#"> 
# -------------------------------------------------------------------------------------------------
def fetch2programbinary4apu(program2explore4apu):
    global fetch2code4binary                                                                                        # Fetched code dictionary
    
    global kb2fetch4apu                                                                                             # Knowledgebase for fetching

    global keys2profile4binary         

        
    class2print  = "fetch2programbinary4apu"
    method2print = "fetch2programbinary4apu" 
 
    message2print = "{(Step 1/8) got" + "; program2explore4apu-got-by-fetch2programbinary4apu: " + str(program2explore4apu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    #folder2lib4apu +  __pycache__//apu2buffer4xi.cpython-33.pyc
               
    # ---------------------------------------------------------------------------------------------
    # SPLIT PROGRAM NAME WITH PATH AND EXTENSION TO TWO PIECES
    # ---------------------------------------------------------------------------------------------    
    name2program4binary=os.path.splitext(program2explore4apu)
    
    if name2program4binary[1] in keys2profile4binary:          
        # -----------------------------------------------------------------------------------------
        # (Step 1/33) OF FETCH2SCOPE4XI - PYTHON BINARY
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 1A/33) analyzing" + "; program2explore4apu-analyzing-by-fetch2programbinary4apu: " + str(program2explore4apu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
        #folder2lib4apu +  __pycache__//apu2buffer4xi.cpython-33.pyc 
        
        path2input4binary = program2explore4apu                                                                          
        message2print = "{(Step 1B/33) set" + "; path2input4binary-set-by-fetch2programbinary4apu to: " + str(path2input4binary)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        #folder2lib4apu +  __pycache__//apu2buffer4xi.cpython-33.pyc
        
        # 
        # -----------------------------------------------------------------------------------------
        # (Step 2/33) OF FETCH2SCOPE4XI - CHECKING WHETHER PROGRAM MUST BE FETCHED 
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A1/33) analyzing" + "; path2input4binary-analyzing-by-fetch2programbinary4apu: " + str(path2input4binary) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #folder2lib4apu +  __pycache__//apu2buffer4xi.cpython-33.pyc 
        # ...
        #folder2lib4apu +  __pycache__//apu2sampler4xi.cpython-33.pyc
        keys2fetch4binary = kb2fetch4apu.keys()
        ack2match4binary = False
        for key2fecth4binary in keys2fetch4binary:                                                                  # 
            string2search4binary = kb2fetch4apu[key2fecth4binary]                                                   # apu2sampler4xi.pyc
            index2search=str.find(path2input4binary,string2search4binary)                                           # E:\Scope4xi\041721\zap\lib\a\__pycache__\apu2sampler4xi.cpython-33.pyc
            if index2search == -1:                                         
                pass                                                                                                # not ordered to be fetched
            else:
                ack2match4binary = True                                                                             # E:\Scope4xi\041721\zap\lib\a\__pycache__\apu2sampler4xi.cpython-33.pyc
                    
        if ack2match4binary is True:
            message2print = "{(Step 2B/33) decided" + "; program-to-read-decided-by-fetch2programbinary4apu: " + str(path2input4binary) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)        
            # folder2lib4apu +  __pycache__//apu2image4xi.cpython-33.pyc


            message2print = "{(Step 2B/33) opening" + "; path2input4binary-opening-by-fetch2programbinary4apu: " + str(path2input4binary) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                 
            # folder2lib4apu +  __pycache__//apu2image4xi.cpython-33.pyc 
            try:
                # -----------------------------------------------------------------------------------------
                # (Step 3/33) OF FETCH2SCOPE4XI - OPEN BINARY INPUT
                # -----------------------------------------------------------------------------------------
                file2input4binary = open(path2input4binary,'rb')
                message2print = "{(Step 3A/33) opened" + "; file2input4binary-opened-by-fetch2programbinary4apu: " + str(file2input4binary) + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)     
                # <_io.BufferedReader name=folder2lib4apu + '__pycache__//apu2image4xi.cpython-33.pyc'>
            
                try:
                    # -----------------------------------------------------------------------------------------
                    # (Step 4/33) OF FETCH2SCOPE4XI - OUTPUT
                    # -----------------------------------------------------------------------------------------
                    key2save4binary = path2input4binary
                    fetch2code4binary[key2save4binary] = {}                                                                # 
                    number2line4binary = 0  
                    message2print = "{(Step 3B/33) saving" + "; key2save4binary-saving-by-fetch2programbinary4apu: " + str(key2save4binary) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)              
                    # folder2lib4apu +  __pycache__//apu2sampler4xi.cpython-33.pyc
        
                    while 1:
                        # -------------------------------------------------------------------------------------
                        # (Step 5/33) OF FETCH2SCOPE4XI - PROCESS FILE
                        # -------------------------------------------------------------------------------------             
                        line2process4binary = file2input4binary.readline()  
                        if not line2process4binary:
                            # ---------------------------------------------------------------------------------
                            # (Step 30/33) OF FETCH2SCOPE4XI - ALL LINES READ, FINISH
                            # ---------------------------------------------------------------------------------             
                            break                                               
                        else:
                            try:
                                # ---------------------------------------------------------------------------------
                                # (Step 7/33) OF FETCH2SCOPE4XI - SAVE LINE
                                # ---------------------------------------------------------------------------------
                                number2line4binary = number2line4binary + 1
                                # message2print = "{(Step 7A/33) reading" + "; number2line4binary-reading-by-fetch2programbinary4apu: " + str(number2line4binary) + "; time: " + time4xi() + "}"
                                # print4xi(class2print, method2print, message2print)              
                                # 1,2,3 .... 1006
                                                
                                fetch2code4binary[key2save4binary][number2line4binary] = line2process4binary
                            except:
                                message2print = "{(Step 7Z1/33) failed" + "; key2fecth4binary-failed-by-fetch2programbinary4apu: " + str(key2fecth4binary) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)
                                
                                message2print = "{(Step 7Z2/33) failed" + "; line2process4binary-failed-by-fetch2programbinary4apu: " + str(line2process4binary) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)
                                
                                message2print = "{(Step 7Z3/33) failed" + "; number2line4binary-failed-by-fetch2programbinary4apu: " + str(number2line4binary) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)        
                except:        
                    message2print = "{(Step 7Z4/33) failed" + "; number2line4binary-failed-by-fetch2programbinary4apu: " + str(number2line4binary) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 52           
                finally:
                    # -----------------------------------------------------------------------------------------
                    # (Step 31/33) OF FETCH2SCOPE4XI - CLOSE INPUT
                    # -----------------------------------------------------------------------------------------            
                    file2input4binary.close()
                    message2print = "{(Step 31A/33) closed" + "; file2input4binary-closed-by-fetch2programbinary4apu: " + str(file2input4binary) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)           
                    # <_io.BufferedReader name=folder2lib4apu + '__pycache__//apu2image4xi.cpython-33.pyc'>
            except:
                message2print = "{(Step 7Z4/33) failed" + "; opening-failed-by-fetch2programbinary4apu: " + str(path2input4binary) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                  

        else:
            pass
    else:
        # -----------------------------------------------------------------------------------------
        # INVALID TYPE - NOT IN PROFILE
        # -----------------------------------------------------------------------------------------       
        message2print = "{(Step 1F/33) skipped" + "; name2program4binary-skipped-by-fetch2programbinary4apu: " + str(name2program4binary)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                      
        # 
        
        extension2skip = name2program4binary[1]
        message2print = "{(Step 1G/33) skipped" + "; extension-skipped-by-fetch2programbinary4apu: " + str(extension2skip[1:])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # 

        
    message2print = "{(Step 33/33) returning" + "; fetch2programbinary4apu-returning: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # 02:24:36 11/17/11 Pacific Standard Time
                             
#</function4xi>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "set2programwithbinary4apu" usage = "
#    
#    print('(Step 4B/33) OF APU2PROGRAM4XI5TEST - TRYING TO GET PLATFORMIDS4XI', time4xi())
#    platform2xi = {}
#    try:
#        platform2xi = get2platformids4xi()
#        print('(Step 4B/33) of CONTEXT4APU5TEST - GOT PLATFORMIDS4XI', platform2xi) 
#    except:
#        print('(Step 4Z/33) of CONTEXT4APU5TEST *** FAILED TO GET PLATFORMIDS4XI', time4xi())     
#    
# 
#    ... other data for set ...
#    
#    response4set2program = {} 
#    print('(Step 37F/700) OF APU2PROGRAM4XI5TEST - TRYING TO PERFORM SET2PROGRAM4APU: ', set2programwithbinary4apu)   
#    # 
#    try:                               
#        response4set2program = set2programwithbinary4apu(task4set2programwithbinary)                                                                                                         
#
#        keys2response = response4set2program.keys()
#        for key2response in sorted(keys2response):
#            print(key2response, ' : ', response4set2program[key2response])
#            # CODE4BINARY  :  {folder2lib4apu + '__pycache__//apu3image4xi.cpython-33.pyc': {1: b'l\x0c\r\n', 2: b'\x00Z\n'}}
#            # CODE4SOURCE  :  {folder2lib4apu + 'apu2image4xi5test.py': {1: '... profile = "11" />\n', 2: '... program = "apu2image4xi5test.py">\n'}}
#            # COUNT4BINARY  :  1
#            # COUNT4SOURCE  :  1
#            # LANGUAGE4BINARY  :  Python
#            # LANGUAGE4SOURCE  :  Python
#            # SIZE4BINARY  :  {folder2lib4apu + '__pycache__//apu2image4xi.cpython-33.pyc': 1006}
#                          
#    except:
#        print('(Step 37Z/700) OF APU2PROGRAM4XI5TEST *** FAILED TO PERFORM SET2PROGRAM4APU: ', set2programwithbinary4apu)                                            
#
#">
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------   
def set2programwithbinary4apu(task4programsetwb2apu=None):

    '''
    sets programs for APU
    '''
    global apu2program4xi  
    
      
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "set2programwithbinary4apu"
    method2print = "set2programwithbinary4apu"

    message2print = "{(Step 1A/33) started" + "; set2programwithbinary4apu-started: " + str(set2programwithbinary4apu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function set2programwithbinary4apu at 0x0000000003B43AC8>
    
    message2print = "{(Step 1B/33) passed" + "; task4programsetwb2apu-to-set2programwithbinary4apu: " + str(task4programsetwb2apu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'CODE2ERROR': 0, 
    #  'DIRECTORY4BINARY': ...//__pycache__//', 
    #  'LANGUAGE4BINARY': 'Python', 
    #  'SIZE4BINARY': 1, 
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003B19048>, 
    #  'CODE4BINARY': {'...//__pycache__//apu3image4xi.cpython-33.pyc': {1: b'l\x0c\r\n', 2: b'\x00Z\n'}}, 
    #   ...


    code2ack  = ' '
    code2nack = ' '
    # -----------------------------------------------------------------------------------------
    # CREATE IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if apu2program4xi is None:
            apu2program4xi = create2programwithbinary4apu()                                                                              # INITIALIZE PROGRAM FOR APU 
            message2print = "{(Step 1C/70) set" + "; apu2program4xi-set-by-set2programwithbinary4apu to: " + str(apu2program4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2programwithbinary4apu-skipped-by-set2programwithbinary4apu: " + str(create2programwithbinary4apu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                 
    except:
        try:
            apu2program4xi = create2programwithbinary4apu()
            message2print = "{(Step 1E/70) set" + "; apu2program4xi-set-by-set2programwithbinary4apu to: " + str(apu2program4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            apu2program4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; apu2program4xi-yielded-by-set2programwithbinary4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
                
    # ---------------------------------------------------------------------------------------------
    # SET            
    # ---------------------------------------------------------------------------------------------     
    message2print = "{(Step 22A0/70) try" + "; data2callback-try-by-set2programwithbinary4apu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    #     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 21A/70) - DATA2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        apu2program4xi['DATA2CALLBACK'] = task4programsetwb2apu['DATA2CALLBACK']
                                        
        message2print = "{(Step 21A1/70) set" + "; data2callback-set-by-set2programwithbinary4apu to: " + str(apu2program4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  
        
        code2ack  = code2ack + '21A1-'
    except: 
        message2print = "{(Step 21Z1/70) yielded" + "; data2callback-apu2program4xi-yielded-by-set2programwithbinary4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        code2nack  = code2nack + '21Z1-'
    
    message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-set2programwithbinary4apu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22B/70) - FUNCTION2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        apu2program4xi['FUNCTION2CALLBACK'] = task4programsetwb2apu['FUNCTION2CALLBACK']
        
        message2print = "{(Step 22B1/70) set" + "; function2callback-set-by-set2programwithbinary4apu to: " + str(apu2program4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  
        code2ack  = code2ack + '22B1-'
    except: 
        message2print = "{(Step 22Z2/70) yielded" + "; function2callback-apu2program4xi-yielded-by-set2programwithbinary4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        code2nack  = code2nack + '22Z2-' 
    
    message2print = "{(Step 22C0/70) try" + "; platform2callback-try-by-set2programwithbinary4apu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22C/70) - PLATFORM - EXTRA
        # -----------------------------------------------------------------------------------------               
        apu2program4xi['PLATFORM'] = task4programsetwb2apu['PLATFORM']
        
        message2print = "{(Step 2C1/70) set" + "; platform-set-by-set2programwithbinary4apu to: " + str(apu2program4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #   
        code2ack  = code2ack + '2C1-'
    except: 
        message2print = "{(Step 22Z3/70) yielded" + "; platform-apu2program4xi-yielded-by-set2programwithbinary4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        code2nack  = code2nack + '22Z3-' 
        
        
      
    message2print = "{(Step 23A/70) try" + "; context2apu-try-by-set2programwithbinary4apu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)       
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 23/70) - CONTEXT - REQUIRED
        # -----------------------------------------------------------------------------------------               
        apu2program4xi['CONTEXT'] = task4programsetwb2apu['CONTEXT']
        
        message2print = "{(Step 23B/70) set" + "; context-set-by-set2programwithbinary4apu to: " + str(apu2program4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': ...
        
        code2ack  = code2ack + '23B-'
    except: 
        message2print = "{(Step 23Z1/70) yielded" + "; context-apu2program4xi-yielded-by-set2programwithbinary4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
    
        code2nack  = code2nack + '23Z1-' 
        
    message2print = "{(Step 24A/70) try" + "; device-try-by-set2programwithbinary4apu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 24/70) - DEVICE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        apu2program4xi['DEVICE'] = task4programsetwb2apu['DEVICE']
        
        message2print = "{(Step 24B/70) set" + "; device-set-by-set2programwithbinary4apu to: " + str(apu2program4xi['DEVICE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'RESOLUTION2TIMER4PROFILING': 318 ... 
        
        
        code2ack  = code2ack + '24B-'
    except: 
        message2print = "{(Step 24Z1/70) yielded" + "; device-apu2program4xi-yielded-by-set2programwithbinary4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '24Z1-'
                 
    message2print = "{(Step 25A/70) try" + "; code2error-try-by-set2programwithbinary4apu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 25/70) - ERROR CODE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        apu2program4xi['CODE2ERROR'] = task4programsetwb2apu['CODE2ERROR']
        
        message2print = "{(Step 25B/70) set" + "; code2error-set-by-set2programwithbinary4apu to: " + str(apu2program4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        
        code2ack  = code2ack + '25B-'
    except: 
        message2print = "{(Step 25Z1/70) yielded" + "; code2error-apu2program4xi-yielded-by-set2programwithbinary4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '25Z1-'                
                
          
    message2print = "{(Step 26A/70) try" + "; queue-try-by-set2programwithbinary4apu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 26/70) - QUEUE - EXTRA
        # -----------------------------------------------------------------------------------------               
        apu2program4xi['QUEUE'] = task4programsetwb2apu['QUEUE']
        
        message2print = "{(Step 26B/70) set" + "; queue-set-by-set2programwithbinary4apu to: " + str(apu2program4xi['QUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <queue4xi.Queue4Xi object at 0x0000000003DF3668>
         
        code2ack  = code2ack + '26B-'
    except: 
        message2print = "{(Step 26Z1/70) yielded" + "; queue-apu2program4xi-yielded-by-set2programwithbinary4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '26Z1-' 
                                
    message2print = "{(Step 27A/70) try" + "; pipe-try-by-set2programwithbinary4apu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 27/70) - PIPE - EXTRA
        # -----------------------------------------------------------------------------------------               
        apu2program4xi['PIPE'] = task4programsetwb2apu['PIPE']
        
        message2print = "{(Step 27B/70) set" + "; pipe-set-by-set2programwithbinary4apu to: " + str(apu2program4xi['PIPE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        
        
        code2ack  = code2ack + '27B-'
    except: 
        message2print = "{(Step 27Z1/70) yielded" + "; pipe-apu2program4xi-yielded-by-set2programwithbinary4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '27Z1-'   

    message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-set2programwithbinary4apu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        apu2program4xi['COMMANDQUEUE']  = task4programsetwb2apu['COMMANDQUEUE']  
 
        message2print = "{(Step 28B/70) set" + "; commandqueue-set-by-set2programwithbinary4apu to: " + str(apu2program4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        
        code2ack  = code2ack + '28B-'
    except: 
        message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-set2programwithbinary4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '28Z1-'    

    message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-set2programwithbinary4apu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (29) BUFFER INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        apu2program4xi['BUFFER']  = task4programsetwb2apu['BUFFER']  
 
        message2print = "{(Step 29B/70) set" + "; bufferinfo-set-by-set2programwithbinary4apu to: " + str(apu2program4xi['BUFFER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '29B-'
    except: 
        message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-set2programwithbinary4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        code2nack  = code2nack + '29Z1-'            

    message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-set2programwithbinary4apu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) EVENT INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        apu2program4xi['EVENT']  = task4programsetwb2apu['EVENT']  
 
        message2print = "{(Step 30B/70) set" + "; eventinfo-set-by-set2programwithbinary4apu to: " + str(apu2program4xi['EVENT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '30B-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-set2programwithbinary4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '30Z1-'            
                
    message2print = "{(Step 31A/70) try" + "; count-try-by-set2programwithbinary4apu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31) COUNT - REQUIRED                           
        # ---------------------------------------------------------------------------------------------             
                        
        apu2program4xi['COUNT4BINARY']  = task4programsetwb2apu['COUNT4BINARY']  
 
        message2print = "{(Step 31B/70) set" + "; count-set-by-set2programwithbinary4apu to: " + str(apu2program4xi['COUNT4BINARY'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 1
        code2ack  = code2ack + '31B-'
    except: 
        message2print = "{(Step 31Z1/70) yielded" + "; count-yielded-by-set2programwithbinary4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '31Z1-'
          
  
    message2print = "{(Step 32A/70) try" + "; directory2binary-try-by-set2programwithdirectory2binary4apu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (32) DIRECTORY4BINARY - REQUIRED                               
        # ---------------------------------------------------------------------------------------------             
                        
        apu2program4xi['DIRECTORY4BINARY']  = task4programsetwb2apu['DIRECTORY4BINARY']  
 
        message2print = "{(Step 32B/70) set" + "; directory2binary-set-by-set2programwithdirectory2binary4apu to: " + str(apu2program4xi['DIRECTORY4BINARY'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # ...//__pycache__//

        code2ack  = code2ack + '32B-'
    except: 
        message2print = "{(Step 32Z1/70) yielded" + "; directory2binary-yielded-by-set2programwithdirectory2binary4apu: " + str(directory2binary) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '32Z1-'     
    
    message2print = "{(Step 33A/70) try" + "; binary-try-by-set2programwithbinary4apu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (33) CODE4BINARY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        apu2program4xi['CODE4BINARY']  = task4programsetwb2apu['CODE4BINARY']  
 
        message2print = "{(Step 33B/70) set" + "; binary-set-by-set2programwithbinary4apu to: " + str(apu2program4xi['CODE4BINARY'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {folder2lib4apu + '__pycache__//apu3image4xi.cpython-33.pyc': {1: b'l\x0c\r\n', 2: b'\x00Z\n'}}

        code2ack  = code2ack + '33B-'
    except: 
        message2print = "{(Step 33Z1/70) yielded" + "; binary-yielded-by-set2programwithbinary4apu: " + str(binary) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '33Z1-' 
        
    message2print = "{(Step 34A/70) try" + "; size-try-by-set2programwithbinary4apu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34) SIZE4BINARY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        apu2program4xi['SIZE4BINARY']  = task4programsetwb2apu['SIZE4BINARY']  
 
        message2print = "{(Step 34B/70) set" + "; size-set-by-set2programwithbinary4apu to: " + str(apu2program4xi['SIZE4BINARY'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 1
        code2ack  = code2ack + '34B-'
    except: 
        message2print = "{(Step 34Z1/70) yielded" + "; size-yielded-by-set2programwithbinary4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '34Z1-'             
        
    message2print = "{(Step 35A/70) try" + "; language-try-by-set2programwithbinary4apu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) LANGUAGE4BINARY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        apu2program4xi['LANGUAGE4BINARY']  = task4programsetwb2apu['LANGUAGE4BINARY']  
 
        message2print = "{(Step 35B/70) set" + "; language-set-by-set2programwithbinary4apu to: " + str(apu2program4xi['LANGUAGE4BINARY'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # Python
        code2ack  = code2ack + '35B-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; language-yielded-by-set2programwithbinary4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '35Z1-'    
                
    message2print = "{(Step 35A/70) try" + "; count2reference-try-by-set2programwithbinary4apu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) COUNT2REFERENCE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        apu2program4xi['COUNT2REFERENCE']  = task4programsetwb2apu['COUNT2REFERENCE']  
 
        message2print = "{(Step 35B/70) set" + "; count2reference-set-by-set2programwithbinary4apu to: " + str(apu2program4xi['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '35B-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; count2reference-yielded-by-set2programwithbinary4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '35Z1-'                           

    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    apu2program4xi ['ACK2RETURN']  = code2ack
    apu2program4xi ['NACK2RETURN'] = code2nack 
    
     
    keys2program4apu = apu2program4xi.keys()
    for key2program4apu in keys2program4apu:
        message2print = "{(Step 70/70) returning" + "; key2program4apu-returning-by-set2programwithbinary4apu: " + str(key2program4apu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # 

    return apu2program4xi
           
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "release2program4apu" usage = "
#
#   task4release2program  = {} 
#   task4release2program  ['COUNT2REFERENCE'] = 1 
#   response4release2program = {}
#   
#   print('(Step 18A/700) OF APU2PROGRAM4XI5TEST - TRYING TO RELEASE PROGRAM OF APU DEVICE ', time4xi())    
#   # 
#   try:                               
#       response4release2program = release2program4apu(task4release2program)
#       print('(Step 18B/700) OF APU2PROGRAM4XI5TEST - RELEASED PROGRAM OF APU DEVICE ', time4xi()) 
#   except:
#       print('(Step 18Z1/700) OF APU2PROGRAM4XI5TEST *** FAILED TO RELEASE PROGRAM OF APU DEVICE ', time4xi())                                                                                                  
#   try:
#       print('(Step 18C/700) OF APU2PROGRAM4XI5TEST COUNT2REFERENCE: ', response4release2program ['COUNT2REFERENCE'])
#       # 0
#   except:
#       print('(Step 18Z2/700) OF APU2PROGRAM4XI5TEST *** FAILED TO PRINT RESPONSE4RELEASE2PROGRAM ', response4release2program)  
#   
#">
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------   
def release2program4apu(task4release2apu=None):

    '''
    releases apu2program4xi
    '''
    global apu2program4xi  
    
             
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "release2program4apu"
    method2print = "release2program4apu" 

    message2print = "{(Step 1A/33) started" + "; release2program4apu-started: " + str(release2program4apu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function release2program4apu at 0x0000000003A98AC8>
    
    
    message2print = "{(Step 1B/33) passed" + "; task4release2apu-to-release2program4apu: " + str(task4release2apu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    

    code2ack  = ' '
    code2nack = ' '

    try:
        if apu2program4xi is None:
            apu2program4xi = create2programwithsource4apu()                                                                       # INITIALIZE PROGRAM                 
    except:
        try:
            apu2program4xi = create2programwithsource4apu()
        except:
            apu2program4xi = {}
            message2print = "{(Step 2Z0/70) yielded" + "; apu2program4xi-yielded-by-release2program4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '2Z0-' 
                
    message2print = "{(Step 1A/70) try" + "; release-try-by-release2program4apu: None"  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    
    if task4release2apu is None:
        task4release2apu = {}
        task4release2apu['COUNT2REFERENCE'] = 1
           
    try:
        # -----------------------------------------------------------------------------------------
        # RELEASE
        # -----------------------------------------------------------------------------------------               
        apu2program4xi['COUNT2REFERENCE'] = apu2program4xi['COUNT2REFERENCE'] - task4release2apu['COUNT2REFERENCE']
        
        message2print = "{(Step 1A/70) calculated" + "; count2reference-calculated-by-release2program4apu: " + str(apu2program4xi['COUNT2REFERENCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
         
        if apu2program4xi['COUNT2REFERENCE'] < 0:
            apu2program4xi['COUNT2REFERENCE'] = 0
            
            message2print = "{(Step 1B/70) warning" + "; count2reference-warning-by-release2program4apu: " + " ** " + str(apu2program4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            # 
             
            code2nack  = code2nack + '1B-'
        else:
            code2ack  = code2ack + '1B-'
    except:
        try:
            apu2program4xi['COUNT2REFERENCE'] = 0
            
            message2print = "{(Step 8Z1/70) warning" + "; count2reference-warning-by-release2program4apu: " + " ** " + str(apu2program4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            code2nack  = code2nack + '8Z1-'
            
        except:
            message2print = "{(Step 8Z2/70) yielded" + "; apu2program4xi-yielded-by-release2program4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '8Z2-'
                                                                                          
            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    apu2program4xi ['ACK2RETURN']  = code2ack
    apu2program4xi ['NACK2RETURN'] = code2nack  
    
    keys2program4apu = apu2program4xi.keys()
    for key2program4apu in keys2program4apu:
        message2print = "{(Step 70/70) returning" + "; key2program4apu-returning-by-release2program4apu: " + str(key2program4apu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # 
    
    return apu2program4xi

#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "retain2program4apu" usage = "
#
#   task4retain2program  = {} 
#   task4retain2program  ['COUNT2REFERENCE'] = 1 
#   response4retain2program = {}
#   
#   print('(Step 19A/700) OF APU2PROGRAM4XI5TEST - TRYING TO RETAIN PROGRAM OF APU DEVICE ', time4xi())    
#   # 
#   try:                               
#       response4retain2program = retain2program4apu(task4retain2program)
#       print('(Step 19B/700) OF APU2PROGRAM4XI5TEST - RETAINED PROGRAM OF APU DEVICE ', time4xi()) 
#   except:
#       print('(Step 19Z1/700) OF APU2PROGRAM4XI5TEST *** FAILED TO RETAIN PROGRAM OF APU DEVICE ', time4xi())                                                                                                  
#   try:
#       print('(Step 19C/700) OF APU2PROGRAM4XI5TEST COUNT2REFERENCE: ', response4retain2program ['COUNT2REFERENCE'])
#       # 2
#   except:
#       print('(Step 19Z2/700) OF APU2PROGRAM4XI5TEST *** FAILED TO PRINT RESPONSE4RETAIN2PROGRAM ', response4retain2program)  
#
#">
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------   
def retain2program4apu(task4retain2apu=None):

    '''
    retains either apu2program4xi
    '''
    global apu2program4xi 
    

    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "retain2program4apu"
    method2print = "retain2program4apu"

    message2print = "{(Step 1A/33) started" + "; retain2program4apu-started: " + str(retain2program4apu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function retain2program4apu at 0x0000000004095BC8>
    
    
    message2print = "{(Step 1B/33) passed" + "; task4retain2apu-to-retain2program4apu: " + str(task4retain2apu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'COUNT2REFERENCE': 1} 
    
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # (Step 2/70) - PROVIDE TASKS, IF NECESSARY AND CHECK GIVEN TASKS 
    # -----------------------------------------------------------------------------------------    
    if task4retain2apu is None:
        task4retain2apu = {}
        task4retain2apu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 2A/33) set" + "; task4retain2apu-set-by-retain2program4apu to: " + str(task4retain2apu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
    else:
        pass

    
    # -----------------------------------------------------------------------------------------
    # (Step 3/70) - CHECK COUNT2REFERENCE
    # -----------------------------------------------------------------------------------------         
    if 'COUNT2REFERENCE' in task4retain2apu.keys():
        pass
    else:
        task4retain2apu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 3A/33) set" + "; count2reference-set-by-retain2program4apu to: " + str(task4retain2apu['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
                                   
    if task4retain2apu['COUNT2REFERENCE'] < 1:
        task4retain2apu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 3B/33) set" + "; count2reference-set-by-retain2program4apu to: " + str(task4retain2apu['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
        
    # -----------------------------------------------------------------------------------------
    # (Step 4/70) - INITIALIZE PROGRAM IF NECESSARY
    # -----------------------------------------------------------------------------------------
    try:
        if apu2program4xi is None:
            apu2program4xi = create2programwithsource4apu()
            message2print = "{(Step 4A/70) created" + "; apu2program4xi-created-by-retain2program4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                                                                          
    except:
        try:
            apu2program4xi = create2programwithsource4apu()
            message2print = "{(Step 4B/70) created" + "; apu2program4xi-created-by-retain2program4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                 
        except:
            apu2program4xi = {}
            message2print = "{(Step 4Z/70) yielded" + "; apu2program4xi-yielded-by-retain2program4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '4Z-' 


    # -----------------------------------------------------------------------------------------
    # (Step 5/70) - RETAIN
    # -----------------------------------------------------------------------------------------             
    try:
        apu2program4xi['COUNT2REFERENCE'] = apu2program4xi['COUNT2REFERENCE'] + task4retain2apu['COUNT2REFERENCE']
        message2print = "{(Step 6A/70) calculated" + "; count2reference-calculated-by-retain2program4apu: " + str(apu2program4xi['COUNT2REFERENCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 1            
        if apu2program4xi['COUNT2REFERENCE'] <= 0:
            apu2program4xi['COUNT2REFERENCE'] = 1
            message2print = "{(Step 6Z1/70) warning" + "; count2reference-warning-by-retain2program4apu: " + " ** " + str(apu2program4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            code2nack  = code2nack + '6Z1-'
        else:
            code2ack  = code2ack + '6C-'

    except:
        try:
            apu2program4xi['COUNT2REFERENCE'] = 1
        
            message2print = "{(Step 7Z1/70) warning" + "; count2reference-warning-by-retain2program4apu: " + " ** " + str(apu2program4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            code2nack  = code2nack + '7Z1-'
        
        except:
            message2print = "{(Step 7Z2/70) yielded" + "; apu2program4xi-yielded-by-retain2program4apu: " + str(apu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '7Z2-'
        
                                                                                          
            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70) - RETURN           
    # -----------------------------------------------------------------------------------------
    
    apu2program4xi ['ACK2RETURN']  = code2ack
    apu2program4xi ['NACK2RETURN'] = code2nack     
     
    keys2program4apu = apu2program4xi.keys()
    for key2program4apu in keys2program4apu:
        message2print = "{(Step 70/70) returning" + "; key2program4apu-returning-by-retain2program4apu: " + str(key2program4apu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # 
    
    return apu2program4xi

#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "get2programinfo4apu" usage = "
#
#   print('(Step 3B1/700) OF APU2PROGRAM4XI5TEST - TRYING TO GET SOME PROGRAM BINARY INFO FOR APU ', time4xi())     
#   task4getinfo2program = {}
#   task4getinfo2program ['FUNCTION2CALLBACK'] = 'Yes'
#   task4getinfo2program ['COUNT4BINARY'] = 'Yes'
#   task4getinfo2program ['DIRECTORY4BINARY'] = 'Yes'
#   task4getinfo2program ['CODE4BINARY'] = 'Yes'
#   task4getinfo2program ['SIZE4BINARY'] = 'Yes'
#   task4getinfo2program ['LANGUAGE4BINARY'] = 'Yes'
#   task4getinfo2program ['COUNT2REFERENCE'] = 'Yes'
#   task4getinfo2program ['CODE4BINARY'] = 'Yes'
#   try:
#       response4programinfo2apu = get2programinfo4apu(task4getinfo2program) 
#       print('(Step 3B2/700) OF APU2PROGRAM4XI5TEST - GOT SOME PROGRAM BINARY INFO FOR APU ', time4xi())
#   except:
#       print('(Step 3Z3/700) OF APU2PROGRAM4XI5TEST *** FAILED TO GET SOME PROGRAM BINARY INFO FOR APU', time4xi())                  
#   try:                                                                                          
#       keys2response = response4programinfo2apu.keys()
#       print('COUNT2REFERENCE:', response4programinfo2apu ['COUNT2REFERENCE'])   
#       print('COUNT4BINARY:', response4programinfo2apu ['COUNT4BINARY'])
#       print('DIRECTORY4BINARY:', response4programinfo2apu ['DIRECTORY4BINARY']) 
#       print('FUNCTION2CALLBACK:', response4programinfo2apu ['FUNCTION2CALLBACK'])   
#       print('LANGUAGE4BINARY:', response4programinfo2apu ['LANGUAGE4BINARY'])
#       print('SIZE4BINARY:', response4programinfo2apu ['SIZE4BINARY'])                
#       # 
#       keys2source = response4programinfo2apu['CODE4BINARY'].keys()
#       for key2source in keys2source:
#           print('BINARY: ', key2source)
#           print(response4programinfo2apu ['CODE4BINARY'] [key2source] [1])
#           # COUNT2REFERENCE: 1
#           # COUNT4BINARY: 1
#           # DIRECTORY4BINARY: ...//__pycache__//
#           # FUNCTION2CALLBACK: <function trace2program4apu at 0x0000000004144AC8>
#           # LANGUAGE4BINARY: Python
#           # SIZE4BINARY: {'...//__pycache__//apu2image4xi.cpython-33.pyc': 1006}
#           # BINARY:  ...//__pycache__//apu2image4xi.cpython-33.pyc
#           # b'l\x0c\r\n'                       
#   except:
#       print('(Step 3Z4/700) OF APU2PROGRAM4XI5TEST *** FAILED TO PRINT response4programinfo2apu : ', response4programinfo2apu)
#   
#">
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------   
def get2programinfo4apu(task4getinfo2program=None):

    '''
    gets apu2program4xi and apu2program4xi
    '''
    
    global apu2program4xi

    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "get2programinfo4apu"
    method2print = "get2programinfo4apu"

    message2print = "{(Step 0A/33) started" + "; get2programinfo4apu-started: " + str(get2programinfo4apu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # <function get2programinfo4apu at 0x0000000003A98BC8>
                                                                
    
    message2print = "{(Step 0B/33) passed" + "; task4getinfo2program-passed-to-get2programinfo4apu: " + str(task4getinfo2program) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   
    # 
    
    
    apu2program = {}
    try: 
        if apu2program4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE SOURCE DATA FOR PROGRAM FOR APU           
            # -------------------------------------------------------------------------------------           
            apu2program = create2programwithsource4apu()
            message2print = "{(Step 1B2/33) initialized" + "; apu2program-initialized-by-get2programinfo4apu to: " + str(apu2program)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
        else:
            message2print = "{(Step 1B3/33) using" + "; apu2program4xi-using-by-get2programinfo4apu: " + str(apu2program4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
        # -------------------------------------------------------------------------------------
        # RETURN THE WHOLE OR PART OF PROGRAM INFO OF APU          
        # -------------------------------------------------------------------------------------     
        if task4getinfo2program is None:
            task4getinfo2program = {}
            apu2program = apu2program4xi
            message2print = "{(Step 1C/33) using" + "; apu2program-got-by-get2programinfo4apu: " + str(apu2program)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
        else:
            pass    

        keys2task4programinfo = task4getinfo2program.keys()
        keys2task = task4getinfo2program.keys()
            
        # -------------------------------------------------------------------------------------
        # SOURCE OR BINARY PROGRAM INFO OF APU          
        # -------------------------------------------------------------------------------------         
        for key2task in keys2task:
            try:
                message2print = "{(Step 2D2/33) using" + "; key2task-using-by-get2program4apu: " + str(key2task)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 
                 
                apu2program [key2task] = apu2program4xi[key2task]
                
                message2print = "{(Step 2D3/33) set" + "; info-of-apu2program-set-by-get2programinfo4apu to: " + str(apu2program [key2task])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
            except:
                message2print = "{(Step 2Z/70) yielded" + "; key2task-yielded-by-get2program4apu: " + str(key2task) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                               
                # 
                                   
                message2print = "{(Step 2Z2/70) using" + "; task4getinfo2program-using-by-get2program4apu: " + str(task4getinfo2program)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 
                
                message2print = "{(Step 2Z3/70) using" + "; apu2program4xi-using-by-get2program4apu: " + str(apu2program4xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 

    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE PROGRAM WITH SOURCE DATA           
            # -------------------------------------------------------------------------------------           
            apu2program = create2programwithsource4apu()
            
            message2print = "{(Step 1F/33) initialized" + "; apu2program-initialized-by-get2programinfo4apu to: " + str(apu2program)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            if task4getinfo2program is None:
                apu2program = apu2program4xi
                message2print = "{(Step 1G/33) set" + "; apu2program-set-by-get2programinfo4apu to: " + str(apu2program)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
            else:
                keys2task = task4getinfo2program.keys()
                for key2task in keys2task:
                    try:
                        message2print = "{(Step HD2/33) using" + "; key2task-using-by-get2program4apu: " + str(key2task)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        # 
                        apu2program [key2task] = apu2program4xi[key2task]
                    
                        message2print = "{(Step HD2/33) set" + "; info-of-apu2program -set-by-get2programinfo4apu to: " + str(apu2program  [key2task])  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        #   
                    except:
                        message2print = "{(Step HZ/70) yielded" + "; key2task-yielded-by-get2program4apu: " + str(key2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                                                                            
        except:
            apu2program = {}
            message2print = "{(Step 1Z3/70) yielded" + "; apu2program-yielded-by-get2program4apu: " + str(apu2program) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
     
    keys2program4apu = apu2program4xi.keys()
    for key2program4apu in keys2program4apu:
        message2print = "{(Step 70/70) returning" + "; key2program4apu-returning-by-get2programinfo4apu: " + str(key2program4apu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # CODE2ERROR ...
        
    return apu2program
            
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "get2buildinfo4apu" usage = "
#
#   response4get2buildinfo = get2buildinfo4apu()
#            
#">
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------   
def get2buildinfo4apu(task4getinfo2build=None):

    '''
    gets apu2build4xi and apu2build4xi
    '''
    
    global apu2build4xi

    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "get2buildinfo4apu"
    method2print = "get2buildinfo4apu"

    message2print = "{(Step 0A/33) started" + "; get2buildinfo4apu-started: " + str(get2buildinfo4apu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 
                                                                
    
    message2print = "{(Step 0B/33) passed" + "; task4getinfo2build-passed-to-get2buildinfo4apu: " + str(task4getinfo2build) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   
    # 
    
    
    apu2build = {}
    try: 
        if apu2build4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE SOURCE DATA FOR PROGRAM FOR APU           
            # -------------------------------------------------------------------------------------           
            apu2build = {}
            message2print = "{(Step 1B2/33) initialized" + "; apu2build-initialized-by-get2buildinfo4apu to: " + str(apu2build)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
        else:
            message2print = "{(Step 1B3/33) using" + "; apu2build4xi-using-by-get2buildinfo4apu: " + str(apu2build4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
        # -------------------------------------------------------------------------------------
        # RETURN THE WHOLE OR PART OF PROGRAM INFO OF APU          
        # -------------------------------------------------------------------------------------     
        if task4getinfo2build is None:
            task4getinfo2build = {}
            apu2build = apu2build4xi
            message2print = "{(Step 1C/33) using" + "; apu2build-got-by-get2buildinfo4apu: " + str(apu2build)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
        else:
            pass    

        keys2task4buildinfo = task4getinfo2build.keys()
        keys2task = task4getinfo2build.keys()
            
        # -------------------------------------------------------------------------------------
        # BUILD INFO OF APU          
        # -------------------------------------------------------------------------------------         
        for key2task in keys2task:
            try:
                message2print = "{(Step 2D2/33) using" + "; key2task-using-by-get2build4apu: " + str(key2task)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 
                 
                apu2build [key2task] = apu2build4xi[key2task]
                
                message2print = "{(Step 2D3/33) set" + "; info-of-apu2build-set-by-get2buildinfo4apu to: " + str(apu2build [key2task])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
            except:
                message2print = "{(Step 2Z/70) yielded" + "; key2task-yielded-by-get2build4apu: " + str(key2task) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                               
                # 
                                   
                message2print = "{(Step 2Z2/70) using" + "; task4getinfo2build-using-by-get2build4apu: " + str(task4getinfo2build)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 
                
                message2print = "{(Step 2Z3/70) using" + "; apu2build4xi-using-by-get2build4apu: " + str(apu2build4xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 

    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE BUILD           
            # -------------------------------------------------------------------------------------           
            apu2build = {}
            message2print = "{(Step 1F/33) initialized" + "; apu2build-initialized-by-get2buildinfo4apu to: " + str(apu2build)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            if task4getinfo2build is None:
                apu2build = apu2build4xi
                message2print = "{(Step 1G/33) set" + "; apu2build-set-by-get2buildinfo4apu to: " + str(apu2build)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
            else:
                keys2task = task4getinfo2build.keys()
                for key2task in keys2task:
                    try:
                        message2print = "{(Step HD2/33) using" + "; key2task-using-by-get2build4apu: " + str(key2task)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        # 
                        apu2build [key2task] = apu2build4xi[key2task]
                    
                        message2print = "{(Step HD2/33) set" + "; info-of-apu2build -set-by-get2buildinfo4apu to: " + str(apu2build  [key2task])  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        #   
                    except:
                        message2print = "{(Step HZ/70) yielded" + "; key2task-yielded-by-get2build4apu: " + str(key2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                                                                            
        except:
            apu2build = {}
            message2print = "{(Step 1Z3/70) yielded" + "; apu2build-yielded-by-get2build4apu: " + str(apu2build) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; apu2build-returning-by-get2buildinfo4apu: " + str(apu2build) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)
    # 
    
    return apu2build
            
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "link2program4apu" usage = "
#     
#   print('(Step 2AB2/700) OF APU2PROGRAM4XI5TEST - TRYING TO LINK INSIDE PROGRAM OBJECT WITH FUNCTION ', time4xi())           
#   try:
#       response4link = link2program4apu()
#       print('(Step 2BB2/700) OF APU2PROGRAM4XI5TEST - LINKED INSIDE PROGRAM OBJECT WITH FUNCTION ', time4xi()) 
#   except:
#       print('(Step 2Z2/700) OF APU2PROGRAM4XI5TEST *** FAILED TO LINK INSIDE PROGRAM OBJECT WITH FUNCTION *** ', time4xi())                                                                     
#   try:
#       print('(Step CC2/700) OF APU2PROGRAM4XI5TEST - response4link:') 
#       keys2response = response4link.keys()
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4link[key2response])
#           # apu2sampler4xi5test.py  :  <Process4Xi(Process4Xi-1, stopped[1])>
#   except:
#       print('(Step 2Z2/700) OF APU2PROGRAM4XI5TEST *** FAILED TO PRINT response4link *** ', response4link)
#   tailor2trace4program(t2xi)                                                                              # TAILOR TRACE AGAIN - TARGET FUNCTIONS OF ACTIVATED PROCESSES MIGHT HAVE RESET TRACE ! 
#          
#">
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------   
def link2program4apu(task4program2apu=None):

    '''
    links program for APU
    '''
    global apu2program4xi                                                                                           # Program of APU    
    global import2use4apu                                                                                           # Imports of APU
    global target2function4apu                                                                                      # Target function of Process4Xi
       
                
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "link2program4apu" 
    method2print = "link2program4apu"

    message2print = "{(Step 1A/70) started" + "; link2program4apu-started: " + str(link2program4apu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function link2program4apu at 0x0000000004332448>
  
             
    message2print = "{(Step 1B/70) passed" + "; task4program2apu-to-link2program4apu: " + str(task4program2apu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None

    code2ack  = ' '
    code2nack = ' '    
    
    process2program4apu = {}                                                                                       # INITIALIZES TO EMPTY DICTIONARY
               
    try:             
        if task4program2apu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER
            # -----------------------------------------------------------------------------------------    
            pass                                                                       
        else:
            # -----------------------------------------------------------------------------------------
            # PARAMETERS GIVEN ARE OMITTED 
            # -----------------------------------------------------------------------------------------         
            pass                                                                                                    # PARAMETER DATA RECEIVED                       
    except: 
        message2print = "{(Step 1Z1/70) yielded" + "; task4program2apu-yielded-by-link2program4apu: " + str(task4program2apu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
       
        code2nack  = code2nack + '1Z1-' 

    
    # ---------------------------------------------------------------------------------------------
    # (Step 20A/70) - LINK SOURCES INSIDE PROGRAM OBJECT FOR APU ! 
    # ---------------------------------------------------------------------------------------------    
    process2program4apu = {}                                                                                                                                                                               
    message2print = "{(Step 20A/70) initialized" + "; process2program4apu-initialized-by-link2program4apu to: " + str(process2program4apu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # {}

    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) - SOURCE CODE WITHOUT EOL CHARACTERS                  
        # ---------------------------------------------------------------------------------------------             
        keys2source4apu = apu2program4xi['CODE4SOURCE'].keys()
        message2print = "{(Step 30A/70) using" + "; keys2source4apu-using-by-link2program4apu: " + str(keys2source4apu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # dict_keys([folder2lib4apu + 'apu2sampler4xi5test.py'])
        
        unit4processing = "apu"
        
        for key2source4apu in keys2source4apu:
            index2search=str.find(key2source4apu,unit4processing) 
            if index2search == -1:
                pass
            else:
                target2link = key2source4apu[index2search:]                                                          # apu2sampler4xi5test.py
                index2search=str.find(target2link,".py")
                if index2search == -1:
                    pass
                else:
                    target2link4apu = target2link[:index2search]                                                  
                    message2print = "{(Step 32A/70) set" + "; target2link4apu-set-by-link2program4apu to: " + str(target2link4apu)  + "; time: " + time4xi() + "}"  
                    print4xi(class2print, method2print, message2print)
                    # apu2sampler4xi5test

                    p2source4apu = None    
                    try:
                        # ---------------------------------------------------------------------------------------------    
                        # IMPORT TARGET FUNCTION FROM ITS MODULE AS TARGET2FUNCTION4APU 
                        # ---------------------------------------------------------------------------------------------    
      
                        try:
                            import2use4apu.get(target2link4apu, handle2error4apu) ()
                            
                            message2print = "{(Step 32B/70) imported" + "; target2link4apu-imported-by-link2program4apu: " + str(target2link4apu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)
                            # apu2sampler4xi5test
                            code2ack  = code2ack + '32B-'
                        except:
                            message2print = "{(Step 32Z1/70) failed" + "; import2use4apu-failed-by-link2program4apu: " + str(import2use4apu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)                            
                            # 
                            message2print = "{(Step 32Z2/70) used" + "; target2link4apu-used-by-link2program4apu: " + str(target2link4apu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)
                            # 
                            code2nack  = code2nack + '32Z2-'
                                    
                        # ---------------------------------------------------------------------------------------------    
                        # CREATE PROCESS WITH TARGET2FUNCTION4APU
                        # ---------------------------------------------------------------------------------------------                          
                        p2apu = Process4Xi(target=target2function4apu, args=())                                
                        message2print = "{(Step 33A/70) created" + "; p2apu-created-by-link2program4apu: " + str(p2apu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        # <Process4Xi(Process4Xi-2, initial)>
                        
                        process2program4apu[key2source4apu] = p2apu
                        
                        code2ack  = code2ack + '33A-'
                        try:
                            # ---------------------------------------------------------------------------------------------    
                            # START PROCESS
                            # --------------------------------------------------------------------------------------------- 
                            p2apu.start()
                            message2print = "{(Step 34A/70) started" + "; p2apu-started-by-link2program4apu: " + str(p2apu.name) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)
                            # Process4Xi-2
                            
                            try:
                                # ---------------------------------------------------------------------------------------------    
                                # STATUS OF PROCESS
                                # ---------------------------------------------------------------------------------------------                             
                                status2process4apu = p2apu.is_alive()
                                message2print = "{(Step 35A/70) got" + "; status-got-by-link2program4apu: " + str(status2process4apu) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)
                                # True
                                
                                try:
                                    # ---------------------------------------------------------------------------------------------    
                                    # RUN PROCESS
                                    # ---------------------------------------------------------------------------------------------                                  
                                    p2apu.run()
                                    message2print = "{(Step 36A/70) run" + "; p2apu-run-by-link2program4apu: " + str(p2apu) + "; time: " + time4xi() + "}"
                                    print4xi(class2print, method2print, message2print)
                                    # <Process4Xi(Process4Xi-1, stopped[1])>
                                except:
                                    message2print = "{(Step 36Z/70) failed" + "; run-failed-by-link2program4apu: " + str(p2apu) + "; time: " + time4xi() + "}"
                                    print4xi(class2print, method2print, message2print)  
                                    code2nack  = code2nack + '36Z-'            
                                    # 
                            except:
                                message2print = "{(Step 35Z/70) failed" + "; status-failed-by-link2program4apu: " + str(p2apu) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)  
                                # 
                                code2nack  = code2nack + '35Z-'                                   
                                
                        except:
                            message2print = "{(Step 33Z/70) failed" + "; start-failed-by-link2program4apu: " + str(p2apu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)  
                            code2nack  = code2nack + '33Z-'     
                        
                    except:
                        message2print = "{(Step 32Z/70) failed" + "; p2source4apu-failed-by-link2program4apu: " + str(p2source4apu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        code2nack  = code2nack + '32Z-' 
                                                                                    

 
        message2print = "{(Step 38A/70) set" + "; process2program4apu-set-by-link2program4apu to: " + str(process2program4apu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
         
        code2ack  = code2ack + '38A-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; source-yielded-by-link2program4apu: " + str(apu2program4xi['CODE4SOURCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '30Z1-'     

                                                                                                               
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    apu2program4xi ['ACK2RETURN']  = code2ack
    apu2program4xi ['NACK2RETURN'] = code2nack  
    
    message2print = "{(Step 70/70) returning" + "; process2program4apu-returning-by-link2program4apu: " + str(process2program4apu) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                              
    # 
    return process2program4apu
            
#</function4xi> 
 
                                        
# =================================================================================================
#<class4xi class = "APU2Program4Xi" base = "object" child = "None"
#       synopsis = "offers  methods for APU" usage = "OpenCL APU"
#>
#<state4xi status = "ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
# =================================================================================================
class APU2Program4Xi(object): 
    ''' 
    APU2Program4Xi offers  methods for APU program
    '''            

    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    classvariable2apu = cpu_corecount4xi()
    #"
    #/>
       
       
    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi expose = "
    _exposed_ = (

            'add2programwithbinary',           # ACK 5.6.1   p132            
            'add2programwithsource',           # ACK 5.6.1   p133  
                                
            'build2program',                   # ACK 5.6.2   p137      

            'compile2program',                 # ACK 5.6.3   p139            
            'create2programwithsource',        # ACK 5.6.1   p132 
            'create2programwithbinary',        # ACK 5.6.1   p133 
            'create2programwithbuiltinkernels',# NACK 5.6.1  p135   
                       
            'delete2programwithbinary',        # ACK  
            'delete2programwithsource',        # ACK               
            
            'fetch2programbinary',             # ACK
            'fetch2programsource',             # ACK
            
            'get2programinfo',                 # ACK 5.6.7   p150 
            'get2programbuildinfo',            # NACK 5.6.7  p153
            
            'link2program',                    # ACK 5.6.3   p142                          
            'release2program',                 # ACK 5.6.1   p136                                         
            'retain2program',                  # ACK 5.6.1   p136
 
            'set2programwithsource',           # ACK
            'set2programwithbinary',           # ACK            
            'set2programcallback',             # ACK 
                              
            'trace2program',                   # ACK
            'unload2compiler',                 # NACK 5.6.6  p149 
        
            'sense',                           # ACK
            'reason',                          # ACK
            'ace')                             # ACK   
    #"
    #/>
 
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-APU2Program4Xi" usage = "
    # 
    #   t('(Step 110A/700) OF APU2PROGRAM4XI5TEST - CREATING PROGRAM4APU OBJECT ', time4xi())        
    #   
    #   # ---------------------------------------------------------------------------------------------
    #   # CREATE WITH OBJECT                  
    #   # ---------------------------------------------------------------------------------------------        
    #   programobject2apu = APU2Program4Xi()
    #   print('(Step 110B/700) OF APU2PROGRAM4XI5TEST - CREATED PROGRAM4APU OBJECT: ', programobject2apu)                                  
    #   # <apu2program4xi.APU2Program4Xi object at 0x0000000003F69198>
    #   
    #   pt:
    #   print('(Step 110Z/700) OF APU2PROGRAM4XI5TEST *** FAILED TO CREATE PROGRAM4APU OBJECT') 
    # 
    #">
    #<state4xi status = "ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------   
    def __init__(self, task4program2apu = None):
        '''
        _init__ method initializes APU2Program4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------
                    
        class2print = "APU2Program4Xi"
        method2print = "__init__"
                     
        message2print = "{(Step 1A/33) started" + "; init-of-APU2Program4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 

        message2print = "{(Step 1B/33) borning" + "; apu2program2xi-of-APU2Program4Xi-borning: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
                
        message2print = "{(Step 1C/33) inheriting" + "; object-inheriting-by-apu2program2xi-of-APU2Program4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <class 'object'>
        
        message2print = "{(Step 1D/33) sharing" + "; classvariable2apu-sharing-by-apu2program2xi-of-APU2Program4Xi: " + str(APU2Program4Xi.classvariable2apu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        #         
                                
        message2print = "{(Step 1E/33) passed" + "; task4program2apu-passed-to-init-of-APU2Program4Xi: " + str(task4program2apu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        #                       
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/33) - source!              
        # -----------------------------------------------------------------------------------------
    
        message2print = "{(Step 2A/33) passing" + "; task4program2apu-passing-to-create2programwithsource4apu-by-init-of-APU2Program4Xi: " + str(task4program2apu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None  
                
        apu2program2xi = create2programwithsource4apu(task4program2apu)
        
        message2print = "{(Step 2B/33) performed" + "; create2programwithsource4apu-performed-by-init-of-APU2Program4Xi: " + str(create2programwithsource4apu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <function create2programwithsource4apu at 0x0000000003A98D48>
       
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)             
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 3A/33) activated" + "; apu2program2xi-of-APU2Program4Xi-activated: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # apu2program2xi-of-APU2Program4Xi-activated
                
        message2print = "{(Step 3B/33) inherited" + "; object-inherited-by-apu2program2xi-of-APU2Program4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <class 'object'>
        
        message2print = "{(Step 3C/33) shared" + "; classvariable2apu-shared-by-apu2program2xi-of-APU2Program4Xi: " + str(APU2Program4Xi.classvariable2apu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 4         
       
        message2print = "{(Step 3D/33) returning" + "; init-of-APU2Program4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 
        

    #</methodize4xi>
    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "add2programwithbinary-of-APU2Program4Xi" usage = "
    #  
    #   print('(Step 132A/700) OF APU2PROGRAM4XI5TEST - TRYING TO ADD BINARY INTO PROGRAM OBJECT WITH METHOD ', time4xi())         
    #   try:
    #       task4add2program = {}
    #       task4add2program ['CODE4BINARY'] = {}
    #       task4add2program ['SIZE4BINARY'] = {}            
    #       # NOT OK - EXISTS
    #       task4add2program ['CODE4BINARY'] [folder2lib4apu + '__pycache__//apu2image4xi.cpython-33.pyc'] = {}
    #       task4add2program ['CODE4BINARY'] [folder2lib4apu + '__pycache__//apu2image4xi.cpython-33.pyc'] [1] = b'l\x0c\r\n'
    #       task4add2program ['CODE4BINARY'] [folder2lib4apu + '__pycache__//apu2image4xi.cpython-33.pyc'] [2] = b'l\x0c\z\n' 
    #       task4add2program ['SIZE4BINARY'] [folder2lib4apu + '__pycache__//apu2image4xi.cpython-33.pyc'] = len(task4add2program ['CODE4BINARY'] [folder2lib4apu + '__pycache__//apu2image4xi.cpython-33.pyc'])
    #                
    #       # OK
    #       task4add2program ['CODE4BINARY'] [folder2lib4apu + '__pycache__//apu3image4xi.cpython-33.pyc'] = {}
    #       task4add2program ['CODE4BINARY'] [folder2lib4apu + '__pycache__//apu3image4xi.cpython-33.pyc'] [1] = b'l\x0c\r\n'
    #       task4add2program ['CODE4BINARY'] [folder2lib4apu + '__pycache__//apu3image4xi.cpython-33.pyc'] [2] = b'l\x0c\z\n' 
    #       task4add2program ['SIZE4BINARY'] [folder2lib4apu + '__pycache__//apu3image4xi.cpython-33.pyc'] = len(task4add2program ['CODE4BINARY'] [folder2lib4apu + '__pycache__//apu3image4xi.cpython-33.pyc'])
    #       
    #       response4add2programwithbinary = programobject2apu.add2programwithbinary(task4add2program) 
    #       print('(Step 132B/700) OF APU2PROGRAM4XI5TEST - ADDED BINARY INTO PROGRAM OBJECT WITH METHOD ', time4xi())
    #   except:
    #       print('(Step 132Z1/700) OF APU2PROGRAM4XI5TEST *** FAILED TO ADD BINARY INTO PROGRAM OBJECT WITH METHOD *** ', time4xi())
    #   try:                                                                                          
    #       keys2response = response4add2programwithbinary.keys()
    #       print('COUNT2REFERENCE:', response4add2programwithbinary ['COUNT2REFERENCE'])   
    #       print('COUNT4BINARY:', response4add2programwithbinary ['COUNT4BINARY'])
    #       print('DIRECTORY4BINARY:', response4add2programwithbinary ['DIRECTORY4BINARY']) 
    #       print('FUNCTION2CALLBACK:', response4add2programwithbinary ['FUNCTION2CALLBACK'])   
    #       print('LANGUAGE4BINARY:', response4add2programwithbinary ['LANGUAGE4BINARY'])
    #       print('SIZE4BINARY:', response4add2programwithbinary ['SIZE4BINARY'])                
    #       # 
    #       keys2source = response4add2programwithbinary['CODE4BINARY'].keys()
    #       for key2source in keys2source:
    #           print('BINARY: ', key2source)
    #           print(response4add2programwithbinary ['CODE4BINARY'] [key2source] [1])
    #       # COUNT2REFERENCE: 1
    #       # COUNT4BINARY: 2
    #       # DIRECTORY4BINARY: ... //__pycache__//
    #       # FUNCTION2CALLBACK: <function trace2program4apu at 0x000000000419BBC8>
    #       # LANGUAGE4BINARY: Python
    #       # SIZE4BINARY: {'...//__pycache__//apu2image4xi.cpython-33.pyc': 1006, '...//__pycache__//apu3image4xi.cpython-33.pyc': 2}
    #       # BINARY:  ...__pycache__//apu2image4xi.cpython-33.pyc
    #       # b'l\x0c\r\n'
    #       # BINARY:  ...//__pycache__//apu3image4xi.cpython-33.pyc
    #       # b'l\x0c\r\n'       
    #   except:
    #       print('(Step 132Z2/700) OF APU2PROGRAM4XI5TEST *** FAILED TO PRINT RESPONSE4ADD2PROGRAMWITHBINARY : ', response4add2programwithbinary) 
    #  
    #">
    #<state4xi status = "ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------   
    def add2programwithbinary(self, task4add2programwithbinary = None):
        '''
        sets program APU objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "APU2Program4Xi"
        method2print = "add2programwithbinary"
                     
        message2print = "{(Step 1A/33) started" + "; add2programwithbinary-of-APU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:44:02 11/22/11 Pacific Standard Time
            
        message2print = "{(Step 1B/33) passed" + "; task4add2programwithbinary-passed-to-add2programwithbinary-of-APU2Program4Xi: " + str(task4add2programwithbinary) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # {'CODE4BINARY': {folder2lib4apu + '__pycache__//apu2image4xi.cpython-33.pyc': {1: b'l\x0c\r\n', 2: b'l\x0c\\z\n'}, 
        #                  folder2lib4apu + '__pycache__//apu3image4xi.cpython-33.pyc': {1: b'l\x0c\r\n', 2: b'l\x0c\\z\n'}}, 
        #  'SIZE4BINARY': {folder2lib4apu + '__pycache__//apu2image4xi.cpython-33.pyc': 2, 
        #                  folder2lib4apu + '__pycache__//apu3image4xi.cpython-33.pyc': 2}
        # }

          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4add2programwithbinary-passing-to-add2programwithbinary4apu-by-add2programwithbinary-of-APU2Program4Xi: " + str(task4add2programwithbinary) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # {'CODE4BINARY': {folder2lib4apu + '__pycache__//apu2image4xi.cpython-33.pyc': {1: b'l\x0c\r\n', 2: b'l\x0c\\z\n'}, 
        #                  folder2lib4apu + '__pycache__//apu3image4xi.cpython-33.pyc': {1: b'l\x0c\r\n', 2: b'l\x0c\\z\n'}}, 
        #  'SIZE4BINARY': {folder2lib4apu + '__pycache__//apu2image4xi.cpython-33.pyc': 2, 
        #                  folder2lib4apu + '__pycache__//apu3image4xi.cpython-33.pyc': 2}
        
        response4add2programwithbinary = add2programwithbinary4apu(task4add2programwithbinary)

        message2print = "{(Step 2B/33) performed" + "; add2programwithbinary4apu-performed-by-add2programwithbinary-of-APU2Program4Xi: " + str(add2programwithbinary4apu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function add2programwithbinary4apu at 0x00000000040EDCC8>
        
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4add2programwithbinary-returning-by-add2programwithbinary-of-APU2Program4Xi: " + str(response4add2programwithbinary) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 1 ...
        
                
        return response4add2programwithbinary
                                
                            
    #</methodize4xi>    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "add2programwithsource-of-APU2Program4Xi" usage = "
    #  
    #   print('(Step 122A/700) OF APU2PROGRAM4XI5TEST - TRYING TO ADD SOURCE INTO PROGRAM OBJECT WITH CLASS ', time4xi())          
    #   try:
    #       task4add2program = {}
    #       # 
    #       task4add2program ['CODE4SOURCE'] = {}
    #       task4add2program ['SIZE4SOURCE'] = {}
    #                   
    #       task4add2program ['CODE4SOURCE'] [folder2lib4apu + 'apu2buffer4xi.py'] = {}
    #       task4add2program ['CODE4SOURCE'] [folder2lib4apu + 'apu2buffer4xi.py'] [1] = '... profile = "33" />\n'
    #       task4add2program ['CODE4SOURCE'] [folder2lib4apu + 'apu2buffer4xi.py'] [2] = '... program = "apu2buffer4xi.py" />\n'
    #       task4add2program ['SIZE4SOURCE'] [folder2lib4apu + 'apu2buffer4xi.py'] = len(task4add2program ['CODE4SOURCE'] [folder2lib4apu + 'apu2buffer4xi.py'])
    #       # 
    #       task4add2program ['CODE4SOURCE'] [folder2lib4apu + 'apu2sampler4xi5test.py'] = {}
    #       task4add2program ['CODE4SOURCE'] [folder2lib4apu + 'apu2sampler4xi5test.py'] [1] = '... profile = "33" />\n'
    #       task4add2program ['CODE4SOURCE'] [folder2lib4apu + 'apu2sampler4xi5test.py'] [2] = '... program = "apu2sampler4xi5test.py" />\n'
    #       
    #       task4add2program ['SIZE4SOURCE'] [folder2lib4apu + 'apu2sampler4xi5test.py'] = len(task4add2program ['CODE4SOURCE'] [folder2lib4apu + 'apu2sampler4xi5test.py'])            
    #       task4add2program ['SIZE4SOURCE'] [folder2lib4apu + 'apu2buffer4xi.py'] = len(task4add2program ['CODE4SOURCE'] [folder2lib4apu + 'apu2buffer4xi.py'])            
    #       
    #       response4add2programwithsource = programobject2apu.add2programwithsource(task4add2program)
    #       print('(Step 122B/700) OF APU2PROGRAM4XI5TEST - ADDED SOURCE INTO PROGRAM OBJECT WITH CLASS ', time4xi())    
    #   except:
    #       print('(Step 122Z1/700) OF APU2PROGRAM4XI5TEST *** FAILED TO ADD SOURCE INTO PROGRAM OBJECT WITH CLASS *** ', time4xi())
    #   try:                                                                                      
    #       print('COUNT4SOURCE: ', response4add2programwithsource['COUNT4SOURCE'])
    #       print('DIRECTORY4SOURCE: ', response4add2programwithsource['DIRECTORY4SOURCE'])
    #       print('LANGUAGE4SOURCE: ', response4add2programwithsource['LANGUAGE4SOURCE'])
    #       print('SIZE4SOURCE: ', response4add2programwithsource['SIZE4SOURCE'])
    #       keys2source = response4add2programwithsource['CODE4SOURCE'].keys()
    #       for key2source in keys2source:
    #           print('SOURCE: ', key2source)
    #           print(response4add2programwithsource ['CODE4SOURCE'] [key2source] [1])
    #       # COUNT4SOURCE:  2
    #       # DIRECTORY4SOURCE:  OK
    #       # LANGUAGE4SOURCE:  Python
    #       # SIZE4SOURCE:  {' ... //apu2buffer4xi.py': 2, '... apu2sampler4xi5test.py': 1337}
    #       # SOURCE:  ... //apu2buffer4xi.py
    #       # ... profile = "33" />
    #       # 
    #       # SOURCE:  ... apu2sampler4xi5test.py
    #       # ... profile = "11" />                              
    #   except:
    #       print('(Step 1222Z2/700) OF APU2PROGRAM4XI5TEST *** FAILED TO PRINT RESPONSE4ADD2PROGRAMWITHSOURCE: ', response4add2programwithsource)  
    #    
    #">
    #<state4xi status = "ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------   
    def add2programwithsource(self, task4add2programwithsource = None):
        '''
        sets program APU objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "APU2Program4Xi"
        method2print = "add2programwithsource"
                     
        message2print = "{(Step 1A/33) started" + "; add2programwithsource-of-APU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:44:01 11/22/11 Pacific Standard Time
            
        message2print = "{(Step 1B/33) passed" + "; task4add2programwithsource-passed-to-add2programwithsource-of-APU2Program4Xi: " + str(task4add2programwithsource) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # {'CODE4SOURCE': {folder2lib4apu + 'apu2buffer4xi.py': {1: '... profile = "33" />\n', 2: '... program = "apu2buffer4xi.py" />\n'}, 
        #                  folder2lib4apu + 'apu2sampler4xi5test.py': {1: '... profile = "33" />\n', 2: '... program = "apu2sampler4xi5test.py" />\n'}}, 
        #  'SIZE4SOURCE': {folder2lib4apu + 'apu2buffer4xi.py': 2, 
        #                  folder2lib4apu + 'apu2sampler4xi5test.py': 2}
        #}

          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4add2programwithsource-passing-to-add2programwithsource4apu-by-add2programwithsource-of-APU2Program4Xi: " + str(task4add2programwithsource) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # {'CODE4SOURCE': {folder2lib4apu + 'apu2buffer4xi.py': {1: '... profile = "33" />\n', 2: '... program = "apu2buffer4xi.py" />\n'}, 
        #                  folder2lib4apu + 'apu2sampler4xi5test.py': {1: '... profile = "33" />\n', 2: '... program = "apu2sampler4xi5test.py" />\n'}}, 
        #  'SIZE4SOURCE': {folder2lib4apu + 'apu2buffer4xi.py': 2, 
        #                  folder2lib4apu + 'apu2sampler4xi5test.py': 2}
        #}; time: 00:44:01 11/22/11 Pacific Standard Time}
        
        response4add2programwithsource = add2programwithsource4apu(task4add2programwithsource)

        message2print = "{(Step 2B/33) performed" + "; add2programwithsource4apu-performed-by-add2programwithsource-of-APU2Program4Xi: " + str(add2programwithsource4apu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function add2programwithsource4apu at 0x00000000040DDD48>
        
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4add2programwithsource-returning-by-add2programwithsource-of-APU2Program4Xi: " + str(response4add2programwithsource) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 
        #  'LANGUAGE4SOURCE': 'Python', 
        #  'COUNT4SOURCE': 2, 
        #  'SIZE4SOURCE': {folder2lib4apu + 'apu2buffer4xi.py': 2, 
        #                  folder2lib4apu + 'apu2sampler4xi5test.py': 1337}, 
        #  'DATA2CALLBACK': ...
         
        return response4add2programwithsource
                                
                            
    #</methodize4xi>    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "build2program-of-APU2Program4Xi" usage = "
    # 
    #   task4build2program  = {}
    #   task4build2program  ['PROGRAM'] = folder2lib4apu + 'apu2sampler4xi.py'
    #   task4build2program  ['OPTIONS'] = {}
    #   task4build2program  ['OPTIONS'] ['MATH']          = 'SINGLE-PRECISION-CONSTANT'
    #   task4build2program  ['OPTIONS'] ['OPTIMIZATION']  = 'DISABLE'
    #   task4build2program  ['OPTIONS'] ['VERSION']       = 'CL1.1'
    #   task4build2program  ['OPTIONS'] ['WARNINGS']      = 'INHIBIT'
    #   print('(Step 240A/700) OF APU2PROGRAM4XI5TEST - TRYING TO BUILD PROGRAM OF APU DEVICE ', time4xi())    
    #   try:                               
    #       response4build2program = programobject2apu.build2program(task4build2program) 
    #       print('(Step 240B/700) OF APU2PROGRAM4XI5TEST - BUILT PROGRAM OF APU DEVICE ', time4xi())
    #   except:
    #       print('(Step 240Z1/700) OF APU2PROGRAM4XI5TEST *** FAILED TO BUILD PROGRAM OF APU DEVICE ', time4xi())
    #   try:                                                                                                     
    #       keys2response = response4build2program.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4build2program[key2response])
    #   # ACK2RETURN  :   11A1-12A1-
    #   # NACK2RETURN  :   13Z1-14Z1-
    #   # OPTIONS  :  {'VERSION': 'CL1.1', 'OPTIMIZATION': 'DISABLE', 'MATH': 'SINGLE-PRECISION-CONSTANT', 'WARNINGS': 'INHIBIT'}
    #   # PROGRAM  :  folder2lib4apu + apu2sampler4xi.py                
    #   except:
    #       print('(Step 240Z2/700) OF APU2PROGRAM4XI5TEST *** FAILED TO PRINT ', response4build2program)              
    #                     
    #">   
    #<state4xi status = "ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------   
    def build2program(self, task4build = None):
        '''
        build2program method builds program objects of APU2Program4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "APU2Program4Xi"
        method2print = "build2program"
                     
        message2print = "{(Step 1A/33) started" + "; build2program-of-APU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4build-passed-to-create-of-APU2Program4Xi: " + str(task4build) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                      
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4build-passing-to-build2program4apu-by-build2program-of-APU2Program4Xi: " + str(task4build) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #    
  
        apu2program2xi = build2program4apu(task4build)

        message2print = "{(Step 2B/33) performed" + "; build2program4apu-performed-by-build2program-of-APU2Program4Xi: " + str(build2program4apu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 3/33) returning" + "; apu2program2xi-returning-by-build2program-of-APU2Program4Xi: " + str(apu2program2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          # 
        # 

        return apu2program2xi
                     
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "compile2program-of-APU2Program4Xi" usage = "
    # 
    #   print('(Step 121A1/700) OF APU2PROGRAM4XI5TEST - COMPILING PROGRAM FOR APU', time4xi())      
    #   try:
    #       # ---------------------------------------------------------------------------------------------
    #       # COMPILE                        
    #       # ---------------------------------------------------------------------------------------------        
    #       response4compile = programobject2apu.compile2program()
    #       print('(Step 121A2/700) OF APU2PROGRAM4XI5TEST - COMPILING PROGRAM FOR APU', time4xi())                                   
    #   except:
    #       print('(Step 121AZ1/700) OF APU2PROGRAM4XI5TEST *** FAILED TO COMPILE PROGRAM FOR APU')
    #   try:
    #       keys2response4compile = response4compile.keys()
    #       for key2response4compile in keys2response4compile:
    #           print('(Step 121A3/700) OF APU2PROGRAM4XI5TEST - KEY OF RESPONSE4COMPILE: ', key2response4compile)
    #           # apu2sampler4xi5test.py 
    #   except:
    #       print('(Step 121AZ2/700) OF APU2PROGRAM4XI5TEST *** FAILED TO PRINT RESPONSE4COMPILE:' , response4compile)
    #                
    #   tailor2trace4program(t2xi)                  # <-- tailor trace again - target functions of activated processes might have reset trace !
    #                     
    #">   
    #<state4xi status = "ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------   
    def compile2program(self, task4program2create = None):
        '''
        compile2program compiles for program object of APU2Program4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "APU2Program4Xi"
        method2print = "compile2program"
                     
        message2print = "{(Step 1A/33) started" + "; compile2program-of-APU2Program4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        # 03:09:40 11/23/11 Pacific Standard Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4program2create-passed-to-create-of-APU2Program4Xi: " + str(task4program2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                      
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4program2create-passing-to-compile2program4apu-by-compile2program-of-APU2Program4Xi: " + str(task4program2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # None
  
        apu2program2xi = compile2program4apu(task4program2create)

        message2print = "{(Step 2B/33) performed" + "; compile2program4apu-performed-by-compile2program-of-APU2Program4Xi: " + str(compile2program4apu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 3/33) returning" + "; apu2program2xi-returning-by-compile2program-of-APU2Program4Xi: " + str(apu2program2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 
        
        return apu2program2xi
                                
    #                        
    #</methodize4xi>
        

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "link2program-of-APU2Program4Xi" usage = "
    # 
    #   print('(Step 121C1/700) OF APU2PROGRAM4XI5TEST - LINKING PROGRAM FOR APU', time4xi())      
    #   try:
    #       # ---------------------------------------------------------------------------------------------
    #       # LINK                        
    #       # ---------------------------------------------------------------------------------------------        
    #       response4link = programobject2apu.link2program()
    #       print('(Step 121C2/700) OF APU2PROGRAM4XI5TEST - COMPILING PROGRAM FOR APU', time4xi())                                   
    #   except:
    #       print('(Step 121CZ1/700) OF APU2PROGRAM4XI5TEST *** FAILED TO LINK PROGRAM FOR APU')
    #   try:
    #       keys2response4link = response4link.keys()
    #       for key2response4link in keys2response4link:
    #           print('(Step 121C3/700) OF APU2PROGRAM4XI5TEST - KEY OF RESPONSE4LINK: ', key2response4link)
    #           # 
    #   except:
    #       print('(Step 121CZ2/700) OF APU2PROGRAM4XI5TEST *** FAILED TO PRINT RESPONSE4LINK:' , response4link)
    #                     
    #">   
    #<state4xi status = "ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------   
    def link2program(self, task4program2create = None):
        '''
        link2program links for program object of APU2Program4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "APU2Program4Xi"
        method2print = "link2program"
                     
        message2print = "{(Step 1A/33) started" + "; link2program-of-APU2Program4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        # 04:02:10 11/23/11 Pacific Standard Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4program2create-passed-to-create-of-APU2Program4Xi: " + str(task4program2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                      
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4program2create-passing-to-link2program4apu-by-link2program-of-APU2Program4Xi: " + str(task4program2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # None
  
        apu2program2xi = link2program4apu(task4program2create)

        message2print = "{(Step 2B/33) performed" + "; link2program4apu-performed-by-link2program-of-APU2Program4Xi: " + str(link2program4apu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 3/33) returning" + "; apu2program2xi-returning-by-link2program-of-APU2Program4Xi: " + str(apu2program2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 
        
        return apu2program2xi
                                
    #                        
    #</methodize4xi>
        

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "create2programwithsource-of-APU2Program4Xi" usage = "
    # 
    #   print('(Step 190A/700) OF APU2PROGRAM4XI5TEST - TRYING TO CREATE PROGRAM WITH SOURCE FOR APU ', time4xi()) 
    #   try:
    #       programobject2apu = APU2Program4Xi()                            
    #       response4create2programwithsource = programobject2apu.create2programwithsource()
    #       print('(Step 190B/700) OF APU2PROGRAM4XI5TEST - CREATED PROGRAM WITH SOURCE FOR APU ', time4xi())
    #   except:
    #       print('(Step 190Z1/700) OF APU2PROGRAM4XI5TEST *** FAILED TO CREATE PROGRAM WITH SOURCE FOR APU ', time4xi())                                    
    #   try:                                                                                      
    #       print('COUNT4SOURCE: ', response4create2programwithsource['COUNT4SOURCE'])
    #       print('DIRECTORY4SOURCE: ', response4create2programwithsource['DIRECTORY4SOURCE'])
    #       print('LANGUAGE4SOURCE: ', response4create2programwithsource['LANGUAGE4SOURCE'])
    #       print('SIZE4SOURCE: ', response4create2programwithsource['SIZE4SOURCE'])
    #       keys2source = response4create2programwithsource['CODE4SOURCE'].keys()
    #       for key2source in keys2source:
    #           print('SOURCE: ', key2source)
    #           print(response4create2programwithsource ['CODE4SOURCE'] [key2source] [1])
    #           print(response4create2programwithsource ['CODE4SOURCE'] [key2source] [3])
    #           print(response4create2programwithsource ['CODE4SOURCE'] [key2source] [4])
    #           print(response4create2programwithsource ['CODE4SOURCE'] [key2source] [5])
    #           
    #       # COUNT4SOURCE:  1
    #       # DIRECTORY4SOURCE:  folder2lib4apu
    #       # LANGUAGE4SOURCE:  Python
    #       # SIZE4SOURCE:  {folder2lib4apu + 'apu2sampler4xi5test.py': 1337}
    #       # SOURCE:  folder2lib4apu + apu2sampler4xi5test.py
    #       # ... profile = "11" />
    #       # ... program = "apu2sampler4xi5test.py">
    #       # ... abstract = "tests apu2sampler4xi.py"/>
    #       # ... verification = "11" target = "apu2sampler4xi.py"/>              
    #   except:
    #       print('(Step 190Z2/700) OF APU2PROGRAM4XI5TEST *** FAILED TO PRINT response4create2programwithsource *** ', response4create2programwithsource)  
    #           
    #">   
    #<state4xi status = "ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------   
    def create2programwithsource(self, task4program2create = None):
        '''
        create2programwithsource method creates program objects of APU2Program4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "APU2Program4Xi"
        method2print = "create2programwithsource"
                     
        message2print = "{(Step 1A/33) started" + "; create2programwithsource-of-APU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 05:16:42 11/17/11 Pacific Standard Time
            
        message2print = "{(Step 1B/33) passed" + "; task4program2create-passed-to-create-of-APU2Program4Xi: " + str(task4program2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                      
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4program2create-passing-to-create2programwithsource4apu-by-create2programwithsource-of-APU2Program4Xi: " + str(task4program2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # None   
  
        apu2program2xi = create2programwithsource4apu(task4program2create)

        message2print = "{(Step 2B/33) performed" + "; create2programwithsource4apu-performed-by-create2programwithsource-of-APU2Program4Xi: " + str(create2programwithsource4apu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function create2programwithsource4apu at 0x0000000004024848> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 3/33) returning" + "; apu2program2xi-returning-by-create2programwithsource-of-APU2Program4Xi: " + str(apu2program2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          # 
        # {'CODE2ERROR': 0, 
        #  'LANGUAGE4SOURCE': 'Python', 
        #  'COUNT4SOURCE': 1, 
        #  'SIZE4SOURCE': {folder2lib4apu + 'apu2sampler4xi5test.py': 1337}, 
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2programwithsource4apu-failed ; time: 05:16:42 11/17/11 Pacific Standard Time}', 
        # ...

        return apu2program2xi
                                
    #                        
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "create2programwithbinary-of-APU2Program4Xi" usage = "
    # 
    #   print('(Step 130A/700) OF APU2PROGRAM4XI5TEST - CREATING WITH BINARY PROGRAM FOR APU', time4xi())      
    #   try:
    #       # ---------------------------------------------------------------------------------------------
    #       # CREATE BINARY                         
    #       # ---------------------------------------------------------------------------------------------        
    #       response4create2programwithbinary = programobject2apu.create2programwithbinary()
    #       print('(Step 130A/700) OF APU2PROGRAM4XI5TEST - CREATED WITH BINARY PROGRAM FOR APU', time4xi())
    #   except:
    #       print('(Step 130Z1/700) OF APU2PROGRAM4XI5TEST *** FAILED TO CREATE  WITH BINARY PROGRAM FOR APU')  
    #   try:                                                                                          
    #       keys2response = response4create2programwithbinary.keys()
    #       print('COUNT2REFERENCE:', response4create2programwithbinary ['COUNT2REFERENCE'])   
    #       print('COUNT4BINARY:', response4create2programwithbinary ['COUNT4BINARY'])
    #       print('DIRECTORY4BINARY:', response4create2programwithbinary ['DIRECTORY4BINARY']) 
    #       print('FUNCTION2CALLBACK:', response4create2programwithbinary ['FUNCTION2CALLBACK'])   
    #       print('LANGUAGE4BINARY:', response4create2programwithbinary ['LANGUAGE4BINARY'])
    #       print('SIZE4BINARY:', response4create2programwithbinary ['SIZE4BINARY'])                
    #       #
    #       keys2source = response4create2programwithbinary['CODE4BINARY'].keys()
    #       for key2source in keys2source:
    #           print('BINARY: ', key2source)
    #           print(response4create2programwithbinary ['CODE4BINARY'] [key2source] [1])
    #       # COUNT2REFERENCE: 1
    #       # COUNT4BINARY: 1
    #       # DIRECTORY4BINARY: folder2lib4apu + __pycache__//
    #       # FUNCTION2CALLBACK: <function trace2program4apu at 0x000000000415CBC8>
    #       # LANGUAGE4BINARY: Python
    #       # SIZE4BINARY: {'folder2lib4apu + '__pycache__//apu2image4xi.cpython-33.pyc': 1006}
    #       # BINARY:  folder2lib4apu + __pycache__//apu2image4xi.cpython-33.pyc
    #       # b'l\x0c\r\n'
    #   except:
    #       print('(Step 130Z2/700) OF APU2PROGRAM4XI5TEST *** FAILED TO PRINT RESPONSE4CREATE2PROGRAMWITHBINARY : ', response4create2programwithbinary)              
    #                     
    #">   
    #<state4xi status = "ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------   
    def create2programwithbinary(self, task4program2create = None):
        '''
        create2programwithbinary method creates program objects of APU2Program4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "APU2Program4Xi"
        method2print = "create2programwithbinary"
                     
        message2print = "{(Step 1A/33) started" + "; create2programwithbinary-of-APU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 05:16:42 11/17/11 Pacific Standard Time
            
        message2print = "{(Step 1B/33) passed" + "; task4program2create-passed-to-create-of-APU2Program4Xi: " + str(task4program2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                      
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4program2create-passing-to-create2programwithbinary4apu-by-create2programwithbinary-of-APU2Program4Xi: " + str(task4program2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # None   
  
        apu2program2xi = create2programwithbinary4apu(task4program2create)

        message2print = "{(Step 2B/33) performed" + "; create2programwithbinary4apu-performed-by-create2programwithbinary-of-APU2Program4Xi: " + str(create2programwithbinary4apu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function create2programwithbinary4apu at 0x00000000040248C8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 3/33) returning" + "; apu2program2xi-returning-by-create2programwithbinary-of-APU2Program4Xi: " + str(apu2program2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          # 
        # {'CODE2ERROR': 0, 
        #  'COUNT2REFERENCE': 1, 
        #  'DIRECTORY4BINARY': folder2lib4apu + '__pycache__//', 
        # ... 

        return apu2program2xi
                                
    #                        
    #</methodize4xi>
        

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "delete2programwithbinary-of-APU2Program4Xi" usage = "
    #  
    #   print('(Step 134A/700) OF APU2PROGRAM4XI5TEST - TRYING TO DELETE BINARY INSIDE PROGRAM OBJECT WITH METHOD ', time4xi())        
    #   try:
    #       task4delete2program = {}
    #       task4delete2program ['CODE4BINARY'] = {}
    #       task4delete2program ['CODE4BINARY'] [folder2lib4apu + '__pycache__//apu3image4xi.cpython-33.pyc'] = 'Yes'
    #       task4delete2program ['CODE4BINARY'] [folder2lib4apu + '__pycache__//dummy3image4xi.cpython-33.pyc'] = 'Yes'
    #                task4delete2program ['SIZE4BINARY'] = {}
    #       task4delete2program ['SIZE4BINARY'] [folder2lib4apu + '__pycache__//apu3image4xi.cpython-33.pyc'] = 'Yes'
    #       task4delete2program ['SIZE4BINARY'] [folder2lib4apu + '__pycache__//dummy3image4xi.cpython-33.pyc'] = 'Yes'
    #       
    #       response4delete2programwithbinary = programobject2apu.delete2programwithbinary(task4delete2program)
    #       print('(Step 134B/700) OF APU2PROGRAM4XI5TEST - DELETED BINARY INSIDE PROGRAM OBJECT WITH METHOD ', time4xi())                                                                  
    #   except:
    #       print('(Step 134Z1/700) OF APU2PROGRAM4XI5TEST *** FAILED TO DELETE BINARY INSIDE PROGRAM OBJECT WITH METHOD *** ', time4xi())
    #   try:                                                                                          
    #       keys2response = response4delete2programwithbinary.keys()
    #       print('COUNT2REFERENCE:', response4delete2programwithbinary ['COUNT2REFERENCE'])   
    #       print('COUNT4BINARY:', response4delete2programwithbinary ['COUNT4BINARY'])
    #       print('DIRECTORY4BINARY:', response4delete2programwithbinary ['DIRECTORY4BINARY']) 
    #       print('FUNCTION2CALLBACK:', response4delete2programwithbinary ['FUNCTION2CALLBACK'])   
    #       print('LANGUAGE4BINARY:', response4delete2programwithbinary ['LANGUAGE4BINARY'])
    #       print('SIZE4BINARY:', response4delete2programwithbinary ['SIZE4BINARY'])                
    #       # 
    #       keys2source = response4delete2programwithbinary['CODE4BINARY'].keys()
    #       for key2source in keys2source:
    #           print('BINARY: ', key2source)
    #           print(response4delete2programwithbinary ['CODE4BINARY'] [key2source] [1])
    #       # COUNT2REFERENCE: 1
    #       # COUNT4BINARY: 1
    #       # DIRECTORY4BINARY: ...//__pycache__//
    #       # FUNCTION2CALLBACK: <function trace2program4apu at 0x000000000412BBC8>
    #       # LANGUAGE4BINARY: Python
    #       # SIZE4BINARY: {'...//__pycache__//apu2image4xi.cpython-33.pyc': 1006}
    #       # BINARY:  ...__pycache__//apu2image4xi.cpython-33.pyc
    #       # b'l\x0c\r\n'
    #                       
    #   except:
    #       print('(Step 134Z2/700) OF APU2PROGRAM4XI5TEST *** FAILED TO PRINT response4delete2programwithbinary : ', response4delete2programwithbinary)         
    #                  
    #">
    #<state4xi status = "ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------   
    def delete2programwithbinary(self, task4delete2programwithbinary = None):
        '''
        sets program APU objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "APU2Program4Xi"
        method2print = "delete2programwithbinary"
                     
        message2print = "{(Step 1A/33) started" + "; delete2programwithbinary-of-APU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4delete2programwithbinary-passed-to-delete2programwithbinary-of-APU2Program4Xi: " + str(task4delete2programwithbinary) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4delete2programwithbinary-passing-to-delete2programwithbinary4apu-by-delete2programwithbinary-of-APU2Program4Xi: " + str(task4delete2programwithbinary) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
        
        response4delete2programwithbinary = delete2programwithbinary4apu(task4delete2programwithbinary)

        message2print = "{(Step 2B/33) performed" + "; delete2programwithbinary4apu-performed-by-delete2programwithbinary-of-APU2Program4Xi: " + str(delete2programwithbinary4apu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
        
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4delete2programwithbinary-returning-by-delete2programwithbinary-of-APU2Program4Xi: " + str(response4delete2programwithbinary) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
        
                
        return response4delete2programwithbinary
                                
                            
    #</methodize4xi>    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "delete2programwithsource-of-APU2Program4Xi" usage = "
    #  
    #   print('(Step 124A/700) OF APU2PROGRAM4XI5TEST - TRYING TO DELETE SOURCE IN PROGRAM OBJECT WITH CLASS ', time4xi())        
    #   try:
    #       task4delete2program = {}
    #       task4delete2program ['CODE4SOURCE'] = {}
    #       task4delete2program ['CODE4SOURCE'] [folder2lib4apu + 'apu2buffer4xi.py'] = 'Yes'
    #       task4delete2program ['CODE4SOURCE'] [folder2lib4apu + 'apu2dummy4xi.py'] = 'Yes'
    #                task4delete2program ['SIZE4SOURCE'] = {}
    #       task4delete2program ['SIZE4SOURCE'] [folder2lib4apu + 'apu2buffer4xi.py'] = 'Yes'
    #       task4delete2program ['SIZE4SOURCE'] [folder2lib4apu + 'apu2dummy4xi.py'] = 'Yes'
    #       
    #       response4delete2programwithsource = programobject2apu.delete2programwithsource(task4delete2program)
    #       print('(Step 124B/700) OF APU2PROGRAM4XI5TEST - DELETED SOURCE IN PROGRAM OBJECT WITH CLASS ', time4xi())
    #   except:
    #       print('(Step 124Z1/700) OF APU2PROGRAM4XI5TEST *** FAILED TO DELETE SOURCE IN PROGRAM OBJECT WITH CLASS *** ', time4xi())    
    #   try:                                                                                      
    #       print('COUNT4SOURCE: ', response4delete2programwithsource['COUNT4SOURCE'])
    #       print('DIRECTORY4SOURCE: ', response4delete2programwithsource['DIRECTORY4SOURCE'])
    #       print('LANGUAGE4SOURCE: ', response4delete2programwithsource['LANGUAGE4SOURCE'])
    #       print('SIZE4SOURCE: ', response4delete2programwithsource['SIZE4SOURCE'])
    #       keys2source = response4delete2programwithsource['CODE4SOURCE'].keys()
    #       for key2source in keys2source:
    #           print('SOURCE: ', key2source)
    #           print(response4delete2programwithsource ['CODE4SOURCE'] [key2source] [1])
    #       # COUNT4SOURCE:  1
    #       # DIRECTORY4SOURCE:  OK 
    #       # LANGUAGE4SOURCE:  Python
    #       # SIZE4SOURCE:  {'... apu2sampler4xi5test.py': 1337}
    #       # SOURCE:  ... apu2sampler4xi5test.py
    #       # ... profile = "11" />                        
    #   except:
    #       print('(Step 124Z2/700) OF APU2PROGRAM4XI5TEST *** FAILED TO PRINT response4delete2programwithsource *** ', response4delete2programwithsource)                            
    #                    
    #">
    #<state4xi status = "ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------   
    def delete2programwithsource(self, task4delete2programwithsource = None):
        '''
        sets program APU objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "APU2Program4Xi"
        method2print = "delete2programwithsource"
                     
        message2print = "{(Step 1A/33) started" + "; delete2programwithsource-of-APU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:44:01 11/22/11 Pacific Standard Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4delete2programwithsource-passed-to-delete2programwithsource-of-APU2Program4Xi: " + str(task4delete2programwithsource) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # {'CODE4SOURCE': {folder2lib4apu + 'apu2buffer4xi.py': 'Yes', 
        #                  folder2lib4apu + 'apu2dummy4xi.py': 'Yes'}, 
        #  'SIZE4SOURCE': {folder2lib4apu + 'apu2buffer4xi.py': 'Yes', 
        #  folder2lib4apu + 'apu2dummy4xi.py': 'Yes'}}

          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4delete2programwithsource-passing-to-delete2programwithsource4apu-by-delete2programwithsource-of-APU2Program4Xi: " + str(task4delete2programwithsource) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # {'CODE4SOURCE': {folder2lib4apu + 'apu2buffer4xi.py': 'Yes', 
        #                  folder2lib4apu + 'apu2dummy4xi.py': 'Yes'}, 
        #  'SIZE4SOURCE': {folder2lib4apu + 'apu2buffer4xi.py': 'Yes', 
        #  folder2lib4apu + 'apu2dummy4xi.py': 'Yes'}}
        
        response4delete2programwithsource = delete2programwithsource4apu(task4delete2programwithsource)

        message2print = "{(Step 2B/33) performed" + "; delete2programwithsource4apu-performed-by-delete2programwithsource-of-APU2Program4Xi: " + str(delete2programwithsource4apu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function delete2programwithsource4apu at 0x00000000040DDE48>
        
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4delete2programwithsource-returning-by-delete2programwithsource-of-APU2Program4Xi: " + str(response4delete2programwithsource) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 'LANGUAGE4SOURCE': 'Python' ...
        
                
        return response4delete2programwithsource
                                
                            
    #</methodize4xi>    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set2programwithsource-of-APU2Program4Xi" usage = "
    # 
    #   ... prepare task4set2program ...
    #   try: 
    #       response4set2program = programobject2apu.set2programwithsource(task4set2program)
    #       print('(Step 158B/700) OF APU2PROGRAM4XI5TEST - SET PROGRAM SOURCE FOR APU: ', time4xi())    
    #   except:
    #       print('(Step 158Z1/700) OF APU2PROGRAM4XI5TEST *** FAILED TO SET PROGRAM SOURCE FOR APU: ', time4xi())
    #   try:                                                                                          
    #       keys2response = response4set2program.keys()
    #       print('COUNT2REFERENCE:', response4set2program ['COUNT2REFERENCE'])   
    #       print('COUNT4SOURCE:', response4set2program ['COUNT4SOURCE'])
    #       print('DIRECTORY4SOURCE:', response4set2program ['DIRECTORY4SOURCE']) 
    #       print('FUNCTION2CALLBACK:', response4set2program ['FUNCTION2CALLBACK'])   
    #       print('LANGUAGE4SOURCE:', response4set2program ['LANGUAGE4SOURCE'])
    #       print('SIZE4SOURCE:', response4set2program ['SIZE4SOURCE'])                
    #            keys2source = response4set2program['CODE4SOURCE'].keys()
    #       for key2source in keys2source:
    #           print('SOURCE: ', key2source)
    #           print(response4set2program ['CODE4SOURCE'] [key2source] [1])
    #       # COUNT2REFERENCE: 1
    #       # COUNT4SOURCE: 1
    #       # DIRECTORY4SOURCE: OK
    #       # FUNCTION2CALLBACK: <function trace2program4apu at 0x0000000003AC5BC8>
    #       # LANGUAGE4SOURCE: Python
    #       # SIZE4SOURCE: 1
    #       # SOURCE:  ... //apu2image4xi5test.py
    #       # ... profile = "11" />        
    #   except:
    #       print('(Step 158Z2/700) OF APU2PROGRAM4XI5TEST *** FAILED TO PRINT response4set2program : ', response4set2program)  
    #       
    #">
    #<state4xi status = "ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------   
    def set2programwithsource(self, task4set2programwithsource = None):
        '''
        sets program APU objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "APU2Program4Xi"
        method2print = "set2programwithsource"
                     
        message2print = "{(Step 1A/33) started" + "; set2programwithsource-of-APU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4set2programwithsource-passed-to-set2programwithsource-of-APU2Program4Xi: " + str(task4set2programwithsource) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
        
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set2programwithsource-passing-to-set2programwithsource4apu-by-set2programwithsource-of-APU2Program4Xi: " + str(task4set2programwithsource) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
        
        apu2program2xi = set2programwithsource4apu(task4set2programwithsource)

        message2print = "{(Step 2B/33) performed" + "; set2programwithsource4apu-performed-by-set2programwithsource-of-APU2Program4Xi: " + str(set2programwithsource4apu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
        
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; apu2program2xi-returning-by-set2programwithsource-of-APU2Program4Xi: " + str(apu2program2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
        
                
        return apu2program2xi
                                
                            
    #</methodize4xi>
    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set2programwithbinary-of-APU2Program4Xi" usage = "
    # 
    #   ... prepare task4set2programwithbinary ...
    #   task4set2programwithbinary ['CODE4BINARY'] = None
    #   print('(Step 167A/700) OF APU2PROGRAM4XI5TEST - TRYING TO SET BINARY OF TASK FOR PROGRAM4APU  ', time4xi())      
    #   try: 
    #       binary4apu = {}
    #       key2binary4apu = folder2lib4apu + '__pycache__//apu3image4xi.cpython-33.pyc'
    #       code2binary4apu = {}
    #       code2binary4apu [1] = b'l\x0c\r\n'
    #       code2binary4apu [2] = b'\x00Z\n'
    #       
    #       binary4apu [key2binary4apu] = code2binary4apu
    #                                 
    #       task4set2programwithbinary ['CODE4BINARY'] = binary4apu                                                                                                              
    #       print('(Step 167B/700) OF APU2PROGRAM4XI5TEST - SET BINARY OF TASK FOR PROGRAM4APU TO: ', task4set2programwithbinary ['CODE4BINARY'])                           
    #   except:
    #       print('(Step 167Z1/700) OF APU2PROGRAM4XI5TEST *** FAILED TO SET BINARY OF TASK FOR PROGRAM4APU  ', time4xi()) 
    #       
    #   print('(Step 167CB/700) OF APU2PROGRAM4XI5TEST - TRYING TO SET BINARY PROGRAM FOR APU: ', time4xi())      
    #   try: 
    #       response4set2program = programobject2apu.set2programwithbinary(task4set2programwithbinary)
    #       print('(Step 167D/700) OF APU2PROGRAM4XI5TEST - SET BINARY PROGRAM FOR APU: ', time4xi())
    #   except:
    #       print('(Step 167Z1/700) OF APU2PROGRAM4XI5TEST *** FAILED TO PRINT RESPONSE4SET2PROGRAM: ', response4set2program)                     
    #   try:                                                                                          
    #       keys2response = response4set2program.keys()
    #       print('COUNT2REFERENCE:', response4set2program ['COUNT2REFERENCE']) 
    #   except:
    #       pass
    #   try:        
    #       print('COUNT4BINARY:', response4set2program ['COUNT4BINARY'])
    #   except:
    #       pass
    #   try:        
    #       print('DIRECTORY4BINARY:', response4set2program ['DIRECTORY4BINARY']) 
    #   except:
    #       pass
    #   try:        
    #       print('FUNCTION2CALLBACK:', response4set2program ['FUNCTION2CALLBACK'])   
    #   except:
    #       pass
    #   try:        
    #       print('LANGUAGE4BINARY:', response4set2program ['LANGUAGE4BINARY'])
    #   except:
    #       pass
    #   try:        
    #       print('SIZE4BINARY:', response4set2program ['SIZE4BINARY'])                
    #   except:
    #       pass
    #   try:        
    #            keys2binary = response4set2program['CODE4BINARY'].keys()
    #       for key2binary in keys2binary:
    #           print('BINARY: ', key2binary)
    #           print(response4set2program ['CODE4BINARY'] [key2binary] [1])
    #       #    
    #   except:
    #       pass
    #       # COUNT2REFERENCE: 1
    #       # DIRECTORY4BINARY: ... //__pycache__//
    #       # FUNCTION2CALLBACK: <function trace2program4apu at 0x0000000003B1FBC8>
    #       # LANGUAGE4BINARY: Python
    #       # SIZE4BINARY: 1
    #       # BINARY:  ... //__pycache__//apu3image4xi.cpython-33.pyc
    #       # b'l\x0c\r\n'           
    # 
    #">
    #<state4xi status = "ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------   
    def set2programwithbinary(self, task4set2programwithbinary = None):
        '''
        sets program APU objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "APU2Program4Xi"
        method2print = "set2programwithbinary"
                     
        message2print = "{(Step 1A/33) started" + "; set2programwithbinary-of-APU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 05:32:44 11/17/11 Pacific Standard Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4set2programwithbinary-passed-to-set2programwithbinary-of-APU2Program4Xi: " + str(task4set2programwithbinary) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # {'CODE2ERROR': 0, 
        #  'LANGUAGE4SOURCE': 'C99', 
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003C68EF0>, 
        #  'CONTEXT': ...

          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set2programwithbinary-passing-to-set2programwithbinary4apu-by-set2programwithbinary-of-APU2Program4Xi: " + str(task4set2programwithbinary) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
        
        response4set2programwithbinary = set2programwithbinary4apu(task4set2programwithbinary)

        message2print = "{(Step 2B/33) performed" + "; set2programwithbinary4apu-performed-by-set2programwithbinary-of-APU2Program4Xi: " + str(set2programwithbinary4apu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
        
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4set2programwithbinary-returning-by-set2programwithbinary-of-APU2Program4Xi: " + str(response4set2programwithbinary) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
        
                
        return response4set2programwithbinary
                                
                            
    #</methodize4xi>    
    

    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get2programinfo-of-APU2Program4Xi" usage = "
    #
    #   print('(Step 106A1/700) OF APU2PROGRAM4XI5TEST - TRYING TO GET PROGRAM INFO ', time4xi())    
    #   try:                               
    #       response4get2programinfo = programobject2apu.get2programinfo()                                 
    #                                
    #       keys2response = response4get2programinfo.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4get2programinfo[key2response])
    #           # CODE4SOURCE  :  {folder2lib4apu + 'apu2sampler4xi5test.py': {1: '... profile = "11" />\n', 2: ....
    #           
    #   except:
    #       print('(Step 106Z1/700) OF APU2PROGRAM4XI5TEST *** FAILED TO GET PROGRAM INFO ', time4xi()) 
    #   
    # 
    # ">
    #<state4xi status = "ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------   
    def get2programinfo(self, task4getinfo2program=None):
        '''
        get2programinfo method gets objects of APU2Program4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "APU2Program4Xi"
        method2print = "get2programinfo"
                     
        message2print = "{(Step 1A/33) started" + "; get2programinfo-of-APU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 05:01:22 11/17/11 Pacific Standard Time

        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # ----------------------------------------------------------------------------------------- 
        apu2program2xi = get2programinfo4apu(task4getinfo2program)
        
        message2print = "{(Step 2A/33) returning" + "; apu2program2xi-returning-by-get2programinfo-of-APU2Program4Xi: " + str(apu2program2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 
        #  'COUNT2REFERENCE': 1, 
        #  'DIRECTORY4BINARY': folder2lib4apu + '__pycache__//', 
        #  'LANGUAGE4BINARY': 'Python', 
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2programwithbinary4apu-failed ; time: 05:16:42 11/17/11 Pacific Standard Time}', 
        #  'CLASS': 'create2programwithbinary4apu', 
        #  'METHOD': 'create2programwithbinary4apu'}, 
        #  'CODE4BINARY': ...

        return apu2program2xi
                            
    #</methodize4xi>

   
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get2buildinfo-of-APU2Program4Xi" usage = "
    #
    #   print('(Step 106A1/700) OF APU2BUILD4XI5TEST - TRYING TO GET BUILD INFO ', time4xi())    
    #   try:                               
    #                               
    #       response4get2buildinfo = programobject2apu.get2buildinfo()
    #       print('(Step 106A2/700) OF APU2BUILD4XI5TEST - GOT BUILD INFO ', time4xi())    
    #   except:
    #       print('(Step 106Z1/700) OF APU2BUILD4XI5TEST *** FAILED TO GET BUILD INFO ', time4xi()) 
    # 
    # ">
    #<state4xi status = "ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------   
    def get2buildinfo(self, task4getinfo2build=None):
        '''
        get2buildinfo method gets objects of APU2Program4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "APU2Program4Xi"
        method2print = "get2buildinfo"
                     
        message2print = "{(Step 1A/33) started" + "; get2buildinfo-of-APU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 

        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # ----------------------------------------------------------------------------------------- 
        apu2build2xi = get2buildinfo4apu(task4getinfo2build)
        
        message2print = "{(Step 2A/33) returning" + "; apu2build2xi-returning-by-get2buildinfo-of-APU2Program4Xi: " + str(apu2build2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
        
        
        return apu2build2xi
                            
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "release2program-of-APU2Program4Xi" usage = "
    #
    #   print('(Step 170A/700) OF APU2PROGRAM4XI5TEST - RELEASING PROGRAM', time4xi())  
    #   task4release2program  = {}
    #   task4release2program  ['COUNT2REFERENCE'] = 1
    #   try:
    #       response4release2program = programobject2apu.release2program(task4release2program)
    #       print('(Step 170B/700) OF APU2PROGRAM4XI5TEST - RELEASED PROGRAM', time4xi())
    #   except:
    #       print('(Step 170Z1/700) OF APU2PROGRAM4XI5TEST *** FAILED TO RELEASE PROGRAM', time4xi())                        
    #       keys2response = response4release2program.keys()
    #   try:
    #       print('(Step 170C/700) OF APU2PROGRAM4XI5TEST - COUNT2REFERENCE: ', response4release2program['COUNT2REFERENCE'] )
    #       # 0
    #   except:
    #       print('(Step 170Z2/700) OF APU2PROGRAM4XI5TEST *** FAILED TO PRINT RESPONSE4RELEASE2PROGRAM', response4release2program)  
    #  
    #">
    #<state4xi status = "ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------   
    def release2program(self, task4release2program = None):
        '''
        releases APU2Program4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "APU2Program4Xi"
        method2print = "release2program"
                     
        message2print = "{(Step 1A/33) started" + "; release2program-of-APU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4release2program-passed-to-release2program-of-APU2Program4Xi: " + str(task4release2program) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4release2program-passing-to-release2program4apu-by-release2program-of-APU2Program4Xi: " + str(task4release2program) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
  
        apu2program2xi = release2program4apu(task4release2program)

        message2print = "{(Step 2B/33) performed" + "; release2program4apu-performed-by-release2program-of-APU2Program4Xi: " + str(release2program4apu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; apu2program2xi-returning-by-release2program-of-APU2Program4Xi: " + str(apu2program2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #               
                
        return apu2program2xi
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "retain2program-of-APU2Program4Xi" usage = "
    #
    #   print('(Step 180A/700) OF APU2PROGRAM4XI5TEST - RETAINING PROGRAM ', time4xi())
    #   task4retain2program  = {}
    #   task4retain2program  ['COUNT2REFERENCE'] = 1
    #   try:
    #       response4retain2program = programobject2apu.retain2program(task4retain2program)
    #       print('(Step 180B/700) OF APU2PROGRAM4XI5TEST - RETAINED PROGRAM ', time4xi())
    #   except:
    #       print('(Step 180Z1/700) OF APU2PROGRAM4XI5TEST *** FAILED TO RETAIN PROGRAM ', time4xi())                    
    #   try:
    #       print('(Step 180C/700) OF APU2PROGRAM4XI5TEST COUNT2REFERENCE: ', response4retain2program ['COUNT2REFERENCE'])
    #       # 2
    #   except:
    #       print('(Step 180Z2/700) OF APU2PROGRAM4XI5TEST *** FAILED TO PRINT RESPONSE4RETAIN2PROGRAM ', response4retain2program)
    # 
    #">
    #<state4xi status = "ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------   
    def retain2program(self, task4retain2program = None):
        '''
        retains APU2Program4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "APU2Program4Xi"
        method2print = "retain2program"
                     
        message2print = "{(Step 1A/33) started" + "; retain2program-of-APU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
            
        message2print = "{(Step 1B/33) passed" + "; task4retain-passed-to-retain2program-of-APU2Program4Xi: " + str(task4retain2program) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
        
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4retain-passing-to-retain2program4apu-by-retain-of-APU2Program4Xi: " + str(task4retain2program) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
        
  
        apu2program2xi = retain2program4apu(task4retain2program)

        message2print = "{(Step 2B/33) performed" + "; retain2program4apu-performed-by-retain2program-of-APU2Program4Xi: " + str(retain2program4apu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; apu2program2xi-returning-by-retain2program-of-APU2Program4Xi: " + str(apu2program2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
            
                
        return apu2program2xi
                                                           
    #</methodize4xi>   
    
 
                    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "trace2program-of-APU2Program4Xi" usage = "
    #     
    #   print('(Step 210A/25) OF APU2PROGRAM4XI5TEST - TRYING TO TRACE APU: ', time4xi())      
    #   trace2apu = None
    #   try:
    #       trace2apu = programobject2apu.trace2program() 
    #               print('(Step 210B/25) OF APU2PROGRAM4XI5TEST - TRACE2APU: ', trace2apu)  
    #       # {'MESSAGE': '{(Step 2Z0/33) failed; trace2program4apu-failed {}; time: 04:40:24 11/16/11 Pacific Standard Time}', 'CLASS': 'trace2program4apu', 'METHOD': 'trace2program4apu'}
    #        except:
    #           print('(Step 210Z1/25) OF APU2PROGRAM4XI5TEST *** FAILED TO TRACE APU: ', trace2apu) 
    #">
    #<state4xi status = "ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------   
    def trace2program(self, task4trace2program = None):
        '''
        traces APU2Program4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "APU2Program4Xi"
        method2print = "trace2program"
                     
        message2print = "{(Step 1A/33) started" + "; trace2program-of-APU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
         
            
        message2print = "{(Step 1B/33) passed" + "; task4trace2program-passed-to-trace-of-APU2Program4Xi: " + str(task4trace2program) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
         
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4trace2program-passing-to-trace2program4apu-by-trace2program-of-APU2Program4Xi: " + str(task4trace2program) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
         
  
        trace2xi = trace2program4apu(task4trace2program)

        message2print = "{(Step 2B/33) performed" + "; trace2program4apu-performed-by-trace2program-of-APU2Program4Xi: " + str(trace2program4apu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
        
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; trace2xi-returning-by-trace2program-of-APU2Program4Xi: " + str(trace2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #       
                
        return trace2xi
                                                           
    #</methodize4xi>       

                    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "unload2compiler-of-APU2Program4Xi" usage = "
    #        
    #   print('(Step 220A/25) OF APU2PROGRAM4XI5TEST - TRYING TO UNLOAD COMPILER OF APU: ', time4xi())      
    #   try:
    #       response4unload2compiler = programobject2apu.unload2compiler() 
    #       print('(Step 220B/25) OF APU2PROGRAM4XI5TEST - UNLOADED COMPILER OF APU: ', response4unload2compiler)  
    #       # 
    #   except:
    #       print('(Step 220Z1/25) OF APU2PROGRAM4XI5TEST *** FAILED TO TO UNLOAD COMPILER OF APU: ', time4xi()) 
    #    
    #">
    #<state4xi status = "ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------   
    def unload2compiler(self, task4unload2compiler = None):
        '''
        unloads compiler
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "APU2Program4Xi"
        method2print = "unload2compiler"
                     
        message2print = "{(Step 1A/33) started" + "; unload2compiler-of-APU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
         
            
        message2print = "{(Step 1B/33) passed" + "; task4unload2compiler-passed-to-trace-of-APU2Program4Xi: " + str(task4unload2compiler) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
         
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4unload2compiler-passing-to-unload2compiler4apu-by-unload2compiler-of-APU2Program4Xi: " + str(task4unload2compiler) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
         
  
        trace2xi = unload2compiler4apu(task4unload2compiler)

        message2print = "{(Step 2B/33) performed" + "; unload2compiler4apu-performed-by-unload2compiler-of-APU2Program4Xi: " + str(unload2compiler4apu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
        
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; trace2xi-returning-by-unload2compiler-of-APU2Program4Xi: " + str(trace2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #       
                
        return trace2xi
                                                           
    #</methodize4xi>       

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set2programcallback-of-APU2Program4Xi" usage = "
    #
    #   print('(Step 200A/700) OF APU2PROGRAM4XI5TEST - TRYING TO SET PROGRAM CALLBACK WITH METHOD OF OBJECT XI2APU: ', programobject2apu) 
    #   try:
    #       callback2apu = programobject2apu.set2programcallback()                                  
    #       print('(Step 200B/700) OF APU2PROGRAM4XI5TEST - SET PROGRAM WITH CALLBACK TO: ', callback2apu)
    #       # <function trace2program4apu at 0x00000000038E02C8>
    #   except:
    #       print('(Step 200Z/700) OF APU2PROGRAM4XI5TEST *** FAILED TO SET PROGRAM CALLBACK WITH METHOD ***', time4xi())
    #
    #">
    #<state4xi status = "ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------   
    def set2programcallback(self, task4set2programcallback = None): 
        '''
        set2programcallback of APU2Program4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "APU2Program4Xi"
        method2print = "set2programcallback"
                     
        message2print = "{(Step 1A/33) started" + "; set2programcallback-of-APU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
            
        message2print = "{(Step 1B/33) passed" + "; task4set2programcallback-passed-to-set2programcallback-of-APU2Program4Xi: " + str(task4set2programcallback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
           
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set2programcallback-passing-to-set2programcallback-by-set2programcallback-of-APU2Program4Xi: " + str(task4set2programcallback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
        
        
        callback2apu4program = {} 
        try:
            callback2apu4program = set2programcallback4apu (task4set2programcallback)

            
        except:
            message2print = "{(Step 4A/33) failed" + "; callback2apu4program-failed-by-set2programcallback-of-APU2Program4Xi: " + str(callback2apu4program) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                                            
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; callback2apu4program-returning-by-set2programcallback-of-APU2Program4Xi: " + str(callback2apu4program) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # <function trace2program4apu at 0x00000000038CF5C8> 
        
                                             
        return callback2apu4program
                                                           
    #</methodize4xi>     
    
                                                              
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-APU2Program4Xi" usage = "
    #
    #   print('(Step 310A/700) OF APU2PROGRAM4XI5TEST - SENSING ', time4xi())  
    #   try:
    #       print(programobject2apu.sense())
    #       print('(Step 310B/700) OF APU2PROGRAM4XI5TEST - SENSED ', time4xi()) 
    #       # {'object': <apu2program4xi.APU2Program4Xi object at 0x0000000003E00AC8>, 'classvariable2apu-of-APU2Program4Xi': 4}
    #   except:
    #       print('(Step 310Z/700) OF APU2PROGRAM4XI5TEST *** FAILED TO SENSE ', time4xi())     
    #
    # ">
    #<state4xi status = "ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses APU2Program4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'APU2Program4Xi'
        method2print = 'sense'
        
                     
        message2print = "{(Step 1A/33) started" + "; sense-of-APU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        
        message2print = "{(Step 2A/33) sensing" + "; APU2Program4Xi-sensing-by-sense-of-APU2Program4Xi: " + str(APU2Program4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # 
            
        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        
        kb2xi = sense_object4xi(self)
        
        message2print = "{(Step 3A/33) got" + "; kb2xi-got-by-sense-of-APU2Program4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
         
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb2xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------         
        try:
            kb2xi['classvariable2apu-of-APU2Program4Xi'] = APU2Program4Xi.classvariable2apu
        except:
            message2print = "{(Step 4A/33) yielded" + "; kb2xi-yielded-by-sense-of-APU2Program4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
      
        message2print = "{(Step 5A/33) sensed" + "; APU2Program4Xi-sensed-by-sense-of-APU2Program4Xi: " + str(APU2Program4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        # 
        
        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        
        message2print = "{(Step 6A/33) returning" + "; kb2xi-returning-by-sense-of-APU2Program4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 
                   
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-APU2Program4Xi" usage = "
    # 
    #   print('(Step 320A/700) OF APU2PROGRAM4XI5TEST - REASONING ', time4xi())
    #   try:
    #       print(programobject2apu.reason())
    #       print('(Step 320B/700) OF APU2PROGRAM4XI5TEST - REASONED ', time4xi())
    #       # APU2Program4Xi-reasoned-by-reason-of-APU2Program4Xi: <class 'apu2program4xi.APU2Program4Xi'>; time: 04:40:25 11/16/11 Pacific Standard Time}
    #   except:
    #       print('(Step 320Z/700) OF APU2PROGRAM4XI5TEST *** FAILED TO REASON ', time4xi()) 
    #
    #">
    #<state4xi status = "ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons APU2Program4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'APU2Program4Xi'
        method2print = 'reason'

                     
        message2print = "{(Step 1A/33) started" + "; reason-of-APU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) reasoning" + "; APU2Program4Xi-reasoning-by-reason-of-APU2Program4Xi: " + str(APU2Program4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        
        kb4xi = reason_object4xi(self)
        
        
        message2print = "{(Step 3A/33) got" + "; kb4xi-got-by-reason-of-APU2Program4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
             
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        try:
            kb4xi['time-reason-of-APU2Program4Xi'] = time4xi() #adding time
        except:
            message2print = "{(Step 4Z/33) yielded" + "; kb4xi-yielded-by-reason-of-APU2Program4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
          
        
        message2print = "{(Step 5A/33) reasoned" + "; APU2Program4Xi-reasoned-by-reason-of-APU2Program4Xi: " + str(APU2Program4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        # 
        
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        
        message2print = "{(Step 6A/33) returning" + "; kb4xi-returning-by-reason-of-APU2Program4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        #           
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-APU2Program4Xi" usage = "
    #
    #   print('(Step 330A/700) OF APU2PROGRAM4XI5TEST - ACING ', time4xi())   
    #   try:
    #       print(programobject2apu.ace())
    #       print('(Step 330B/700) OF APU2PROGRAM4XI5TEST - ACED ', time4xi())                                                 
    #       # {'object-reason-object4xi': <apu2program4xi.APU2Program4Xi object at 0x0000000003E00AC8>, 'time-reason-of-APU2Program4Xi': '04:40:25 11/16/11 Pacific Standard Time'}
    #   except:
    #       print('(Step 330Z/700) OF APU2PROGRAM4XI5TEST *** FAILED TO ACE ', time4xi())                                 
    #    
    #">
    #<state4xi status = "ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces APU2Program4Xi class objects 
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------       
        
        
        class2print = 'APU2Program4Xi'
        method2print = 'ace'
        
                     
        message2print = "{(Step 1A/33) started" + "; ace-of-APU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) acing" + "; APU2Program4Xi-acing-by-ace-of-APU2Program4Xi: " + str(APU2Program4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        #      
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{(Step 3A/33) got" + "; kb8xi-got-by-ace-of-APU2Program4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        try:
             kb8xi['cores-of-APU2Program4Xi'] = APU2Program4Xi.classvariable2apu
        except:
             message2print = "{(Step 4A/33) yielded" + "; kb8xi-yielded-by-ace-of-APU2Program4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
             print4xi(class2print, method2print, message2print) 
  
        message2print = "{(Step 5A/33) aced" + "; APU2Program4Xi-aced-by-ace-of-APU2Program4Xi: " + str(APU2Program4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'apu2sourceprogram4xi.APU2Program4Xi'> 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 6A/33) returning" + "; kb8xi-returning-by-ace-of-APU2Program4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
#</class4xi>

# =================================================================================================
#<class4xi class = "DirectoryWalker" base = "None" child = "None" synopsis = "a forward iterator that traverses a directory tree" usage = "
#   to review all tests"
#>
# =================================================================================================
class DirectoryWalker:

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init" usage = "initializes DirectoryWalker"> 
    # ---------------------------------------------------------------------------------------------
    def __init__(self, directory):
        self.stack = [directory]
        self.files = []
        self.index = 0
        
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "getitem" usage = "getting item"> 
    # ---------------------------------------------------------------------------------------------
    def __getitem__(self, index):
        while 1:
            # ------------------------------------------------------------------------------------- 
            #
            # -------------------------------------------------------------------------------------          
            try:
                # --------------------------------------------------------------------------------- 
                #
                # ---------------------------------------------------------------------------------             
                file = self.files[self.index]
                self.index = self.index + 1
            except IndexError:
                # --------------------------------------------------------------------------------- 
                # pop next directory from stack
                # ---------------------------------------------------------------------------------             
                self.directory = self.stack.pop()
                self.files = os.listdir(self.directory)
                self.index = 0
            else:
                # --------------------------------------------------------------------------------- 
                # got a filename
                # ---------------------------------------------------------------------------------             
                fullname = os.path.join(self.directory, file)
                if os.path.isdir(fullname) and not os.path.islink(fullname):
                    self.stack.append(fullname)
                return fullname
    #</methodize4xi>
#</class4xi>

#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   apu2sourceprogram4xi5test.txt
#   sense-apu2sourceprogram4xi5test.xml
#   reason-apu2sourceprogram4xi5test.xml
#   ace-apu2sourceprogram4xi5test.xml
#"
#>

#<acknowledge4xi acknowledgement = "33" tester = "Small & Smart"/>


# -------------------------------------------------------------------------------------------------
# to-be-done                              
# -------------------------------------------------------------------------------------------------    
#<do4xi do = "
# None
#"
#/>

#<note4xi note = "
# parse4xi.py - to parse this module
# "/>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of apu2program4xi.py                              
# ------------------------------------------------------------------------------------------------- 
#"
#/>
